@@ -18,5 +18,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifndef __LIBPKCS11_H
+#define __LIBPKCS11_H
 void *C_LoadModule(const char *name, CK_FUNCTION_LIST_PTR_PTR);
 CK_RV C_UnloadModule(void *module);
+#endif
