@@ -39,7 +39,7 @@
  *  @param  proto  the desired protocol
  *  @return length of the encoded APDU
  */
-static size_t sc_apdu_get_length(const sc_apdu_t *apdu, unsigned int proto)
+size_t sc_apdu_get_length(const sc_apdu_t *apdu, unsigned int proto)
 {
 	size_t ret = 4;
 
@@ -80,7 +80,7 @@ static size_t sc_apdu_get_length(const sc_apdu_t *apdu, unsigned int proto)
  *  @param  outlen  size of hte output buffer
  *  @return SC_SUCCESS on success and an error code otherwise
  */
-static int sc_apdu2bytes(sc_context_t *ctx, const sc_apdu_t *apdu,
+int sc_apdu2bytes(sc_context_t *ctx, const sc_apdu_t *apdu,
 	unsigned int proto, u8 *out, size_t outlen)
 {
 	u8     *p = out;
@@ -244,7 +244,7 @@ int sc_apdu_set_resp(sc_context_t *ctx, sc_apdu_t *apdu, const u8 *buf,
  *                        +-------------------------------+
  *                                                        |
  *                                                        v
- *                                               card->reader->ops->tranmit
+ *                                               card->reader->ops->transmit
  */
 
 /** basic consistency check of the sc_apdu_t object
@@ -291,10 +291,6 @@ sc_check_apdu(sc_card_t *card, const sc_apdu_t *apdu)
 		/* data is expected       */
 		if (apdu->resplen == 0 || apdu->resp == NULL)
 			goto error;
-		/* return buffer to small */
-		if ((apdu->le == 0 && apdu->resplen < SC_MAX_APDU_BUFFER_SIZE-2)
-				|| (apdu->resplen < apdu->le))
-			goto error;
 		break;
 	case SC_APDU_CASE_3_SHORT:
 		/* data is sent */
@@ -314,10 +310,6 @@ sc_check_apdu(sc_card_t *card, const sc_apdu_t *apdu)
 		/* data is expected       */
 		if (apdu->resplen == 0 || apdu->resp == NULL)
 			goto error;
-		/* return buffer to small */
-		if ((apdu->le == 0 && apdu->resplen < SC_MAX_APDU_BUFFER_SIZE-2)
-				|| (apdu->resplen < apdu->le))
-			goto error;
 		/* inconsistent datalen   */
 		if (apdu->datalen != apdu->lc)
 			goto error;
@@ -350,7 +342,7 @@ sc_detect_apdu_cse(const sc_card_t *card, sc_apdu_t *apdu)
 	if (apdu->cse == SC_APDU_CASE_2 || apdu->cse == SC_APDU_CASE_3 ||
 	    apdu->cse == SC_APDU_CASE_4) {
 		int btype = apdu->cse & SC_APDU_SHORT_MASK;
-		/* if either Lc or Le is bigger than the maximun for
+		/* if either Lc or Le is bigger than the maximum for
 		 * short APDUs and the card supports extended APDUs
 		 * use extended APDUs (unless Lc is greater than
 		 * 255 and command chaining is activated) */
@@ -372,11 +364,15 @@ sc_single_transmit(struct sc_card *card, struct sc_apdu *apdu)
 	if (card->reader->ops->transmit == NULL)
 		LOG_TEST_RET(card->ctx, SC_ERROR_NOT_SUPPORTED, "cannot transmit APDU");
 
-	sc_log(ctx, "CLA:%X, INS:%X, P1:%X, P2:%X, data(%i) %p",
-			apdu->cla, apdu->ins, apdu->p1, apdu->p2, apdu->datalen, apdu->data);
+	sc_log(ctx,
+	       "CLA:%X, INS:%X, P1:%X, P2:%X, data(%"SC_FORMAT_LEN_SIZE_T"u) %p",
+	       apdu->cla, apdu->ins, apdu->p1, apdu->p2, apdu->datalen,
+	       apdu->data);
 #ifdef ENABLE_SM
-	if (card->sm_ctx.sm_mode == SM_MODE_TRANSMIT)
-		return sc_sm_single_transmit(card, apdu);
+	if (card->sm_ctx.sm_mode == SM_MODE_TRANSMIT
+		   	&& (apdu->flags & SC_APDU_FLAGS_NO_SM) == 0) {
+		LOG_FUNC_RETURN(ctx, sc_sm_single_transmit(card, apdu));
+	}
 #endif
 
 	/* send APDU to the reader driver */
@@ -401,7 +397,7 @@ sc_set_le_and_transmit(struct sc_card *card, struct sc_apdu *apdu, size_t olen)
 		LOG_TEST_RET(ctx, SC_ERROR_WRONG_LENGTH, "wrong length: required length exceeds resplen");
 
 	/* don't try again if it doesn't work this time */
-	apdu->flags  |= SC_APDU_FLAGS_NO_GET_RESP;
+	apdu->flags  |= SC_APDU_FLAGS_NO_RETRY_WL;
 	/* set the new expected length */
 	apdu->resplen = olen;
 	apdu->le      = nlen;
@@ -439,7 +435,7 @@ sc_get_response(struct sc_card *card, struct sc_apdu *apdu, size_t olen)
 	if (!card->ops->get_response)
 		LOG_TEST_RET(ctx, SC_ERROR_NOT_SUPPORTED, "no GET RESPONSE command");
 
-	/* call GET RESPONSE until we have read all data requested or until the card retuns 0x9000,
+	/* call GET RESPONSE until we have read all data requested or until the card returns 0x9000,
 	 * whatever happens first. */
 
 	/* if there are already data in response append a new data to the end of the buffer */
@@ -464,7 +460,7 @@ sc_get_response(struct sc_card *card, struct sc_apdu *apdu, size_t olen)
 		if (rv < 0)   {
 #ifdef ENABLE_SM
 			if (resp_len)   {
-				sc_log(ctx, "SM response data %s", sc_dump_hex(resp, resp_len));
+				sc_log_hex(ctx, "SM response data", resp, resp_len);
 				sc_sm_update_apdu_response(card, resp, resp_len, rv, apdu);
 			}
 #endif
@@ -520,7 +516,7 @@ sc_transmit(sc_card_t *card, sc_apdu_t *apdu)
 	LOG_TEST_RET(ctx, r, "transmit APDU failed");
 
 	/* ok, the APDU was successfully transmitted. Now we have two special cases:
-	 * 1. the card returned 0x6Cxx: in this case APDU will be re-trasmitted with Le set to SW2
+	 * 1. the card returned 0x6Cxx: in this case APDU will be re-transmitted with Le set to SW2
 	 * (possible only if response buffer size is larger than new Le = SW2)
 	 */
 	if (apdu->sw1 == 0x6C && (apdu->flags & SC_APDU_FLAGS_NO_RETRY_WL) == 0)
@@ -623,9 +619,18 @@ int sc_transmit_apdu(sc_card_t *card, sc_apdu_t *apdu)
 			len -= plen;
 			buf += plen;
 		}
-	} else
+	} else {
 		/* transmit single APDU */
 		r = sc_transmit(card, apdu);
+	}
+
+	if (r == SC_ERROR_CARD_RESET || r == SC_ERROR_READER_REATTACHED) {
+		sc_invalidate_cache(card);
+		/* give card driver a chance to react on resets */
+		if (card->ops->card_reader_lock_obtained)
+			card->ops->card_reader_lock_obtained(card, 1);
+	}
+
 	/* all done => release lock */
 	if (sc_unlock(card) != SC_SUCCESS)
 		sc_log(card->ctx, "sc_unlock failed");
@@ -659,8 +664,9 @@ sc_bytes2apdu(sc_context_t *ctx, const u8 *buf, size_t len, sc_apdu_t *apdu)
 
 	if (!len) {
 		apdu->cse = SC_APDU_CASE_1;
-		sc_log(ctx, "CASE_1 APDU: %lu bytes:\tins=%02x p1=%02x p2=%02x lc=%04x le=%04x",
-			(unsigned long) len0, apdu->ins, apdu->p1, apdu->p2, apdu->lc, apdu->le);
+		sc_log(ctx,
+		       "CASE_1 APDU: %"SC_FORMAT_LEN_SIZE_T"u bytes:\tins=%02x p1=%02x p2=%02x lc=%04"SC_FORMAT_LEN_SIZE_T"x le=%04"SC_FORMAT_LEN_SIZE_T"x",
+		       len0, apdu->ins, apdu->p1, apdu->p2, apdu->lc, apdu->le);
 		return SC_SUCCESS;
 	}
 
@@ -681,7 +687,9 @@ sc_bytes2apdu(sc_context_t *ctx, const u8 *buf, size_t len, sc_apdu_t *apdu)
 			apdu->lc += *p++;
 			len -= 3;
 			if (len < apdu->lc) {
-				sc_log(ctx, "APDU too short (need %lu more bytes)", (unsigned long) apdu->lc - len);
+				sc_log(ctx,
+				       "APDU too short (need %"SC_FORMAT_LEN_SIZE_T"u more bytes)",
+				       apdu->lc - len);
 				return SC_ERROR_INVALID_DATA;
 			}
 			apdu->data = p;
@@ -719,7 +727,9 @@ sc_bytes2apdu(sc_context_t *ctx, const u8 *buf, size_t len, sc_apdu_t *apdu)
 			apdu->lc = *p++;
 			len--;
 			if (len < apdu->lc) {
-				sc_log(ctx, "APDU too short (need %lu more bytes)", (unsigned long) apdu->lc - len);
+				sc_log(ctx,
+				       "APDU too short (need %"SC_FORMAT_LEN_SIZE_T"u more bytes)",
+				       apdu->lc - len);
 				return SC_ERROR_INVALID_DATA;
 			}
 			apdu->data = p;
@@ -743,10 +753,12 @@ sc_bytes2apdu(sc_context_t *ctx, const u8 *buf, size_t len, sc_apdu_t *apdu)
 		return SC_ERROR_INVALID_DATA;
 	}
 
-	sc_log(ctx, "Case %d %s APDU, %lu bytes:\tins=%02x p1=%02x p2=%02x lc=%04x le=%04x",
-			apdu->cse & SC_APDU_SHORT_MASK,
-			(apdu->cse & SC_APDU_EXT) != 0 ? "extended" : "short",
-			(unsigned long) len0, apdu->ins, apdu->p1, apdu->p2, apdu->lc, apdu->le);
+	sc_log(ctx,
+	       "Case %d %s APDU, %"SC_FORMAT_LEN_SIZE_T"u bytes:\tins=%02x p1=%02x p2=%02x lc=%04"SC_FORMAT_LEN_SIZE_T"x le=%04"SC_FORMAT_LEN_SIZE_T"x",
+	       apdu->cse & SC_APDU_SHORT_MASK,
+	       (apdu->cse & SC_APDU_EXT) != 0 ? "extended" : "short",
+	       len0, apdu->ins, apdu->p1, apdu->p2, apdu->lc,
+	       apdu->le);
 
 	return SC_SUCCESS;
 }
