@@ -27,6 +27,7 @@
 
 #include <ctype.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include "internal.h"
 #include "asn1.h"
@@ -42,16 +43,9 @@ static struct sc_card_driver cardos_drv = {
 	NULL, 0, NULL
 };
 
-static struct sc_atr_table cardos_atrs[] = {
+static const struct sc_atr_table cardos_atrs[] = {
 	/* 4.0 */
 	{ "3b:e2:00:ff:c1:10:31:fe:55:c8:02:9c", NULL, NULL, SC_CARD_TYPE_CARDOS_GENERIC, 0, NULL },
-	/* Italian eID card, postecert */
-	{ "3b:e9:00:ff:c1:10:31:fe:55:00:64:05:00:c8:02:31:80:00:47", NULL, NULL, SC_CARD_TYPE_CARDOS_CIE_V1, 0, NULL },
-	/* Italian eID card, infocamere */
-	{ "3b:fb:98:00:ff:c1:10:31:fe:55:00:64:05:20:47:03:31:80:00:90:00:f3", NULL, NULL, SC_CARD_TYPE_CARDOS_GENERIC, 0, NULL },
-	/* Another Italian InfocamereCard */
-	{ "3b:fc:98:00:ff:c1:10:31:fe:55:c8:03:49:6e:66:6f:63:61:6d:65:72:65:28", NULL, NULL, SC_CARD_TYPE_CARDOS_GENERIC, 0, NULL },
-	{ "3b:f4:98:00:ff:c1:10:31:fe:55:4d:34:63:76:b4", NULL, NULL, SC_CARD_TYPE_CARDOS_GENERIC, 0, NULL},
 	/* cardos m4.2 and above */
 	{ "3b:f2:18:00:ff:c1:0a:31:fe:55:c8:06:8a", "ff:ff:0f:ff:00:ff:00:ff:ff:00:00:00:00", NULL, SC_CARD_TYPE_CARDOS_M4_2, 0, NULL },
 	/* CardOS 4.4 */
@@ -96,14 +90,14 @@ static int cardos_match_card(sc_card_t *card)
 			return 0;
 		/* get the os version using GET DATA and compare it with
 		 * version in the ATR */
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "checking cardos version ...");
+		sc_log(card->ctx,  "checking cardos version ...");
 		sc_format_apdu(card, &apdu, SC_APDU_CASE_2_SHORT, 0xca, 0x01, 0x82);
 		apdu.resp = rbuf;
 		apdu.resplen = sizeof(rbuf);
 		apdu.le = 256;
 		apdu.lc = 0;
 		rv = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, rv, "APDU transmit failed");
+		LOG_TEST_RET(card->ctx, rv, "APDU transmit failed");
 		if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
 			return 0;
 		if (apdu.resp[0] != atr[10] ||
@@ -111,19 +105,19 @@ static int cardos_match_card(sc_card_t *card)
 			/* version mismatch */
 			return 0;
 		if (atr[11] <= 0x04) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "found cardos m4.01");
+			sc_log(card->ctx,  "found cardos m4.01");
 			card->type = SC_CARD_TYPE_CARDOS_M4_01;
 		} else if (atr[11] == 0x08) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "found cardos v4.3b");
+			sc_log(card->ctx,  "found cardos v4.3b");
 			card->type = SC_CARD_TYPE_CARDOS_M4_3;
 		} else if (atr[11] == 0x09) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "found cardos v4.2b");
+			sc_log(card->ctx,  "found cardos v4.2b");
 			card->type = SC_CARD_TYPE_CARDOS_M4_2B;
 		} else if (atr[11] >= 0x0B) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "found cardos v4.2c or higher");
+			sc_log(card->ctx,  "found cardos v4.2c or higher");
 			card->type = SC_CARD_TYPE_CARDOS_M4_2C;
 		} else {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "found cardos m4.2");
+			sc_log(card->ctx,  "found cardos m4.2");
 		}
 	}
 	return 1;
@@ -143,7 +137,7 @@ static int cardos_have_2048bit_package(sc_card_t *card)
 	apdu.lc = 0;
 	apdu.le = 256;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 	if ((len = apdu.resplen) == 0)
 		/* looks like no package has been installed  */
@@ -167,30 +161,30 @@ static int cardos_have_2048bit_package(sc_card_t *card)
 
 static int cardos_init(sc_card_t *card)
 {
-	unsigned long	flags, rsa_2048 = 0;
+	unsigned long flags = 0, rsa_2048 = 0;
 	size_t data_field_length;
 	sc_apdu_t apdu;
 	u8 rbuf[2];
+	int r;
 
 	card->name = "Atos CardOS";
 	card->cla = 0x00;
 
 	/* Set up algorithm info. */
-	flags = SC_ALGORITHM_RSA_RAW
-		| SC_ALGORITHM_RSA_HASH_NONE
-		| SC_ALGORITHM_ONBOARD_KEY_GEN
-		;
-	if (card->type != SC_CARD_TYPE_CARDOS_V5_0)
-		flags |= SC_ALGORITHM_NEED_USAGE;
-
-	_sc_card_add_rsa_alg(card,  512, flags, 0);
-	_sc_card_add_rsa_alg(card,  768, flags, 0);
-	_sc_card_add_rsa_alg(card, 1024, flags, 0);
+	flags = 0;
+	if (card->type == SC_CARD_TYPE_CARDOS_V5_0) {
+		flags |= SC_ALGORITHM_RSA_PAD_PKCS1;
+	} else {
+		flags |= SC_ALGORITHM_RSA_RAW
+			| SC_ALGORITHM_RSA_HASH_NONE
+			| SC_ALGORITHM_NEED_USAGE
+			| SC_ALGORITHM_ONBOARD_KEY_GEN;
+	}
 
 	if (card->type == SC_CARD_TYPE_CARDOS_M4_2) {
-		int r = cardos_have_2048bit_package(card);
+		r = cardos_have_2048bit_package(card);
 		if (r < 0)
-			return r;
+			return SC_ERROR_INVALID_CARD;
 		if (r == 1)
 			rsa_2048 = 1;
 		card->caps |= SC_CARD_CAP_APDU_EXT;
@@ -208,14 +202,18 @@ static int cardos_init(sc_card_t *card)
 	apdu.le = sizeof rbuf;
 	apdu.resp = rbuf;
 	apdu.resplen = sizeof(rbuf);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL,
-			sc_transmit_apdu(card, &apdu),
-			"APDU transmit failed");
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL,
-			sc_check_sw(card, apdu.sw1, apdu.sw2),
-			"GET DATA command returned error");
+	r = sc_transmit_apdu(card, &apdu);
+	if (r < 0)
+		LOG_TEST_RET(card->ctx,
+				SC_ERROR_INVALID_CARD,
+				"APDU transmit failed");
+	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
+	if (r < 0)
+		LOG_TEST_RET(card->ctx,
+				SC_ERROR_INVALID_CARD,
+				"GET DATA command returned error");
 	if (apdu.resplen != 2)
-		return SC_ERROR_WRONG_LENGTH;
+		return SC_ERROR_INVALID_CARD;
 	data_field_length = ((rbuf[0] << 8) | rbuf[1]);
 
 	/* strip the length of possible Lc and Le bytes */
@@ -226,6 +224,9 @@ static int cardos_init(sc_card_t *card)
 	/* strip the length of SW bytes */
 	card->max_recv_size = data_field_length - 2;
 
+	_sc_card_add_rsa_alg(card,  512, flags, 0);
+	_sc_card_add_rsa_alg(card,  768, flags, 0);
+	_sc_card_add_rsa_alg(card, 1024, flags, 0);
 	if (rsa_2048 == 1) {
 		_sc_card_add_rsa_alg(card, 1280, flags, 0);
 		_sc_card_add_rsa_alg(card, 1536, flags, 0);
@@ -236,6 +237,8 @@ static int cardos_init(sc_card_t *card)
 	if (card->type == SC_CARD_TYPE_CARDOS_V5_0) {
 		/* Starting with CardOS 5, the card supports PIN query commands */
 		card->caps |= SC_CARD_CAP_ISO7816_PIN_INFO;
+		_sc_card_add_rsa_alg(card, 3072, flags, 0);
+		_sc_card_add_rsa_alg(card, 4096, flags, 0);
 	}
 
 	return 0;
@@ -310,13 +313,13 @@ static int cardos_check_sw(sc_card_t *card, unsigned int sw1, unsigned int sw2)
 	for (i = 0; i < err_count; i++) {
 		if (cardos_errors[i].SWs == ((sw1 << 8) | sw2)) {
 			if ( cardos_errors[i].errorstr ) 
-				sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "%s\n",
+				sc_log(card->ctx,  "%s\n",
 				 	cardos_errors[i].errorstr);
 			return cardos_errors[i].errorno;
 		}
 	}
 
-        sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Unknown SWs; SW1=%02X, SW2=%02X\n", sw1, sw2);
+        sc_log(card->ctx,  "Unknown SWs; SW1=%02X, SW2=%02X\n", sw1, sw2);
 	return SC_ERROR_CARD_CMD_FAILED;
 }
 
@@ -341,12 +344,12 @@ get_next_part:
 	apdu.resp = rbuf;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "DIRECTORY command returned error");
+	LOG_TEST_RET(card->ctx, r, "DIRECTORY command returned error");
 
 	if (apdu.resplen > 256) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "directory listing > 256 bytes, cutting");
+		sc_log(card->ctx,  "directory listing > 256 bytes, cutting");
 	}
 
 	len = apdu.resplen;
@@ -355,7 +358,7 @@ get_next_part:
 		/* is there a file information block (0x6f) ? */
 		p = sc_asn1_find_tag(card->ctx, p, len, 0x6f, &tlen);
 		if (p == NULL) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "directory tag missing");
+			sc_log(card->ctx,  "directory tag missing");
 			return SC_ERROR_INTERNAL;
 		}
 		if (tlen == 0)
@@ -363,7 +366,7 @@ get_next_part:
 			break;
 		q = sc_asn1_find_tag(card->ctx, p, tlen, 0x86, &ilen);
 		if (q == NULL || ilen != 2) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "error parsing file id TLV object");
+			sc_log(card->ctx,  "error parsing file id TLV object");
 			return SC_ERROR_INTERNAL;
 		}
 		/* put file id in buf */
@@ -387,7 +390,7 @@ get_next_part:
 
 	r = fids;
 
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
+	LOG_FUNC_RETURN(card->ctx, r);
 }
 
 static void add_acl_entry(sc_file_t *file, int op, u8 byte)
@@ -486,7 +489,7 @@ static int cardos_select_file(sc_card_t *card,
 	r = iso_ops->select_file(card, in_path, file);
 	if (r >= 0 && file)
 		parse_sec_attr((*file), (*file)->sec_attr, (*file)->sec_attr_len);
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
+	LOG_FUNC_RETURN(card->ctx, r);
 }
 
 static int cardos_acl_to_bytes(sc_card_t *card, const sc_file_t *file,
@@ -505,7 +508,7 @@ static int cardos_acl_to_bytes(sc_card_t *card, const sc_file_t *file,
 		else
 			byte = acl_to_byte(sc_file_get_acl_entry(file, idx[i]));
 		if (byte < 0) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Invalid ACL\n");
+			sc_log(card->ctx,  "Invalid ACL\n");
 			return SC_ERROR_INVALID_ARGUMENTS;
 		}
 		buf[i] = byte;
@@ -590,7 +593,7 @@ static int cardos_construct_fcp(sc_card_t *card, const sc_file_t *file,
 	size_t inlen = *outlen, len;
 	int    r;
 
-	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_NORMAL);
+	LOG_FUNC_CALLED(card->ctx);
 
 	if (out == NULL || inlen < 64)
 		return SC_ERROR_INVALID_ARGUMENTS;
@@ -638,7 +641,7 @@ static int cardos_construct_fcp(sc_card_t *card, const sc_file_t *file,
 			buf[4] |= (u8) file->record_count;
 			break;
 		default:
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "unknown EF type: %u", file->type);
+			sc_log(card->ctx,  "unknown EF type: %u", file->type);
 			return SC_ERROR_INVALID_ARGUMENTS;
 		}
 		if (file->ef_structure == SC_FILE_EF_CYCLIC ||
@@ -715,7 +718,7 @@ static int cardos_create_file(sc_card_t *card, sc_file_t *file)
 
 		r = cardos_construct_fcp(card, file, sbuf, &len);
 		if (r < 0) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "unable to create FCP");
+			sc_log(card->ctx,  "unable to create FCP");
 			return r;
 		}
 	
@@ -725,7 +728,7 @@ static int cardos_create_file(sc_card_t *card, sc_file_t *file)
 		apdu.data    = sbuf;
 
 		r = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+		LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 		return sc_check_sw(card, apdu.sw1, apdu.sw2);
 	} else
@@ -747,12 +750,12 @@ cardos_restore_security_env(sc_card_t *card, int se_num)
 	apdu.p1 = (card->type == SC_CARD_TYPE_CARDOS_CIE_V1 ? 0xF3 : 0x03);
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
+	LOG_FUNC_RETURN(card->ctx, r);
 }
 
 /*
@@ -771,7 +774,7 @@ cardos_set_security_env(sc_card_t *card,
 			    int se_num)
 {
 	sc_apdu_t apdu;
-	u8	data[3];
+	u8	data[6];
 	int	key_id, r;
 
 	assert(card != NULL && env != NULL);
@@ -800,17 +803,29 @@ cardos_set_security_env(sc_card_t *card,
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 
-	data[0] = 0x83;
-	data[1] = 0x01;
-	data[2] = key_id;
-	apdu.lc = apdu.datalen = 3;
+	if (card->type == SC_CARD_TYPE_CARDOS_V5_0) {
+		/* Private key reference */
+		data[0] = 0x84;
+		data[1] = 0x01;
+		data[2] = key_id;
+		/* Usage qualifier byte */
+		data[3] = 0x95;
+		data[4] = 0x01;
+		data[5] = 0x40;
+		apdu.lc = apdu.datalen = 6;
+	} else {
+		data[0] = 0x83;
+		data[1] = 0x01;
+		data[2] = key_id;
+		apdu.lc = apdu.datalen = 3;
+	}
 	apdu.data = data;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	do   {
 		const struct sc_supported_algo_info* algorithm_info = env->supported_algos;
@@ -860,7 +875,7 @@ do_compute_signature(sc_card_t *card, const u8 *data, size_t datalen,
 	apdu.lc      = datalen;
 	apdu.datalen = datalen;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 	if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
 		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, apdu.resplen);
@@ -873,23 +888,16 @@ cardos_compute_signature(sc_card_t *card, const u8 *data, size_t datalen,
 			 u8 *out, size_t outlen)
 {
 	int    r;
-	u8     buf[SC_MAX_APDU_BUFFER_SIZE];
-	size_t buf_len = sizeof(buf), tmp_len = buf_len;
 	sc_context_t *ctx;
 	int do_rsa_pure_sig = 0;
 	int do_rsa_sig = 0;
+	size_t i;
 
 
 	assert(card != NULL && data != NULL && out != NULL);
 	ctx = card->ctx;
 	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_VERBOSE);
 
-	if (datalen > SC_MAX_APDU_BUFFER_SIZE)
-		LOG_FUNC_RETURN(ctx, SC_ERROR_INVALID_ARGUMENTS);
-	if (outlen < datalen)
-		LOG_FUNC_RETURN(ctx, SC_ERROR_BUFFER_TOO_SMALL);
-	outlen = datalen;
-
 	/* There are two ways to create a signature, depending on the way,
 	 * the key was created: RSA_SIG and RSA_PURE_SIG.
 	 * We can use the following reasoning, to determine the correct operation:
@@ -905,23 +913,13 @@ cardos_compute_signature(sc_card_t *card, const u8 *data, size_t datalen,
 	 *   and www.crysys.hu/infsec/M40_Manual_E_2001_10.pdf)
 	 */
 
-	if (card->caps & SC_CARD_CAP_ONLY_RAW_HASH_STRIPPED){
-		sc_log(ctx, "Forcing RAW_HASH_STRIPPED");
-		do_rsa_sig = 1;
-	}
-	else if (card->caps & SC_CARD_CAP_ONLY_RAW_HASH){
-		sc_log(ctx, "Forcing RAW_HASH");
-		do_rsa_sig = 1;
-	}
-	else  {
-		/* check the the algorithmIDs from the AlgorithmInfo */
-		size_t i;
-		for(i=0; i<algorithm_ids_in_tokeninfo_count;++i){
-			unsigned int id = algorithm_ids_in_tokeninfo[i];
-			if(id == 0x86 || id == 0x88)
-				do_rsa_sig = 1;
-			else if(id == 0x8C || id == 0x8A)
-				do_rsa_pure_sig = 1;
+	/* check the the algorithmIDs from the AlgorithmInfo */
+	for (i = 0; i < algorithm_ids_in_tokeninfo_count; ++i) {
+		unsigned int id = algorithm_ids_in_tokeninfo[i];
+		if (id == 0x86 || id == 0x88) {
+			do_rsa_sig = 1;
+		} else if (id == 0x8C || id == 0x8A) {
+			do_rsa_pure_sig = 1;
 		}
 	}
 
@@ -942,36 +940,42 @@ cardos_compute_signature(sc_card_t *card, const u8 *data, size_t datalen,
 	}
 
 	if(do_rsa_sig == 1){
+		u8 *buf = malloc(datalen);
+		u8 *stripped_data = buf;
+		size_t stripped_datalen = datalen;
+		if (!buf)
+			LOG_FUNC_RETURN(ctx, SC_ERROR_OUT_OF_MEMORY);
+		memcpy(buf, data, datalen);
+		data = buf;
+
 		sc_log(ctx, "trying RSA_SIG (just the DigestInfo)");
+
 		/* remove padding: first try pkcs1 bt01 padding */
-		r = sc_pkcs1_strip_01_padding(ctx, data, datalen, buf, &tmp_len);
+		r = sc_pkcs1_strip_01_padding(ctx, data, datalen, stripped_data, &stripped_datalen);
 		if (r != SC_SUCCESS) {
-			const u8 *p = data;
 			/* no pkcs1 bt01 padding => let's try zero padding
 			 * This can only work if the data tbs doesn't have a
 			 * leading 0 byte.  */
-			tmp_len = buf_len;
-			while (*p == 0 && tmp_len != 0) {
-				++p;
-				--tmp_len;
+			while (*stripped_data == 0 && stripped_datalen != 0) {
+				++stripped_data;
+				--stripped_datalen;
 			}
-			memcpy(buf, p, tmp_len);
 		}
-		if (!(card->caps & (SC_CARD_CAP_ONLY_RAW_HASH_STRIPPED | SC_CARD_CAP_ONLY_RAW_HASH)) || card->caps & SC_CARD_CAP_ONLY_RAW_HASH ) {
-			sc_log(ctx, "trying to sign raw hash value with prefix");
-			r = do_compute_signature(card, buf, tmp_len, out, outlen);
-			if (r >= SC_SUCCESS)
-				LOG_FUNC_RETURN(ctx, r);
-		}
-		if (card->caps & SC_CARD_CAP_ONLY_RAW_HASH) {
-			sc_log(ctx, "Failed to sign raw hash value with prefix when forcing");
-			LOG_FUNC_RETURN(ctx, SC_ERROR_INVALID_ARGUMENTS);
+		sc_log(ctx, "trying to sign raw hash value with prefix");
+		r = do_compute_signature(card, stripped_data, stripped_datalen, out, outlen);
+		if (r >= SC_SUCCESS) {
+			free(buf);
+			LOG_FUNC_RETURN(ctx, r);
 		}
 		sc_log(ctx, "trying to sign stripped raw hash value (card is responsible for prefix)");
-		r = sc_pkcs1_strip_digest_info_prefix(NULL,buf,tmp_len,buf,&buf_len);
-		if (r != SC_SUCCESS)
+		r = sc_pkcs1_strip_digest_info_prefix(NULL, stripped_data, stripped_datalen, stripped_data, &stripped_datalen);
+		if (r != SC_SUCCESS) {
+			free(buf);
 			LOG_FUNC_RETURN(ctx, r);
-		return do_compute_signature(card, buf, buf_len, out, outlen);
+		}
+		r = do_compute_signature(card, stripped_data, stripped_datalen, out, outlen);
+		free(buf);
+		LOG_FUNC_RETURN(ctx, r);
 	}
 
 	LOG_FUNC_RETURN(ctx, SC_ERROR_INTERNAL);
@@ -1019,13 +1023,13 @@ cardos_lifecycle_get(sc_card_t *card, int *mode)
 	apdu.resp = rbuf;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	if (apdu.resplen < 1) {
-		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Lifecycle byte not in response");
+		LOG_TEST_RET(card->ctx, r, "Lifecycle byte not in response");
 	}
 
 	r = SC_SUCCESS;
@@ -1040,11 +1044,11 @@ cardos_lifecycle_get(sc_card_t *card, int *mode)
 		*mode = SC_CARDCTRL_LIFECYCLE_OTHER;
 		break;
 	default:
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Unknown lifecycle byte %d", rbuf[0]);
+		sc_log(card->ctx,  "Unknown lifecycle byte %d", rbuf[0]);
 		r = SC_ERROR_INTERNAL;
 	}
 
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
+	LOG_FUNC_RETURN(card->ctx, r);
 }
 
 static int
@@ -1075,12 +1079,12 @@ cardos_lifecycle_set(sc_card_t *card, int *mode)
 	apdu.resp = NULL;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
+	LOG_FUNC_RETURN(card->ctx, r);
 }
 
 static int
@@ -1103,12 +1107,12 @@ cardos_put_data_oci(sc_card_t *card,
 	apdu.datalen = args->len;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
+	LOG_FUNC_RETURN(card->ctx, r);
 }
 
 static int
@@ -1129,10 +1133,10 @@ cardos_put_data_seci(sc_card_t *card,
 	apdu.datalen = args->len;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	return r;
 }
@@ -1164,9 +1168,9 @@ cardos_generate_key(sc_card_t *card,
 	apdu.datalen = apdu.lc = sizeof(data);
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "GENERATE_KEY failed");
+	LOG_TEST_RET(card->ctx, r, "GENERATE_KEY failed");
 
 	return r;
 }
@@ -1182,17 +1186,22 @@ static int cardos_get_serialnr(sc_card_t *card, sc_serial_number_t *serial)
 	apdu.resplen = sizeof(rbuf);
 	apdu.le   = 256;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r,  "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r,  "APDU transmit failed");
 	if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
 		return SC_ERROR_INTERNAL;
-	if (apdu.resplen != 32) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "unexpected response to GET DATA serial"
+	if ((apdu.resplen == 8) && (card->type == SC_CARD_TYPE_CARDOS_V5_0)) {
+		/* cache serial number */
+		memcpy(card->serialnr.value, rbuf, 8);
+		card->serialnr.len = 8;
+	} else if (apdu.resplen == 32) {
+		/* cache serial number */
+		memcpy(card->serialnr.value, &rbuf[10], 6);
+		card->serialnr.len = 6;
+	} else {
+		sc_log(card->ctx,  "unexpected response to GET DATA serial"
 				" number\n");
 		return SC_ERROR_INTERNAL;
 	}
-	/* cache serial number */
-	memcpy(card->serialnr.value, &rbuf[10], 6);
-	card->serialnr.len = 6;
 	/* copy and return serial number */
 	memcpy(serial, &card->serialnr, sizeof(*serial));
 	return SC_SUCCESS;
@@ -1285,7 +1294,7 @@ cardos_logout(sc_card_t *card)
 		apdu.cla = 0x80;
 
 		r = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+		LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 		return sc_check_sw(card, apdu.sw1, apdu.sw2);
 	} else
