@@ -78,7 +78,7 @@ extern int dnie_read_file(
  * Override APDU response error codes from iso7816.c to allow 
  * handling of SM specific error
  */
-static struct sc_card_error dnie_errors[] = {
+static const struct sc_card_error dnie_errors[] = {
 	{0x6688, SC_ERROR_SM, "Cryptographic checksum invalid"},
 	{0x6987, SC_ERROR_SM, "Expected SM Data Object missing"},
 	{0x6988, SC_ERROR_SM, "SM Data Object incorrect"},
@@ -163,7 +163,7 @@ const char *user_consent_message="Esta a punto de realizar una firma digital\nco
  */
 char *user_consent_msgs[] = { "SETTITLE", "SETDESC", "CONFIRM", "BYE" };
 
-#ifdef linux
+#if !defined(__APPLE__) && !defined(_WIN32)
 /**
  * Do fgets() without interruptions.
  *
@@ -203,7 +203,7 @@ int dnie_ask_user_consent(struct sc_card * card, const char *title, const char *
 	CFStringRef header_ref; /* to store title */
 	CFStringRef message_ref; /* to store message */
 #endif
-#ifdef linux
+#if !defined(__APPLE__) && !defined(_WIN32)
 	pid_t pid;
 	FILE *fin=NULL;
 	FILE *fout=NULL;	/* to handle pipes as streams */
@@ -271,15 +271,7 @@ int dnie_ask_user_consent(struct sc_card * card, const char *title, const char *
 	if( result == kCFUserNotificationAlternateResponse )
 		LOG_FUNC_RETURN(card->ctx, SC_SUCCESS);
 	LOG_FUNC_RETURN(card->ctx, SC_ERROR_NOT_ALLOWED);
-#elif linux
-	/* check that user_consent_app exists. TODO: check if executable */
-	res = stat(GET_DNIE_UI_CTX(card).user_consent_app, &st_file);
-	if (res != 0) {
-		sc_log(card->ctx, "Invalid pinentry application: %s\n",
-		       GET_DNIE_UI_CTX(card).user_consent_app);
-		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_ARGUMENTS);
-	}
-
+#else
 	/* just a simple bidirectional pipe+fork+exec implementation */
 	/* In a pipe, xx[0] is for reading, xx[1] is for writing */
 	if (pipe(srv_send) < 0) {
@@ -304,10 +296,17 @@ int dnie_ask_user_consent(struct sc_card * card, const char *title, const char *
 		close(srv_send[1]);
 		close(srv_recv[0]);
 		close(srv_recv[1]);
-		/* call exec() with proper user_consent_app from configuration */
-		/* if ok should never return */
-		execlp(GET_DNIE_UI_CTX(card).user_consent_app, GET_DNIE_UI_CTX(card).user_consent_app, (char *)NULL);
-		sc_log(card->ctx, "execlp() error");
+		/* check that user_consent_app exists. TODO: check if executable */
+		res = stat(GET_DNIE_UI_CTX(card).user_consent_app, &st_file);
+		if (res != 0) {
+			sc_log(card->ctx, "Invalid pinentry application: %s\n",
+					GET_DNIE_UI_CTX(card).user_consent_app);
+		} else {
+			/* call exec() with proper user_consent_app from configuration */
+			/* if ok should never return */
+			execlp(GET_DNIE_UI_CTX(card).user_consent_app, GET_DNIE_UI_CTX(card).user_consent_app, (char *)NULL);
+			sc_log(card->ctx, "execlp() error");
+		}
 		abort();
 	default:		/* parent */
 		/* Close the pipe ends that the child uses to read from / write to
@@ -361,8 +360,6 @@ do_error:
 	/* close out channel to force client receive EOF and also die */
 	if (fout != NULL) fclose(fout);
 	if (fin != NULL) fclose(fin);
-#else
-#error "Don't know how to handle user consent in this (rare) Operating System"
 #endif
 	if (msg != NULL)
 		sc_log(card->ctx, "%s", msg);
@@ -760,15 +757,25 @@ static int dnie_sm_free_wrapped_apdu(struct sc_card *card,
 
 	if ((*sm_apdu) != plain) {
 		rv = cwa_decode_response(card, provider, *sm_apdu);
-		if (plain) {
-			plain->resplen = (*sm_apdu)->resplen;
+		if (plain && rv == SC_SUCCESS) {
+			if (plain->resp) {
+				/* copy the response into the original resp buffer */
+				if ((*sm_apdu)->resplen <= plain->resplen) {
+					memcpy(plain->resp, (*sm_apdu)->resp, (*sm_apdu)->resplen);
+					plain->resplen = (*sm_apdu)->resplen;
+				} else {
+					sc_log(card->ctx, "Invalid initial length,"
+							" needed %"SC_FORMAT_LEN_SIZE_T"u bytes"
+							" but has %"SC_FORMAT_LEN_SIZE_T"u",
+							(*sm_apdu)->resplen, plain->resplen);
+					rv = SC_ERROR_BUFFER_TOO_SMALL;
+				}
+			}
 			plain->sw1 = (*sm_apdu)->sw1;
 			plain->sw2 = (*sm_apdu)->sw2;
-			if (((*sm_apdu)->data) != plain->data)
-				free((unsigned char *) (*sm_apdu)->data);
-			if ((*sm_apdu)->resp != plain->resp)
-				free((*sm_apdu)->resp);
 		}
+		free((unsigned char *) (*sm_apdu)->data);
+		free((*sm_apdu)->resp);
 		free(*sm_apdu);
 	}
 	*sm_apdu = NULL;
@@ -849,8 +856,6 @@ static int dnie_init(struct sc_card *card)
 	card->sm_ctx.ops.free_sm_apdu = dnie_sm_free_wrapped_apdu;
 	card->sm_ctx.sm_mode = SM_MODE_NONE;
 
-	init_flags(card);
-
 	res=cwa_create_secure_channel(card,provider,CWA_SM_OFF);
 	LOG_TEST_RET(card->ctx, res, "Failure creating CWA secure channel.");
 
@@ -868,6 +873,8 @@ static int dnie_init(struct sc_card *card)
 	}
 #endif
 
+	init_flags(card);
+
 	GET_DNIE_PRIV_DATA(card)->cwa_provider = provider;
 
 	LOG_FUNC_RETURN(card->ctx, res);
@@ -999,7 +1006,7 @@ static int dnie_fill_cache(sc_card_t * card)
 	size_t count = 0;
 	size_t len = 0;
 	u8 *buffer = NULL;
-	u8 *pt = NULL;
+	u8 *pt = NULL, *p;
 	sc_context_t *ctx = NULL;
 
 	if (!card || !card->ctx)
@@ -1048,19 +1055,22 @@ static int dnie_fill_cache(sc_card_t * card)
 			}
 			if (r == SC_ERROR_INCORRECT_PARAMETERS)
 				goto read_done;
+			free(buffer);
 			if (apdu.resp != tmp)
 				free(apdu.resp);
 			LOG_FUNC_RETURN(ctx, r);	/* arriving here means response error */
 		}
 		/* copy received data into buffer. realloc() if not enough space */
 		count = apdu.resplen;
-		buffer = realloc(buffer, len + count);
-		if (!buffer) {
+		p = realloc(buffer, len + count);
+		if (!p) {
+			free(buffer);
 			free((void *)apdu.data);
 			if (apdu.resp != tmp)
 				free(apdu.resp);
 			LOG_FUNC_RETURN(ctx, SC_ERROR_OUT_OF_MEMORY);
 		}
+		buffer = p;
 		memcpy(buffer + len, apdu.resp, count);
 		if (apdu.resp != tmp) {
 			free(apdu.resp);
@@ -1340,51 +1350,27 @@ static int dnie_select_file(struct sc_card *card,
  * @param len requested challenge length
  * @return SC_SUCCESS if OK; else error code
  */
-#define BUFFER_SIZE 8
 
 static int dnie_get_challenge(struct sc_card *card, u8 * rnd, size_t len)
 {
-	sc_apdu_t apdu;
-	u8 buf[MAX_RESP_BUFFER_SIZE];
-	int result = SC_SUCCESS;
-	if ((card == NULL) || (card->ctx == NULL))
-		return SC_ERROR_INVALID_ARGUMENTS;
-	LOG_FUNC_CALLED(card->ctx);
-	/* just a copy of iso7816::get_challenge() but call dnie_check_sw to
-	 * look for extra error codes */
-	if ( (rnd==NULL) || (len==0) ) {
-		/* no valid buffer provided */
-		result = SC_ERROR_INVALID_ARGUMENTS;
-		goto dnie_get_challenge_error;
-	}
-	dnie_format_apdu(card, &apdu, SC_APDU_CASE_2_SHORT, 0x84, 0x00, 0x00, BUFFER_SIZE, 0,
-					buf, MAX_RESP_BUFFER_SIZE, NULL, 0);
+	/* As DNIe cannot handle other data length than 0x08 and 0x14 */
+	u8 rbuf[8];
+	size_t out_len;
+	int r;
 
-	/* 
-	* As DNIe cannot handle other data length than 0x08 and 0x14, 
-	* perform consecutive reads of 8 bytes until retrieve requested length
-	*/
-	while (len > 0) {
-		size_t n = len > BUFFER_SIZE ? BUFFER_SIZE : len;
-		sc_format_apdu(card, &apdu, SC_APDU_CASE_2_SHORT, 0x84, 0x00, 0x00);
-		apdu.le = BUFFER_SIZE;
-		apdu.resp = buf;
-		apdu.resplen = MAX_RESP_BUFFER_SIZE;	/* include SW's */
-		result = sc_transmit_apdu(card, &apdu);
-		if (result != SC_SUCCESS) {
-			LOG_TEST_RET(card->ctx, result, "APDU transmit failed");
-		}
-		if (apdu.resplen != BUFFER_SIZE) {
-			result = sc_check_sw(card, apdu.sw1, apdu.sw2);
-			goto dnie_get_challenge_error;
-		}
-		memcpy(rnd, apdu.resp, n);
-		len -= n;
-		rnd += n;
+	LOG_FUNC_CALLED(card->ctx);
+
+	r = iso_ops->get_challenge(card, rbuf, sizeof rbuf);
+	LOG_TEST_RET(card->ctx, r, "GET CHALLENGE cmd failed");
+
+	if (len < (size_t) r) {
+		out_len = len;
+	} else {
+		out_len = (size_t) r;
 	}
-	result = SC_SUCCESS;
- dnie_get_challenge_error:
-	LOG_FUNC_RETURN(card->ctx, result);
+	memcpy(rnd, rbuf, out_len);
+
+	LOG_FUNC_RETURN(card->ctx, (int) out_len);
 }
 
 /*
@@ -1533,11 +1519,6 @@ static int dnie_set_security_env(struct sc_card *card,
 		apdu.p1 = 0x41;	/* SET; internal operation */
 		apdu.p2 = 0xB6;	/* Template for Digital Signature */
 		break;
-	case SC_SEC_OPERATION_AUTHENTICATE:
-		/* TODO: _set_security_env() study diffs on internal/external auth */
-		apdu.p1 = 0x41;	/* SET; internal operation */
-		apdu.p2 = 0xA4;	/* Template for Authenticate */
-		break;
 	default:
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_ARGUMENTS);
 	}
