@@ -86,7 +86,7 @@ static int	gpk_get_info(sc_card_t *, int, int, u8 *, size_t);
 /*
  * ATRs of GPK4000 cards courtesy of libscez
  */
-static struct sc_atr_table gpk_atrs[] = {
+static const struct sc_atr_table gpk_atrs[] = {
 	{ "3B:27:00:80:65:A2:04:01:01:37", NULL, "GPK 4K", SC_CARD_TYPE_GPK_GPK4000_s, 0, NULL },
 	{ "3B:27:00:80:65:A2:05:01:01:37", NULL, "GPK 4K", SC_CARD_TYPE_GPK_GPK4000_sp, 0, NULL },
 	{ "3B:27:00:80:65:A2:0C:01:01:37", NULL, "GPK 4K", SC_CARD_TYPE_GPK_GPK4000_su256, 0, NULL },
@@ -206,7 +206,7 @@ gpk_init(sc_card_t *card)
 	/* Make sure max send/receive size is 4 byte aligned and <256. */
 	card->max_recv_size = 252;
 
-	return 0;
+	return SC_SUCCESS;
 }
 
 /*
@@ -409,6 +409,9 @@ gpk_parse_fileinfo(sc_card_t *card,
 		if (sp[0] == 0x85) {
 			unsigned int	ac[3], n;
 
+			if (sp + 11 + 2*3 >= end)
+				break;
+
 			file->id = (sp[4] << 8) | sp[5];
 			file->size = (sp[8] << 8) | sp[9];
 			file->record_length = sp[7];
@@ -501,9 +504,9 @@ gpk_select(sc_card_t *card, int kind,
 	}
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	/* Nothing we can say about it... invalidate
 	 * path cache */
@@ -531,7 +534,7 @@ gpk_select_id(sc_card_t *card, int kind, unsigned int fid,
 	u8		fbuf[2];
 	int		r;
 
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+	sc_log(card->ctx, 
 		"gpk_select_id(0x%04X, kind=%u)\n", fid, kind);
 
 	fbuf[0] = fid >> 8;
@@ -593,6 +596,7 @@ try_again:
 	if ((path->len & 1) || path->len > sizeof(pathtmp))
 		return SC_ERROR_INVALID_ARGUMENTS;
 	pathptr = pathtmp;
+	memset(pathtmp, 0, sizeof pathtmp);
 	for (n = 0; n < path->len; n += 2)
 		pathptr[n>>1] = (path->value[n] << 8)|path->value[n+1];
 	pathlen = path->len >> 1;
@@ -620,7 +624,7 @@ try_again:
 	} else {
 		if (!locked++) {
 			r = sc_lock(card);
-			SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "sc_lock() failed");
+			LOG_TEST_RET(card->ctx, r, "sc_lock() failed");
 		}
 
 		/* Do we need to select the MF first? */
@@ -628,7 +632,7 @@ try_again:
 			r = gpk_select_id(card, GPK_SEL_MF, GPK_FID_MF, NULL);
 			if (r)
 				sc_unlock(card);
-			SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Unable to select MF");
+			LOG_TEST_RET(card->ctx, r, "Unable to select MF");
 
 			/* Consume the MF FID if it's there */
 			if (pathptr[0] == GPK_FID_MF) {
@@ -646,7 +650,7 @@ try_again:
 			r = gpk_select_id(card, GPK_SEL_DF, pathptr[0], NULL);
 			if (r)
 				sc_unlock(card);
-			SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Unable to select DF");
+			LOG_TEST_RET(card->ctx, r, "Unable to select DF");
 			pathptr++;
 			pathlen--;
 		}
@@ -682,7 +686,7 @@ gpk_read_binary(sc_card_t *card, unsigned int offset,
 	struct gpk_private_data *priv = DRVDATA(card);
 
 	if (offset & priv->offset_mask) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Invalid file offset (not a multiple of %d)",
+		sc_log(card->ctx,  "Invalid file offset (not a multiple of %d)",
 				priv->offset_mask + 1);
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
@@ -697,7 +701,7 @@ gpk_write_binary(sc_card_t *card, unsigned int offset,
 	struct gpk_private_data *priv = DRVDATA(card);
 
 	if (offset & priv->offset_mask) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Invalid file offset (not a multiple of %d)",
+		sc_log(card->ctx,  "Invalid file offset (not a multiple of %d)",
 				priv->offset_mask + 1);
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
@@ -712,7 +716,7 @@ gpk_update_binary(sc_card_t *card, unsigned int offset,
 	struct gpk_private_data *priv = DRVDATA(card);
 
 	if (offset & priv->offset_mask) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Invalid file offset (not a multiple of %d)",
+		sc_log(card->ctx,  "Invalid file offset (not a multiple of %d)",
 				priv->offset_mask + 1);
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
@@ -782,7 +786,7 @@ gpk_verify_crycks(sc_card_t *card, sc_apdu_t *apdu, u8 *crycks)
 {
 	if (apdu->resplen < 3
 	 || memcmp(apdu->resp + apdu->resplen - 3, crycks, 3)) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+		sc_log(card->ctx, 
 			"Invalid secure messaging reply\n");
 		return SC_ERROR_UNKNOWN_DATA_RECEIVED;
 	}
@@ -805,7 +809,7 @@ gpk_create_file(sc_card_t *card, sc_file_t *file)
 	size_t		datalen, namelen;
 	int		r;
 
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+	sc_log(card->ctx, 
 		"gpk_create_file(0x%04X)\n", file->id);
 
 	/* Prepare APDU */
@@ -867,9 +871,9 @@ gpk_create_file(sc_card_t *card, sc_file_t *file)
 	}
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	/* verify secure messaging response */
 	if (priv->key_set)
@@ -964,9 +968,9 @@ gpk_select_key(sc_card_t *card, int key_sfi, const u8 *buf, size_t buflen)
 	apdu.le = 12;
 	
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	if (apdu.resplen != 12) {
 		r = SC_ERROR_UNKNOWN_DATA_RECEIVED;
@@ -1011,7 +1015,7 @@ gpk_set_security_env(sc_card_t *card,
 	if (env->flags & SC_SEC_ENV_ALG_PRESENT)
 		algorithm = env->algorithm;
 	if (algorithm != SC_ALGORITHM_RSA) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Algorithm not supported.\n");
+		sc_log(card->ctx,  "Algorithm not supported.\n");
 		return SC_ERROR_NOT_SUPPORTED;
 	}
 	priv->sec_algorithm = algorithm;
@@ -1019,7 +1023,7 @@ gpk_set_security_env(sc_card_t *card,
 	/* If there's a key reference, it must be 0 */
 	if ((env->flags & SC_SEC_ENV_KEY_REF_PRESENT)
 	 && (env->key_ref_len != 1 || env->key_ref[0] != 0)) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Unknown key referenced.\n");
+		sc_log(card->ctx,  "Unknown key referenced.\n");
 		return SC_ERROR_NOT_SUPPORTED;
 	}
 
@@ -1032,7 +1036,7 @@ gpk_set_security_env(sc_card_t *card,
 	else if (env->flags & SC_ALGORITHM_RSA_PAD_ISO9796)
 		priv->sec_padding = 2;
 	else {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Padding algorithm not supported.\n");
+		sc_log(card->ctx,  "Padding algorithm not supported.\n");
 		return SC_ERROR_NOT_SUPPORTED;
 	}
 
@@ -1055,7 +1059,7 @@ gpk_set_security_env(sc_card_t *card,
 			context = GPK_SIGN_RSA_MD5;
 			priv->sec_hash_len = 16;
 		} else {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Unsupported signature algorithm");
+			sc_log(card->ctx,  "Unsupported signature algorithm");
 			return SC_ERROR_NOT_SUPPORTED;
 		}
 		break;
@@ -1063,38 +1067,38 @@ gpk_set_security_env(sc_card_t *card,
 		context = GPK_UNWRAP_RSA;
 		break;
 	default:
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Crypto operation not supported.\n");
+		sc_log(card->ctx,  "Crypto operation not supported.\n");
 		return SC_ERROR_NOT_SUPPORTED;
 	}
 
 	/* Get the file ID */
 	if (env->flags & SC_SEC_ENV_FILE_REF_PRESENT) {
 		if (env->file_ref.len != 2) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "File reference: invalid length.\n");
+			sc_log(card->ctx,  "File reference: invalid length.\n");
 			return SC_ERROR_INVALID_ARGUMENTS;
 		}
 		file_id = (env->file_ref.value[0] << 8)
 			| env->file_ref.value[1];
 	} else {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "File reference missing.\n");
+		sc_log(card->ctx,  "File reference missing.\n");
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 
 	/* Select the PK file. The caller has already selected
 	 * the DF. */
 	r = gpk_select_id(card, GPK_SEL_EF, file_id, NULL);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Failed to select PK file");
+	LOG_TEST_RET(card->ctx, r, "Failed to select PK file");
 
 	/* Read the sys record of the PK file to find out the key length */
 	r = sc_read_record(card, 1, sysrec, sizeof(sysrec),
 			SC_RECORD_BY_REC_NR);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Failed to read PK sysrec");
+	LOG_TEST_RET(card->ctx, r, "Failed to read PK sysrec");
 	if (r != 7 || sysrec[0] != 0) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "First record of file is not the sysrec");
+		sc_log(card->ctx,  "First record of file is not the sysrec");
 		return SC_ERROR_OBJECT_NOT_VALID;
 	}
 	if (sysrec[5] != 0x00) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Public key is not an RSA key");
+		sc_log(card->ctx,  "Public key is not an RSA key");
 		return SC_ERROR_OBJECT_NOT_VALID;
 	}
 	switch (sysrec[1]) {
@@ -1102,7 +1106,7 @@ gpk_set_security_env(sc_card_t *card,
 	case 0x10: priv->sec_mod_len =  768 / 8; break;
 	case 0x11: priv->sec_mod_len = 1024 / 8; break;
 	default:
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Unsupported modulus length");
+		sc_log(card->ctx,  "Unsupported modulus length");
 		return SC_ERROR_OBJECT_NOT_VALID;
 	}
 
@@ -1115,9 +1119,9 @@ gpk_set_security_env(sc_card_t *card,
 	apdu.p2  = context;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	return r;
 }
@@ -1181,9 +1185,9 @@ gpk_hash(sc_card_t *card, const u8 *data, size_t datalen)
 		apdu.datalen = len + 2;
 
 		r = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+		LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 		r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+		LOG_TEST_RET(card->ctx, r, "Card returned error");
 		chain = 0;
 	}
 
@@ -1202,7 +1206,7 @@ gpk_init_hashed(sc_card_t *card, const u8 *digest, unsigned int len)
 	int		r;
 
 	r = reverse(tsegid, sizeof(tsegid), digest, len);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Failed to reverse buffer");
+	LOG_TEST_RET(card->ctx, r, "Failed to reverse buffer");
 
 	memset(&apdu, 0, sizeof(apdu));
 	apdu.cse = SC_APDU_CASE_3_SHORT;
@@ -1213,9 +1217,9 @@ gpk_init_hashed(sc_card_t *card, const u8 *digest, unsigned int len)
 	apdu.datalen = len;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	return r;
 }
@@ -1234,7 +1238,7 @@ gpk_compute_signature(sc_card_t *card, const u8 *data,
 	int		r;
 
 	if (data_len > priv->sec_mod_len) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+		sc_log(card->ctx, 
 			 "Data length (%"SC_FORMAT_LEN_SIZE_T"u) does not match key modulus %u.\n",
 			 data_len, priv->sec_mod_len);
 		return SC_ERROR_INTERNAL;
@@ -1243,7 +1247,7 @@ gpk_compute_signature(sc_card_t *card, const u8 *data,
 		return SC_ERROR_BUFFER_TOO_SMALL;
 
 	r = gpk_init_hashed(card, data, data_len);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Failed to send hash to card");
+	LOG_TEST_RET(card->ctx, r, "Failed to send hash to card");
 
 	/* Now sign the hash.
 	 * The GPK has Internal Authenticate and PK_Sign. I am not
@@ -1258,14 +1262,14 @@ gpk_compute_signature(sc_card_t *card, const u8 *data,
 	apdu.le  = priv->sec_mod_len;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	/* The GPK returns the signature as little endian numbers.
 	 * Need to revert these */
 	r = reverse(out, outlen, cardsig, apdu.resplen);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Failed to reverse signature");
+	LOG_TEST_RET(card->ctx, r, "Failed to reverse signature");
 
 	return r;
 }
@@ -1288,7 +1292,7 @@ gpk_decipher(sc_card_t *card, const u8 *in, size_t inlen,
 	int		r;
 
 	if (inlen != priv->sec_mod_len) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+		sc_log(card->ctx, 
 			 "Data length (%"SC_FORMAT_LEN_SIZE_T"u) does not match key modulus %u.\n",
 			 inlen, priv->sec_mod_len);
 		return SC_ERROR_INVALID_ARGUMENTS;
@@ -1296,7 +1300,7 @@ gpk_decipher(sc_card_t *card, const u8 *in, size_t inlen,
 
 	/* First revert the cryptogram */
 	r = reverse(buffer, sizeof(buffer), in, inlen);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Cryptogram too large");
+	LOG_TEST_RET(card->ctx, r, "Cryptogram too large");
 	in = buffer;
 
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_4_SHORT, 0x1C, 0x00, 0x00);
@@ -1309,13 +1313,13 @@ gpk_decipher(sc_card_t *card, const u8 *in, size_t inlen,
 	apdu.resplen = sizeof(buffer);
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	/* Reverse the data we got back */
 	r = reverse(out, outlen, buffer, apdu.resplen);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Failed to reverse buffer");
+	LOG_TEST_RET(card->ctx, r, "Failed to reverse buffer");
 
 	return r;
 }
@@ -1360,9 +1364,9 @@ gpk_erase_card(sc_card_t *card)
 	apdu.p2  = offset;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	priv->key_set = 0;
 	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
@@ -1385,7 +1389,7 @@ gpk_lock(sc_card_t *card, struct sc_cardctl_gpk_lock *args)
 	u8		data[8], crycks[3], resp[3];
 	int		r;
 
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+	sc_log(card->ctx, 
 		"gpk_lock(0x%04X, %u)\n", file->id, args->operation);
 
 	memset(data, 0, sizeof(data));
@@ -1423,9 +1427,9 @@ gpk_lock(sc_card_t *card, struct sc_cardctl_gpk_lock *args)
 	}
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	if (priv->key_set)
 		r = gpk_verify_crycks(card, &apdu, crycks);
@@ -1442,7 +1446,7 @@ gpk_pkfile_init(sc_card_t *card, struct sc_cardctl_gpk_pkinit *args)
 	sc_apdu_t	apdu;
 	int		r;
 
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+	sc_log(card->ctx, 
 		"gpk_pkfile_init(%u)\n", args->privlen);
 
 	memset(&apdu, 0, sizeof(apdu));
@@ -1453,9 +1457,9 @@ gpk_pkfile_init(sc_card_t *card, struct sc_cardctl_gpk_pkinit *args)
 	apdu.p2  = args->privlen / 4;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	return r;
 }
@@ -1470,10 +1474,10 @@ gpk_generate_key(sc_card_t *card, struct sc_cardctl_gpk_genkey *args)
 	int		r;
 	u8		buffer[256];
 
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+	sc_log(card->ctx, 
 		"gpk_generate_key(%u)\n", args->privlen);
 	if (args->privlen != 512 && args->privlen != 1024) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+		sc_log(card->ctx, 
 			"Key generation not supported for key length %d",
 			args->privlen);
 		return SC_ERROR_NOT_SUPPORTED;
@@ -1490,16 +1494,16 @@ gpk_generate_key(sc_card_t *card, struct sc_cardctl_gpk_genkey *args)
 	apdu.resplen = 256;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	/* Return the public key, inverted.
 	 * The first two bytes must be stripped off. */
 	if (args->pubkey_len && apdu.resplen > 2) {
 		r = reverse(args->pubkey, args->pubkey_len,
 				buffer + 2, apdu.resplen - 2);
-		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Failed to reverse buffer");
+		LOG_TEST_RET(card->ctx, r, "Failed to reverse buffer");
 		args->pubkey_len = r;
 	}
 
@@ -1507,7 +1511,7 @@ gpk_generate_key(sc_card_t *card, struct sc_cardctl_gpk_genkey *args)
 }
 
 /*
- * Store a privat key component
+ * Store a private key component
  */
 static int
 gpk_pkfile_load(sc_card_t *card, struct sc_cardctl_gpk_pkload *args)
@@ -1519,7 +1523,7 @@ gpk_pkfile_load(sc_card_t *card, struct sc_cardctl_gpk_pkload *args)
 	int		r = SC_SUCCESS, outl;
 	EVP_CIPHER_CTX  * ctx;
 
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "gpk_pkfile_load(fid=%04x, len=%d, datalen=%d)\n",
+	sc_log(card->ctx,  "gpk_pkfile_load(fid=%04x, len=%d, datalen=%d)\n",
 			args->file->id, args->len, args->datalen);
 
 	ctx = EVP_CIPHER_CTX_new();
@@ -1527,13 +1531,8 @@ gpk_pkfile_load(sc_card_t *card, struct sc_cardctl_gpk_pkload *args)
 		return SC_ERROR_INTERNAL;
 
 	if (0) {
-		char buf[2048];
-
-		sc_hex_dump(card->ctx, SC_LOG_DEBUG_NORMAL,
-				args->data, args->datalen,
-				buf, sizeof(buf));
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Sending %d bytes (cleartext):\n%s",
-				args->datalen, buf);
+		sc_log_hex(card->ctx, "Sending (cleartext)",
+				args->data, args->datalen);
 	}
 
 	memset(&apdu, 0, sizeof(apdu));
@@ -1547,7 +1546,7 @@ gpk_pkfile_load(sc_card_t *card, struct sc_cardctl_gpk_pkload *args)
 	/* encrypt the private key material */
 	assert(args->datalen <= sizeof(temp));
 	if (!priv->key_set) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "No secure messaging key set!\n");
+		sc_log(card->ctx,  "No secure messaging key set!\n");
 		return SC_ERROR_SECURITY_STATUS_NOT_SATISFIED;
 	}
 
@@ -1570,11 +1569,11 @@ gpk_pkfile_load(sc_card_t *card, struct sc_cardctl_gpk_pkload *args)
 	priv->key_set = 0;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
+	LOG_FUNC_RETURN(card->ctx, r);
 }
 
 /*
@@ -1612,7 +1611,7 @@ static int gpk_get_info(sc_card_t *card, int p1, int p2, u8 *buf,
 	 * without collecting the response :)
 	 */
 	r = sc_lock(card);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "sc_lock() failed");
+	LOG_TEST_RET(card->ctx, r, "sc_lock() failed");
 
 	do {
 		memset(&apdu, 0, sizeof(apdu));
@@ -1626,7 +1625,7 @@ static int gpk_get_info(sc_card_t *card, int p1, int p2, u8 *buf,
 		apdu.resplen = buflen;
 
 		if ((r = sc_transmit_apdu(card, &apdu)) < 0) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "APDU transmit failed: %s",
+			sc_log(card->ctx,  "APDU transmit failed: %s",
 					sc_strerror(r));
 			sc_unlock(card);
 			return r;
@@ -1635,7 +1634,7 @@ static int gpk_get_info(sc_card_t *card, int p1, int p2, u8 *buf,
 	sc_unlock(card);
 
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Card returned error");
+	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	return r;
 }
@@ -1665,7 +1664,7 @@ static int gpk_get_serialnr(sc_card_t *card, sc_serial_number_t *serial)
 	apdu.lc   = 0;
 	apdu.datalen = 0;
         r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
+	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
 		return SC_ERROR_INTERNAL;
 	/* cache serial number */
