@@ -31,6 +31,7 @@
 #include "sm/sm-eac.h"
 #include <string.h>
 
+static int fread_to_eof(const char *file, unsigned char **buf, size_t *buflen);
 #include "../tools/fread_to_eof.c"
 
 struct npa_drv_data {
@@ -135,7 +136,7 @@ static int npa_match_card(sc_card_t * card)
 {
 	int r = 0;
 
-	if (0 == r && SC_SUCCESS == sc_enum_apps(card)) {
+	if (SC_SUCCESS == sc_enum_apps(card)) {
 		unsigned char esign_aid_0[] = {
 			0xE8, 0x28, 0xBD, 0x08, 0x0F, 0xA0, 0x00, 0x00, 0x01, 0x67, 0x45, 0x53, 0x49, 0x47, 0x4E,
 		}, esign_aid_1[] = {
@@ -630,8 +631,10 @@ npa_reset_retry_counter(sc_card_t *card, enum s_type pin_id,
 				return SC_ERROR_INTERNAL;
 			}
 			new_len = strlen(p);
-			if (new_len > EAC_MAX_PIN_LEN)
+			if (new_len > EAC_MAX_PIN_LEN) {
+				free(p);
 				return SC_ERROR_INVALID_PIN_LENGTH;
+			}
 			new = p;
 #else
 			return SC_ERROR_NOT_SUPPORTED;
@@ -762,7 +765,7 @@ static int npa_pin_cmd(struct sc_card *card,
 	}
 
 err:
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
+	LOG_FUNC_RETURN(card->ctx, r);
 }
 
 static int npa_logout(sc_card_t *card)
