@@ -39,6 +39,7 @@
 #include "internal.h"
 #include "cardctl.h"
 #include "pkcs15.h"
+#include "gp.h"
 
 #define OBERTHUR_PIN_LOCAL	0x80
 #define OBERTHUR_PIN_REFERENCE_USER	0x81
@@ -46,15 +47,7 @@
 #define OBERTHUR_PIN_REFERENCE_SO	0x04
 #define OBERTHUR_PIN_REFERENCE_PUK	0x84
 
-/* keep OpenSSL 0.9.6 users happy ;-) */
-#if OPENSSL_VERSION_NUMBER < 0x00907000L
-#define DES_cblock			des_cblock
-#define DES_key_schedule		des_key_schedule
-#define DES_set_key_unchecked(a,b)	des_set_key_unchecked(a,*b)
-#define DES_ecb_encrypt(a,b,c,d) 	des_ecb_encrypt(a,b,*c,d)
-#endif
-
-static struct sc_atr_table oberthur_atrs[] = {
+static const struct sc_atr_table oberthur_atrs[] = {
 	{ "3B:7D:18:00:00:00:31:80:71:8E:64:77:E3:01:00:82:90:00", NULL,
 			"Oberthur 64k v4/2.1.1", SC_CARD_TYPE_OBERTHUR_64K, 0, NULL },
 	{ "3B:7D:18:00:00:00:31:80:71:8E:64:77:E3:02:00:82:90:00", NULL,
@@ -157,19 +150,10 @@ auth_select_aid(struct sc_card *card)
 	unsigned char apdu_resp[SC_MAX_APDU_BUFFER_SIZE];
 	struct auth_private_data *data =  (struct auth_private_data *) card->drv_data;
 	int rv, ii;
-	unsigned char cm[7] = {0xA0,0x00,0x00,0x00,0x03,0x00,0x00};
 	struct sc_path tmp_path;
 
 	/* Select Card Manager (to deselect previously selected application) */
-	sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT, 0xA4, 0x04, 0x0C);
-	apdu.lc = sizeof(cm);
-	/* apdu.le = sizeof(cm)+4; */
-	apdu.data = cm;
-	apdu.datalen = sizeof(cm);
-	apdu.resplen = sizeof(apdu_resp);
-	apdu.resp = apdu_resp;
-
-	rv = sc_transmit_apdu(card, &apdu);
+	rv = gp_select_card_manager(card);
 	LOG_TEST_RET(card->ctx, rv, "APDU transmit failed");
 
 	/* Get smart card serial number */
@@ -242,20 +226,27 @@ auth_init(struct sc_card *card)
 
 	if (auth_select_aid(card))   {
 		sc_log(card->ctx, "Failed to initialize %s", card->name);
-		LOG_TEST_RET(card->ctx, SC_ERROR_INVALID_CARD, "Failed to initialize");
+		rv = SC_ERROR_INVALID_CARD;
+		LOG_TEST_GOTO_ERR(card->ctx, SC_ERROR_INVALID_CARD, "Failed to initialize");
 	}
 
-	flags = SC_ALGORITHM_RSA_PAD_PKCS1 | SC_ALGORITHM_RSA_PAD_ISO9796;
-	flags |= SC_ALGORITHM_RSA_HASH_NONE;
-	flags |= SC_ALGORITHM_ONBOARD_KEY_GEN;
-
-	_sc_card_add_rsa_alg(card, 512, flags, 0);
-	_sc_card_add_rsa_alg(card, 1024, flags, 0);
-	_sc_card_add_rsa_alg(card, 2048, flags, 0);
-
 	sc_format_path("3F00", &path);
 	rv = auth_select_file(card, &path, NULL);
 
+err:
+	if (rv == SC_SUCCESS) {
+		flags = SC_ALGORITHM_RSA_PAD_PKCS1 | SC_ALGORITHM_RSA_PAD_ISO9796;
+		flags |= SC_ALGORITHM_RSA_HASH_NONE;
+		flags |= SC_ALGORITHM_ONBOARD_KEY_GEN;
+
+		_sc_card_add_rsa_alg(card, 512, flags, 0);
+		_sc_card_add_rsa_alg(card, 1024, flags, 0);
+		_sc_card_add_rsa_alg(card, 2048, flags, 0);
+	} else {
+		free(card->drv_data);
+		card->drv_data = NULL;
+	}
+
 	LOG_FUNC_RETURN(card->ctx, rv);
 }
 
@@ -297,82 +288,57 @@ add_acl_entry(struct sc_card *card, struct sc_file *file, unsigned int op,
 }
 
 
-static int
-tlv_get(const unsigned char *msg, int len, unsigned char tag,
-		unsigned char *ret, int *ret_len)
-{
-	int cur = 0;
-
-	while (cur < len)  {
-		if (*(msg+cur)==tag)  {
-			int ii, ln = *(msg+cur+1);
-
-			if (ln > *ret_len)
-				return SC_ERROR_WRONG_LENGTH;
-
-			for (ii=0; ii<ln; ii++)
-				*(ret + ii) = *(msg+cur+2+ii);
-			*ret_len = ln;
-
-			return SC_SUCCESS;
-		}
-
-		cur += 2 + *(msg+cur+1);
-	}
-
-	return SC_ERROR_INCORRECT_PARAMETERS;
-}
-
-
 static int
 auth_process_fci(struct sc_card *card, struct sc_file *file,
             const unsigned char *buf, size_t buflen)
 {
-	unsigned char type, attr[SC_OBERTHUR_MAX_ATTR_SIZE];
-	int attr_len = sizeof(attr);
+	unsigned char type;
+	const unsigned char *attr;
+	size_t attr_len = 0;
 
 	LOG_FUNC_CALLED(card->ctx);
-	attr_len = sizeof(attr);
-	if (tlv_get(buf, buflen, 0x82, attr, &attr_len))
+	attr = sc_asn1_find_tag(card->ctx, buf, buflen, 0x82, &attr_len);
+	if (!attr || attr_len < 1)
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 	type = attr[0];
 
-	attr_len = sizeof(attr);
-	if (tlv_get(buf, buflen, 0x83, attr, &attr_len))
+	attr = sc_asn1_find_tag(card->ctx, buf, buflen, 0x83, &attr_len);
+	if (!attr || attr_len < 2)
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 	file->id = attr[0]*0x100 + attr[1];
 
-	attr_len = sizeof(attr);
-	if (tlv_get(buf, buflen, type==0x01 ? 0x80 : 0x85, attr, &attr_len))
-		LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
-	if (attr_len<2 && type != 0x04)
-		LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
-
+	attr = sc_asn1_find_tag(card->ctx, buf, buflen, type==0x01 ? 0x80 : 0x85, &attr_len);
 	switch (type) {
 	case 0x01:
+		if (!attr || attr_len < 2)
+			LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 		file->type = SC_FILE_TYPE_WORKING_EF;
 		file->ef_structure = SC_FILE_EF_TRANSPARENT;
 		file->size = attr[0]*0x100 + attr[1];
 		break;
 	case 0x04:
+		if (!attr || attr_len < 1)
+			LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 		file->type = SC_FILE_TYPE_WORKING_EF;
 		file->ef_structure = SC_FILE_EF_LINEAR_VARIABLE;
 		file->size = attr[0];
-		attr_len = sizeof(attr);
-		if (tlv_get(buf, buflen, 0x82, attr, &attr_len))
-			LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
-		if (attr_len!=5)
+		attr = sc_asn1_find_tag(card->ctx, buf, buflen, 0x82, &attr_len);
+		if (!attr || attr_len < 5)
 			LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 		file->record_length = attr[2]*0x100+attr[3];
 		file->record_count = attr[4];
 		break;
 	case 0x11:
+		if (!attr || attr_len < 2)
+			LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 		file->type = SC_FILE_TYPE_INTERNAL_EF;
 		file->ef_structure = SC_CARDCTL_OBERTHUR_KEY_DES;
 		file->size = attr[0]*0x100 + attr[1];
 		file->size /= 8;
 		break;
 	case 0x12:
+		if (!attr || attr_len < 2)
+			LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 		file->type = SC_FILE_TYPE_INTERNAL_EF;
 		file->ef_structure = SC_CARDCTL_OBERTHUR_KEY_RSA_PUBLIC;
 
@@ -391,11 +357,15 @@ auth_process_fci(struct sc_card *card, struct sc_file *file,
 		}
 		break;
 	case 0x14:
+		if (!attr || attr_len < 2)
+			LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 		file->type = SC_FILE_TYPE_INTERNAL_EF;
 		file->ef_structure = SC_CARDCTL_OBERTHUR_KEY_RSA_CRT;
 		file->size = attr[0]*0x100 + attr[1];
 		break;
 	case 0x38:
+		if (!attr || attr_len < 1)
+			LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 		file->type = SC_FILE_TYPE_DF;
 		file->size = attr[0];
 		if (SC_SUCCESS != sc_file_set_type_attr(file,attr,attr_len))
@@ -405,10 +375,8 @@ auth_process_fci(struct sc_card *card, struct sc_file *file,
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 	}
 
-	attr_len = sizeof(attr);
-	if (tlv_get(buf, buflen, 0x86, attr, &attr_len))
-		LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
-	if (attr_len<8)
+	attr = sc_asn1_find_tag(card->ctx, buf, buflen, 0x86, &attr_len);
+	if (!attr || attr_len < 8)
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 
 	if (file->type == SC_FILE_TYPE_DF) {
@@ -484,6 +452,9 @@ auth_select_file(struct sc_card *card, const struct sc_path *in_path,
 
 	memcpy(&path, in_path, sizeof(struct sc_path));
 
+	if (!auth_current_df)
+		return SC_ERROR_OBJECT_NOT_FOUND;
+
 	sc_log(card->ctx, "in_path; type=%d, path=%s, out %p",
 			in_path->type, sc_print_path(in_path), file_out);
 	sc_log(card->ctx, "current path; type=%d, path=%s",
@@ -1361,7 +1332,7 @@ auth_update_component(struct sc_card *card, struct auth_update_component_info *a
 
 		ctx = EVP_CIPHER_CTX_new();
 		if (ctx == NULL) 
-		    SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL,SC_ERROR_OUT_OF_MEMORY);
+		    LOG_FUNC_RETURN(card->ctx, SC_ERROR_OUT_OF_MEMORY);
 
 		p2 = 0;
 		if (args->len == 24)
@@ -2121,6 +2092,10 @@ auth_read_binary(struct sc_card *card, unsigned int offset,
 	bn[1].data = NULL;
 
 	LOG_FUNC_CALLED(card->ctx);
+
+	if (!auth_current_ef)
+		LOG_TEST_RET(card->ctx, SC_ERROR_INVALID_ARGUMENTS, "Invalid auth_current_ef");
+
 	sc_log(card->ctx,
 	       "offset %i; size %"SC_FORMAT_LEN_SIZE_T"u; flags 0x%lX",
 	       offset, count, flags);
@@ -2145,6 +2120,8 @@ auth_read_binary(struct sc_card *card, unsigned int offset,
 		for (jj=0; jj<rv && *(resp+jj)==0; jj++)
 			;
 
+		if (rv - jj == 0)
+			return SC_ERROR_INVALID_DATA;
 		bn[0].data = calloc(1, rv - jj);
 		if (!bn[0].data) {
 			rv = SC_ERROR_OUT_OF_MEMORY;
