@@ -28,16 +28,13 @@
 #include <unistd.h>
 #endif
 #include <string.h>
+#include <limits.h>
 
 #include "reader-tr03119.h"
 #include "internal.h"
 #include "asn1.h"
 #include "common/compat_strlcpy.h"
 
-/*
-#define INVALIDATE_CARD_CACHE_IN_UNLOCK
-*/
-
 #ifdef ENABLE_SM
 static int sc_card_sm_load(sc_card_t *card, const char *path, const char *module);
 static int sc_card_sm_unload(sc_card_t *card);
@@ -67,6 +64,65 @@ void sc_format_apdu(sc_card_t *card, sc_apdu_t *apdu,
 	apdu->p2 = (u8) p2;
 }
 
+void sc_format_apdu_cse_lc_le(struct sc_apdu *apdu)
+{
+	/* TODO calculating the APDU case, Lc and Le should actually only be
+	 * done in sc_apdu2bytes, but to gradually change OpenSC we start here. */
+	/* Let sc_detect_apdu_cse set short or extended  and test for chaining */
+
+	if (!apdu)
+		return;
+	if (apdu->datalen > SC_MAX_APDU_DATA_SIZE
+			|| apdu->resplen > SC_MAX_APDU_RESP_SIZE) {
+		/* extended length  or data chaining and/or get response */
+		if (apdu->datalen <= SC_MAX_EXT_APDU_DATA_SIZE)
+			apdu->lc = apdu->datalen;
+		if (apdu->resplen <= SC_MAX_EXT_APDU_RESP_SIZE)
+			apdu->le = apdu->resplen;
+		if (apdu->resplen && !apdu->datalen)
+			apdu->cse = SC_APDU_CASE_2;
+		if (!apdu->resplen && apdu->datalen)
+			apdu->cse = SC_APDU_CASE_3;
+		if (apdu->resplen && apdu->datalen)
+			apdu->cse = SC_APDU_CASE_4;
+	} else {
+		/* short length */
+		if (apdu->datalen <= SC_MAX_APDU_DATA_SIZE)
+			apdu->lc = apdu->datalen;
+		if (apdu->resplen <= SC_MAX_APDU_RESP_SIZE)
+			apdu->le = apdu->resplen;
+		if (!apdu->resplen && !apdu->datalen)
+			apdu->cse = SC_APDU_CASE_1;
+		if (apdu->resplen && !apdu->datalen)
+			apdu->cse = SC_APDU_CASE_2_SHORT;
+		if (!apdu->resplen && apdu->datalen)
+			apdu->cse = SC_APDU_CASE_3_SHORT;
+		if (apdu->resplen && apdu->datalen)
+			apdu->cse = SC_APDU_CASE_4_SHORT;
+	}
+}
+
+void sc_format_apdu_ex(struct sc_apdu *apdu,
+		u8 cla, u8 ins, u8 p1, u8 p2,
+		const u8 *data, size_t datalen,
+		u8 *resp, size_t resplen)
+{
+	if (!apdu) {
+		return;
+	}
+
+	memset(apdu, 0, sizeof(*apdu));
+	apdu->cla = cla;
+	apdu->ins = ins;
+	apdu->p1 = p1;
+	apdu->p2 = p2;
+	apdu->resp = resp;
+	apdu->resplen = resplen;
+	apdu->data = data;
+	apdu->datalen = datalen;
+	sc_format_apdu_cse_lc_le(apdu);
+}
+
 static sc_card_t * sc_card_new(sc_context_t *ctx)
 {
 	sc_card_t *card;
@@ -101,8 +157,6 @@ static void sc_card_free(sc_card_t *card)
 	sc_free_apps(card);
 	sc_free_ef_atr(card);
 
-	sc_file_free(card->ef_dir);
-
 	free(card->ops);
 
 	if (card->algorithms != NULL)   {
@@ -142,7 +196,7 @@ size_t sc_get_max_recv_size(const sc_card_t *card)
 	}
 	max_recv_size = card->max_recv_size;
 
-	/* initialize max_recv_size to a meaningfull value */
+	/* initialize max_recv_size to a meaningful value */
 	if (card->caps & SC_CARD_CAP_APDU_EXT) {
 		if (!max_recv_size)
 			max_recv_size = 65536;
@@ -169,7 +223,7 @@ size_t sc_get_max_send_size(const sc_card_t *card)
 
 	max_send_size = card->max_send_size;
 
-	/* initialize max_send_size to a meaningfull value */
+	/* initialize max_send_size to a meaningful value */
 	if (card->caps & SC_CARD_CAP_APDU_EXT
 			&& card->reader->active_protocol != SC_PROTO_T0) {
 		if (!max_send_size)
@@ -265,8 +319,22 @@ int sc_connect_card(sc_reader_t *reader, sc_card_t **card_out)
 		}
 	}
 	else {
+		sc_card_t uninitialized = *card;
 		sc_log(ctx, "matching built-in ATRs");
 		for (i = 0; ctx->card_drivers[i] != NULL; i++) {
+			/* FIXME If we had a clean API description, we'd propably get a
+			 * cleaner implementation of the driver's match_card and init,
+			 * which should normally *not* modify the card object if
+			 * unsuccessful. However, after years of relentless hacking, reality
+			 * is different: The card object is changed in virtually every card
+			 * driver so in order to prevent unwanted interaction, we reset the
+			 * card object here and hope that the card driver at least doesn't
+			 * allocate any internal ressources that need to be freed. If we
+			 * had more time, we should refactor the existing code to not
+			 * modify sc_card_t until complete success (possibly by combining
+			 * `match_card()` and `init()`) */
+			*card = uninitialized;
+
 			struct sc_card_driver *drv = ctx->card_drivers[i];
 			const struct sc_card_operations *ops = drv->ops;
 
@@ -307,7 +375,7 @@ int sc_connect_card(sc_reader_t *reader, sc_card_t **card_out)
 	if (card->name == NULL)
 		card->name = card->driver->name;
 
-	/* initialize max_send_size/max_recv_size to a meaningfull value */
+	/* initialize max_send_size/max_recv_size to a meaningful value */
 	card->max_recv_size = sc_get_max_recv_size(card);
 	card->max_send_size = sc_get_max_send_size(card);
 
@@ -382,9 +450,7 @@ int sc_reset(sc_card_t *card, int do_cold_reset)
 		return r;
 
 	r = card->reader->ops->reset(card->reader, do_cold_reset);
-	/* invalidate cache */
-	memset(&card->cache, 0, sizeof(card->cache));
-	card->cache.valid = 0;
+	sc_invalidate_cache(card);
 
 	r2 = sc_mutex_unlock(card->ctx, card->mutex);
 	if (r2 != SC_SUCCESS) {
@@ -413,9 +479,7 @@ int sc_lock(sc_card_t *card)
 		if (card->reader->ops->lock != NULL) {
 			r = card->reader->ops->lock(card->reader);
 			while (r == SC_ERROR_CARD_RESET || r == SC_ERROR_READER_REATTACHED) {
-				/* invalidate cache */
-				memset(&card->cache, 0, sizeof(card->cache));
-				card->cache.valid = 0;
+				sc_invalidate_cache(card);
 				if (was_reset++ > 4) /* TODO retry a few times */
 					break;
 				r = card->reader->ops->lock(card->reader);
@@ -466,12 +530,12 @@ int sc_unlock(sc_card_t *card)
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 	if (--card->lock_count == 0) {
-#ifdef INVALIDATE_CARD_CACHE_IN_UNLOCK
-		/* invalidate cache */
-		memset(&card->cache, 0, sizeof(card->cache));
-		card->cache.valid = 0;
-		sc_log(card->ctx, "cache invalidated");
-#endif
+		if (card->flags & SC_CARD_FLAG_KEEP_ALIVE) {
+			/* Multiple processes accessing the card will most likely render
+			 * the card cache useless. To not have a bad cache, we explicitly
+			 * invalidate it. */
+			sc_invalidate_cache(card);
+		}
 		/* release reader lock */
 		if (card->reader->ops->unlock != NULL)
 			r = card->reader->ops->unlock(card->reader);
@@ -590,6 +654,11 @@ int sc_read_binary(sc_card_t *card, unsigned int idx,
 				LOG_TEST_RET(card->ctx, r, "sc_read_binary() failed");
 			}
 			p += r;
+			if ((bytes_read > INT_MAX - r) || idx > UINT_MAX - r) {
+				/* `bytes_read + r` or `idx + r` would overflow */
+				sc_unlock(card);
+				LOG_FUNC_RETURN(card->ctx, SC_ERROR_OFFSET_TOO_LARGE);
+			}
 			idx += r;
 			bytes_read += r;
 			count -= r;
@@ -812,17 +881,42 @@ int sc_put_data(sc_card_t *card, unsigned int tag, const u8 *buf, size_t len)
 int sc_get_challenge(sc_card_t *card, u8 *rnd, size_t len)
 {
 	int r;
+	size_t retry = 10;
 
-	if (card == NULL) {
+	if (len == 0)
+		return SC_SUCCESS;
+
+	if (card == NULL || rnd == NULL) {
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
+
 	LOG_FUNC_CALLED(card->ctx);
 
-	if (card->ops->get_challenge == NULL)
+	if (card->ops == NULL || card->ops->get_challenge == NULL)
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_NOT_SUPPORTED);
-	r = card->ops->get_challenge(card, rnd, len);
 
-	LOG_FUNC_RETURN(card->ctx, r);
+	r = sc_lock(card);
+	if (r != SC_SUCCESS)
+		LOG_FUNC_RETURN(card->ctx, r);
+
+	while (len > 0 && retry > 0) {
+		r = card->ops->get_challenge(card, rnd, len);
+		if (r < 0) {
+			sc_unlock(card);
+			LOG_FUNC_RETURN(card->ctx, r);
+		}
+
+		if (r > 0) {
+			rnd += (size_t) r;
+			len -= (size_t) r;
+		} else {
+			retry--;
+		}
+	}
+
+	sc_unlock(card);
+
+	LOG_FUNC_RETURN(card->ctx, SC_SUCCESS);
 }
 
 int sc_read_record(sc_card_t *card, unsigned int rec_nr, u8 *buf,
@@ -939,10 +1033,6 @@ int _sc_card_add_algorithm(sc_card_t *card, const sc_algorithm_info_t *info)
 	}
 	p = (sc_algorithm_info_t *) realloc(card->algorithms, (card->algorithm_count + 1) * sizeof(*info));
 	if (!p) {
-		if (card->algorithms)
-			free(card->algorithms);
-		card->algorithms = NULL;
-		card->algorithm_count = 0;
 		return SC_ERROR_OUT_OF_MEMORY;
 	}
 	card->algorithms = p;
@@ -985,7 +1075,7 @@ int  _sc_card_add_ec_alg(sc_card_t *card, unsigned int key_length,
 	return _sc_card_add_algorithm(card, &info);
 }
 
-static sc_algorithm_info_t * sc_card_find_alg(sc_card_t *card,
+sc_algorithm_info_t * sc_card_find_alg(sc_card_t *card,
 		unsigned int algorithm, unsigned int key_length, void *param)
 {
 	int i;
@@ -1039,7 +1129,7 @@ sc_algorithm_info_t * sc_card_find_gostr3410_alg(sc_card_t *card,
 	return sc_card_find_alg(card, SC_ALGORITHM_GOSTR3410, key_length, NULL);
 }
 
-static int match_atr_table(sc_context_t *ctx, struct sc_atr_table *table, struct sc_atr *atr)
+static int match_atr_table(sc_context_t *ctx, const struct sc_atr_table *table, struct sc_atr *atr)
 {
 	u8 *card_atr_bin;
 	size_t card_atr_bin_len;
@@ -1054,7 +1144,7 @@ static int match_atr_table(sc_context_t *ctx, struct sc_atr_table *table, struct
 	sc_bin_to_hex(card_atr_bin, card_atr_bin_len, card_atr_hex, sizeof(card_atr_hex), ':');
 	card_atr_hex_len = strlen(card_atr_hex);
 
-	sc_log(ctx, "ATR     : %s", card_atr_hex);
+	sc_debug(ctx, SC_LOG_DEBUG_MATCH, "ATR     : %s", card_atr_hex);
 
 	for (i = 0; table[i].atr != NULL; i++) {
 		const char *tatr = table[i].atr;
@@ -1065,14 +1155,14 @@ static int match_atr_table(sc_context_t *ctx, struct sc_atr_table *table, struct
 		size_t fix_hex_len = card_atr_hex_len;
 		size_t fix_bin_len = card_atr_bin_len;
 
-		sc_log(ctx, "ATR try : %s", tatr);
+		sc_debug(ctx, SC_LOG_DEBUG_MATCH, "ATR try : %s", tatr);
 
 		if (tatr_len != fix_hex_len) {
-			sc_log(ctx, "ignored - wrong length");
+			sc_debug(ctx, SC_LOG_DEBUG_MATCH, "ignored - wrong length");
 			continue;
 		}
 		if (matr != NULL) {
-			sc_log(ctx, "ATR mask: %s", matr);
+			sc_debug(ctx, SC_LOG_DEBUG_MATCH, "ATR mask: %s", matr);
 
 			matr_len = strlen(matr);
 			if (tatr_len != matr_len)
@@ -1082,7 +1172,7 @@ static int match_atr_table(sc_context_t *ctx, struct sc_atr_table *table, struct
 			mbin_len = sizeof(mbin);
 			sc_hex_to_bin(matr, mbin, &mbin_len);
 			if (mbin_len != fix_bin_len) {
-				sc_log(ctx, "length of atr and atr mask do not match - ignored: %s - %s", tatr, matr);
+				sc_debug(ctx, SC_LOG_DEBUG_MATCH, "length of atr and atr mask do not match - ignored: %s - %s", tatr, matr);
 				continue;
 			}
 			for (s = 0; s < tbin_len; s++) {
@@ -1102,7 +1192,7 @@ static int match_atr_table(sc_context_t *ctx, struct sc_atr_table *table, struct
 	return -1;
 }
 
-int _sc_match_atr(sc_card_t *card, struct sc_atr_table *table, int *type_out)
+int _sc_match_atr(sc_card_t *card, const struct sc_atr_table *table, int *type_out)
 {
 	int res;
 
@@ -1233,7 +1323,16 @@ scconf_block *sc_get_conf_block(sc_context_t *ctx, const char *name1, const char
 	return conf_block;
 }
 
-void sc_print_cache(struct sc_card *card)   {
+void sc_invalidate_cache(struct sc_card *card)
+{
+	if (card) {
+		memset(&card->cache, 0, sizeof(card->cache));
+		card->cache.valid = 0;
+	}
+}
+
+void sc_print_cache(struct sc_card *card)
+{
 	struct sc_context *ctx = NULL;
 
 	if (card == NULL)
@@ -1312,6 +1411,8 @@ sc_card_sm_load(struct sc_card *card, const char *module_path, const char *in_mo
 	char temp_path[PATH_MAX];
 	size_t temp_len;
 	const char path_delim = '\\';
+	char expanded_val[PATH_MAX];
+	DWORD expanded_len;
 #else
 	const char path_delim = '/';
 #endif
@@ -1320,21 +1421,27 @@ sc_card_sm_load(struct sc_card *card, const char *module_path, const char *in_mo
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 	ctx = card->ctx;
-	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_NORMAL);
+	LOG_FUNC_CALLED(ctx);
 	if (!in_module)
 		return sc_card_sm_unload(card);
 
 #ifdef _WIN32
-	if (!module_path)   {
-		temp_len = PATH_MAX;
+	if (!module_path || strlen(module_path) == 0)   {
+		temp_len = PATH_MAX-1;
 		rv = sc_ctx_win32_get_config_value(NULL, "SmDir", "Software\\OpenSC Project\\OpenSC",
 				temp_path, &temp_len);
-		if (rv == SC_SUCCESS)
+		if (rv == SC_SUCCESS) {
+			temp_path[temp_len] = '\0';
 			module_path = temp_path;
+		}
 	}
+	expanded_len = PATH_MAX;
+	expanded_len = ExpandEnvironmentStringsA(module_path, expanded_val, expanded_len);
+	if (0 < expanded_len && expanded_len < sizeof expanded_val)
+		module_path = expanded_val;
 #endif
-	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "SM module '%s' located in '%s'", in_module, module_path);
-	if (module_path)   {
+	sc_log(ctx, "SM module '%s' located in '%s'", in_module, module_path);
+	if (module_path && strlen(module_path) > 0)   {
 		int sz = strlen(in_module) + strlen(module_path) + 3;
 		module = malloc(sz);
 		if (module)
@@ -1411,8 +1518,7 @@ sc_card_sm_check(struct sc_card *card)
 	scconf_block *atrblock = NULL, *sm_conf_block = NULL;
 	int rv, ii;
 
-	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_NORMAL);
-	sc_log(ctx, "card->sm_ctx.ops.open %p", card->sm_ctx.ops.open);
+	LOG_FUNC_CALLED(ctx);
 
 	/* get the name of card specific SM configuration section */
 	atrblock = _sc_match_atr_block(ctx, card->driver, &card->atr);
@@ -1440,8 +1546,8 @@ sc_card_sm_check(struct sc_card *card)
 		LOG_TEST_RET(ctx, SC_ERROR_INCONSISTENT_CONFIGURATION, "SM configuration block not preset");
 
 	/* check if an external SM module has to be used */
-	module_path = scconf_get_str(sm_conf_block, "module_path", NULL);
-	module_name = scconf_get_str(sm_conf_block, "module_name", NULL);
+	module_path = scconf_get_str(sm_conf_block, "module_path", DEFAULT_SM_MODULE_PATH);
+	module_name = scconf_get_str(sm_conf_block, "module_name", DEFAULT_SM_MODULE);
 	sc_log(ctx, "SM module '%s' in  '%s'", module_name, module_path);
 	if (!module_name)
 		LOG_TEST_RET(ctx, SC_ERROR_INCONSISTENT_CONFIGURATION, "Invalid SM configuration: module not defined");
@@ -1453,18 +1559,15 @@ sc_card_sm_check(struct sc_card *card)
 	strlcpy(card->sm_ctx.config_section, sm, sizeof(card->sm_ctx.config_section));
 
 	/* allocate resources for the external SM module */
-	sc_log(ctx, "'module_init' handler %p", card->sm_ctx.module.ops.module_init);
 	if (card->sm_ctx.module.ops.module_init)   {
 		module_data = scconf_get_str(sm_conf_block, "module_data", NULL);
-		sc_log(ctx, "module_data '%s'", module_data);
 
 		rv = card->sm_ctx.module.ops.module_init(ctx, module_data);
-		SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, rv, "Cannot initialize SM module");
+		LOG_TEST_RET(ctx, rv, "Cannot initialize SM module");
 	}
 
 	/* initialize SM session in the case of 'APDU TRANSMIT' SM mode */
 	sm_mode = scconf_get_str(sm_conf_block, "mode", NULL);
-	sc_log(ctx, "SM mode '%s'; 'open' handler %p", sm_mode, card->sm_ctx.ops.open);
 	if (sm_mode && !strcasecmp("Transmit", sm_mode))   {
 		if (!card->sm_ctx.ops.open || !card->sm_ctx.ops.get_sm_apdu || !card->sm_ctx.ops.free_sm_apdu)
 			LOG_TEST_RET(ctx, SC_ERROR_NOT_SUPPORTED, "'Transmit' SM asked but not supported by card driver");
