@@ -98,38 +98,40 @@ parse_x509_cert(sc_context_t *ctx, struct sc_pkcs15_der *der, struct sc_pkcs15_c
 	cert->data.len = data_len;
 
 	r = sc_asn1_decode(ctx, asn1_cert, obj, objlen, NULL, NULL);
-	LOG_TEST_RET(ctx, r, "ASN.1 parsing of certificate failed");
-
+	cert->key = pubkey;
 	cert->version++;
 
+	LOG_TEST_GOTO_ERR(ctx, r, "ASN.1 parsing of certificate failed");
+
 	if (!pubkey)
-		LOG_TEST_RET(ctx, SC_ERROR_INVALID_ASN1_OBJECT, "Unable to decode subjectPublicKeyInfo from cert");
-	cert->key = pubkey;
+		LOG_TEST_GOTO_ERR(ctx, SC_ERROR_INVALID_ASN1_OBJECT, "Unable to decode subjectPublicKeyInfo from cert");
 
 	sc_asn1_clear_algorithm_id(&sig_alg);
 
 	if (serial && serial_len)   {
 		sc_format_asn1_entry(asn1_serial_number + 0, serial, &serial_len, 1);
 		r = sc_asn1_encode(ctx, asn1_serial_number, &cert->serial, &cert->serial_len);
-		free(serial);
-		LOG_TEST_RET(ctx, r, "ASN.1 encoding of serial failed");
+		LOG_TEST_GOTO_ERR(ctx, r, "ASN.1 encoding of serial failed");
 	}
 
 	if (subject && subject_len)   {
 		sc_format_asn1_entry(asn1_subject + 0, subject, &subject_len, 1);
 		r = sc_asn1_encode(ctx, asn1_subject, &cert->subject, &cert->subject_len);
-		free(subject);
-		LOG_TEST_RET(ctx, r, "ASN.1 encoding of subject");
+		LOG_TEST_GOTO_ERR(ctx, r, "ASN.1 encoding of subject");
 	}
 
 	if (issuer && issuer_len)   {
 		sc_format_asn1_entry(asn1_issuer + 0, issuer, &issuer_len, 1);
 		r = sc_asn1_encode(ctx, asn1_issuer, &cert->issuer, &cert->issuer_len);
-		free(issuer);
-		LOG_TEST_RET(ctx, r, "ASN.1 encoding of issuer");
+		LOG_TEST_GOTO_ERR(ctx, r, "ASN.1 encoding of issuer");
 	}
 
-	return SC_SUCCESS;
+err:
+	free(serial);
+	free(subject);
+	free(issuer);
+
+	return r;
 }
 
 
@@ -159,7 +161,7 @@ sc_pkcs15_get_name_from_dn(struct sc_context *ctx, const u8 *dn, size_t dn_len,
 
 	rdn = sc_asn1_skip_tag(ctx, &dn, &dn_len, SC_ASN1_TAG_SEQUENCE | SC_ASN1_CONS, &rdn_len);
 	if (rdn == NULL)
-		LOG_TEST_RET(ctx, SC_ERROR_INVALID_ASN1_OBJECT, "ASN.1 decoding of Distiguished Name");
+		LOG_TEST_RET(ctx, SC_ERROR_INVALID_ASN1_OBJECT, "ASN.1 decoding of Distinguished Name");
 
 	for (next_ava = rdn, next_ava_len = rdn_len; next_ava_len; ) {
 		const u8 *ava, *dummy, *oidp;
@@ -220,7 +222,7 @@ sc_pkcs15_get_name_from_dn(struct sc_context *ctx, const u8 *dn, size_t dn_len,
  * The extension is identified by it's oid value.
  * NOTE: extensions can occur in any number or any order, which is why we
  *	can't parse them with a single pass of the asn1 decoder.
- * If is_critical is supplied, then it is set to 1 if the extention is critical
+ * If is_critical is supplied, then it is set to 1 if the extension is critical
  * and 0 if it is not.
  * The data in the extension is extension specific.
  * The following are common extension values:
@@ -282,8 +284,10 @@ sc_pkcs15_get_extension(struct sc_context *ctx, struct sc_pkcs15_cert *cert,
 			}
 			else {
 				*ext_val_len = MIN(*ext_val_len, val_len);
-				memcpy(*ext_val, val, *ext_val_len);
-				free(val);
+				if (val) {
+					memcpy(*ext_val, val, *ext_val_len);
+					free(val);
+				}
 			}
 
 			if (is_critical)
@@ -310,13 +314,13 @@ sc_pkcs15_get_extension(struct sc_context *ctx, struct sc_pkcs15_cert *cert,
 int
 sc_pkcs15_get_bitstring_extension(struct sc_context *ctx,
 	struct sc_pkcs15_cert *cert, const struct sc_object_id *type,
-	unsigned long long *value, int *is_critical)
+	unsigned int *value, int *is_critical)
 {
 	int r;
 	u8 *bit_string = NULL;
 	size_t bit_string_len=0, val_len = sizeof(*value);
 	struct sc_asn1_entry asn1_bit_string[] = {
-		{ "bitString", SC_ASN1_BIT_STRING, SC_ASN1_TAG_BIT_STRING, 0, value, &val_len },
+		{ "bitString", SC_ASN1_BIT_FIELD, SC_ASN1_TAG_BIT_STRING, 0, value, &val_len },
 		{ NULL, 0, 0, 0, NULL, NULL }
 	};
 
@@ -475,7 +479,11 @@ sc_pkcs15_decode_cdf_entry(struct sc_pkcs15_card *p15card, struct sc_pkcs15_obje
 		return r;
 	LOG_TEST_RET(ctx, r, "ASN.1 decoding failed");
 
-	if (!p15card->app || !p15card->app->ddo.aid.len)   {
+	if (!p15card->app || !p15card->app->ddo.aid.len) {
+		if (!p15card->file_app) {
+			free(der->value);
+			return SC_ERROR_INTERNAL;
+		}
 		r = sc_pkcs15_make_absolute_path(&p15card->file_app->path, &info.path);
 		LOG_TEST_RET(ctx, r, "Cannot make absolute path");
 	}
@@ -484,6 +492,17 @@ sc_pkcs15_decode_cdf_entry(struct sc_pkcs15_card *p15card, struct sc_pkcs15_obje
 	}
 	sc_log(ctx, "Certificate path '%s'", sc_print_path(&info.path));
 
+	switch (p15card->opts.private_certificate) {
+		case SC_PKCS15_CARD_OPTS_PRIV_CERT_DECLASSIFY:
+			sc_log(ctx, "Declassifying certificate");
+			obj->flags &= ~SC_PKCS15_CO_FLAG_PRIVATE;
+			break;
+		case SC_PKCS15_CARD_OPTS_PRIV_CERT_IGNORE:
+			sc_log(ctx, "Ignoring certificate");
+			free(der->value);
+			return 0;
+	}
+
 	obj->type = SC_PKCS15_TYPE_CERT_X509;
 	obj->data = malloc(sizeof(info));
 	if (obj->data == NULL)
@@ -539,8 +558,7 @@ sc_pkcs15_free_certificate(struct sc_pkcs15_cert *cert)
 		return;
 	}
 
-	if (cert->key)
-		sc_pkcs15_free_pubkey(cert->key);
+	sc_pkcs15_free_pubkey(cert->key);
 	free(cert->subject);
 	free(cert->issuer);
 	free(cert->serial);
@@ -555,7 +573,6 @@ sc_pkcs15_free_cert_info(sc_pkcs15_cert_info_t *cert)
 {
 	if (!cert)
 		return;
-	if (cert->value.value)
-		free(cert->value.value);
+	free(cert->value.value);
 	free(cert);
 }
