@@ -40,8 +40,6 @@
 
 #define MANU_ID		"piv_II "
 
-int sc_pkcs15emu_piv_init_ex(sc_pkcs15_card_t *, struct sc_aid *aid, sc_pkcs15emu_opt_t *);
-
 typedef struct objdata_st {
 	const char *id;
 	const char *label;
@@ -54,8 +52,8 @@ typedef struct objdata_st {
 typedef struct cdata_st {
 	const char *id;
 	const char *label;
-	int	    authority;
 	const char *path;
+	int	    authority;
 	int         obj_flags;
 } cdata;
 
@@ -332,30 +330,30 @@ static int sc_pkcs15emu_piv_init(sc_pkcs15_card_t *p15card)
 #define PIV_NUM_CERTS_AND_KEYS 24
 
 	static const cdata certs[PIV_NUM_CERTS_AND_KEYS] = {
-		{"01", "Certificate for PIV Authentication", 0, "0101cece", 0},
-		{"02", "Certificate for Digital Signature", 0, "0100cece", 0},
-		{"03", "Certificate for Key Management", 0, "0102cece", 0},
-		{"04", "Certificate for Card Authentication", 0, "0500cece", 0},
-		{"05", "Retired Certificate for Key Management 1", 0, "1001cece", 0},
-		{"06", "Retired Certificate for Key Management 2", 0, "1002cece", 0},
-		{"07", "Retired Certificate for Key Management 3", 0, "1003cece", 0},
-		{"08", "Retired Certificate for Key Management 4", 0, "1004cece", 0},
-		{"09", "Retired Certificate for Key Management 5", 0, "1005cece", 0},
-		{"10", "Retired Certificate for Key Management 6", 0, "1006cece", 0},
-		{"11", "Retired Certificate for Key Management 7", 0, "1007cece", 0},
-		{"12", "Retired Certificate for Key Management 8", 0, "1008cece", 0},
-		{"13", "Retired Certificate for Key Management 9", 0, "1009cece", 0},
-		{"14", "Retired Certificate for Key Management 10", 0, "100Acece", 0},
-		{"15", "Retired Certificate for Key Management 11", 0, "100Bcece", 0},
-		{"16", "Retired Certificate for Key Management 12", 0, "100Ccece", 0},
-		{"17", "Retired Certificate for Key Management 13", 0, "100Dcece", 0},
-		{"18", "Retired Certificate for Key Management 14", 0, "100Ecece", 0},
-		{"19", "Retired Certificate for Key Management 15", 0, "100Fcece", 0},
-		{"20", "Retired Certificate for Key Management 16", 0, "1010cece", 0},
-		{"21", "Retired Certificate for Key Management 17", 0, "1011cece", 0},
-		{"22", "Retired Certificate for Key Management 18", 0, "1012cece", 0},
-		{"23", "Retired Certificate for Key Management 19", 0, "1013cece", 0},
-		{"24", "Retired Certificate for Key Management 20", 0, "1014cece", 0}
+		{"01", "Certificate for PIV Authentication", "0101cece", 0, 0},
+		{"02", "Certificate for Digital Signature", "0100cece", 0, 0},
+		{"03", "Certificate for Key Management", "0102cece", 0, 0},
+		{"04", "Certificate for Card Authentication", "0500cece", 0, 0},
+		{"05", "Retired Certificate for Key Management 1", "1001cece", 0, 0},
+		{"06", "Retired Certificate for Key Management 2", "1002cece", 0, 0},
+		{"07", "Retired Certificate for Key Management 3", "1003cece", 0, 0},
+		{"08", "Retired Certificate for Key Management 4", "1004cece", 0, 0},
+		{"09", "Retired Certificate for Key Management 5", "1005cece", 0, 0},
+		{"10", "Retired Certificate for Key Management 6", "1006cece", 0, 0},
+		{"11", "Retired Certificate for Key Management 7", "1007cece", 0, 0},
+		{"12", "Retired Certificate for Key Management 8", "1008cece", 0, 0},
+		{"13", "Retired Certificate for Key Management 9", "1009cece", 0, 0},
+		{"14", "Retired Certificate for Key Management 10", "100Acece", 0, 0},
+		{"15", "Retired Certificate for Key Management 11", "100Bcece", 0, 0},
+		{"16", "Retired Certificate for Key Management 12", "100Ccece", 0, 0},
+		{"17", "Retired Certificate for Key Management 13", "100Dcece", 0, 0},
+		{"18", "Retired Certificate for Key Management 14", "100Ecece", 0, 0},
+		{"19", "Retired Certificate for Key Management 15", "100Fcece", 0, 0},
+		{"20", "Retired Certificate for Key Management 16", "1010cece", 0, 0},
+		{"21", "Retired Certificate for Key Management 17", "1011cece", 0, 0},
+		{"22", "Retired Certificate for Key Management 18", "1012cece", 0, 0},
+		{"23", "Retired Certificate for Key Management 19", "1013cece", 0, 0},
+		{"24", "Retired Certificate for Key Management 20", "1014cece", 0, 0}
 	};
 
 	static const pindata pins[] = {
@@ -634,7 +632,7 @@ static int sc_pkcs15emu_piv_init(sc_pkcs15_card_t *p15card)
 
 	r = sc_card_ctl(card, SC_CARDCTL_GET_SERIALNR, &serial);
 	if (r < 0) {
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,"sc_card_ctl rc=%d",r);
+		sc_log(card->ctx, "sc_card_ctl rc=%d",r);
 		p15card->tokeninfo->serial_number = strdup("00000000");
 	} else {
 		sc_bin_to_hex(serial.value, serial.len, buf, sizeof(buf), 0);
@@ -645,7 +643,7 @@ static int sc_pkcs15emu_piv_init(sc_pkcs15_card_t *p15card)
 	    follows_nist_fascn = 1;
 	}
 
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "PIV-II adding objects...");
+	sc_log(card->ctx,  "PIV-II adding objects...");
 
 	/* set other objects */
 	for (i = 0; objects[i].label; i++) {
@@ -676,7 +674,7 @@ static int sc_pkcs15emu_piv_init(sc_pkcs15_card_t *p15card)
 		r = sc_pkcs15emu_object_add(p15card, SC_PKCS15_TYPE_DATA_OBJECT, 
 			&obj_obj, &obj_info); 
 		if (r < 0)
-			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
+			LOG_FUNC_RETURN(card->ctx, r);
 /* TODO
  * PIV key 9C requires the pin verify be done just before any
  * crypto operation using the key. 
@@ -705,7 +703,7 @@ static int sc_pkcs15emu_piv_init(sc_pkcs15_card_t *p15card)
  
 	 */
 	/* set certs */
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "PIV-II adding certs...");
+	sc_log(card->ctx,  "PIV-II adding certs...");
 	for (i = 0; i < PIV_NUM_CERTS_AND_KEYS; i++) {
 		struct sc_pkcs15_cert_info cert_info;
 		struct sc_pkcs15_object    cert_obj;
@@ -736,14 +734,14 @@ static int sc_pkcs15emu_piv_init(sc_pkcs15_card_t *p15card)
 		/* See if the cert might be present or not. */
 		r = sc_card_ctl(card, SC_CARDCTL_PIV_OBJECT_PRESENT, &cert_info.path);
 		if (r == 1) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Cert can not be present,i=%d", i);
+			sc_log(card->ctx,  "Cert can not be present,i=%d", i);
 			continue;
 		}
 
 		r = sc_pkcs15_read_file(p15card, &cert_info.path, &cert_der.value, &cert_der.len);
 
 		if (r) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "No cert found,i=%d", i);
+			sc_log(card->ctx,  "No cert found,i=%d", i);
 			continue;
 		}
 
@@ -760,9 +758,10 @@ static int sc_pkcs15emu_piv_init(sc_pkcs15_card_t *p15card)
 		/* following will find the cached cert in cert_info */
 		r =  sc_pkcs15_read_certificate(p15card, &cert_info, &cert_out);
 		if (r < 0 || cert_out->key == NULL) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Failed to read/parse the certificate r=%d",r);
+			sc_log(card->ctx,  "Failed to read/parse the certificate r=%d",r);
 			if (cert_out != NULL)
 				sc_pkcs15_free_certificate(cert_out);
+			free(cert_der.value);
 			continue;
 		}
 
@@ -909,7 +908,7 @@ static int sc_pkcs15emu_piv_init(sc_pkcs15_card_t *p15card)
 				break;
 
 			default:
-				sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Unsupported key.algorithm %d", cert_out->key->algorithm);
+				sc_log(card->ctx,  "Unsupported key.algorithm %d", cert_out->key->algorithm);
 				ckis[i].pubkey_len = 0; /* set some value for now */
 		}
 		ckis[i].pubkey_from_cert = cert_out->key;
@@ -918,13 +917,13 @@ static int sc_pkcs15emu_piv_init(sc_pkcs15_card_t *p15card)
 
 		r = sc_pkcs15emu_add_x509_cert(p15card, &cert_obj, &cert_info);
 		if (r < 0) {
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, " Failed to add cert obj r=%d",r);
+			sc_log(card->ctx,  " Failed to add cert obj r=%d",r);
 			continue;
 		}
 	}
 
 	/* set pins */
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "PIV-II adding pins...");
+	sc_log(card->ctx,  "PIV-II adding pins...");
 	for (i = 0; pins[i].label; i++) {
 		struct sc_pkcs15_auth_info pin_info;
 		struct sc_pkcs15_object   pin_obj;
@@ -955,7 +954,7 @@ static int sc_pkcs15emu_piv_init(sc_pkcs15_card_t *p15card)
 			pin_info.attrs.pin.flags &= ~SC_PKCS15_PIN_FLAG_LOCAL;
 			label = "Global PIN";
 		}
-sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE Adding pin %d label=%s",i, label);
+sc_log(card->ctx,  "DEE Adding pin %d label=%s",i, label);
 		strncpy(pin_obj.label, label, SC_PKCS15_MAX_LABEL_SIZE - 1);
 		pin_obj.flags = pins[i].obj_flags;
 		if (i == 0 && pin_info.attrs.pin.reference == 0x80) {
@@ -969,7 +968,7 @@ sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE Adding pin %d label=%s",i, label);
 
 		r = sc_pkcs15emu_add_pin_obj(p15card, &pin_obj, &pin_info);
 		if (r < 0)
-			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
+			LOG_FUNC_RETURN(card->ctx, r);
 	}
 
 
@@ -979,11 +978,11 @@ sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE Adding pin %d label=%s",i, label);
 	 * gets the pubkey, but it can not be read from the card 
 	 * at a later time. The piv-tool can stash  pubkey in file
 	 */ 
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "PIV-II adding pub keys...");
+	sc_log(card->ctx,  "PIV-II adding pub keys...");
 	for (i = 0; i < PIV_NUM_CERTS_AND_KEYS; i++) {
 		struct sc_pkcs15_pubkey_info pubkey_info;
 		struct sc_pkcs15_object     pubkey_obj;
-		struct sc_pkcs15_pubkey *p15_key;
+		struct sc_pkcs15_pubkey *p15_key = NULL;
 
 		memset(&pubkey_info, 0, sizeof(pubkey_info));
 		memset(&pubkey_obj,  0, sizeof(pubkey_obj));
@@ -1012,7 +1011,7 @@ sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE Adding pin %d label=%s",i, label);
 		if (ckis[i].cert_found == 0 ) { /*  no cert found */
 			char * filename = NULL;
 			
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,"No cert for this pub key i=%d",i);
+			sc_log(card->ctx, "No cert for this pub key i=%d",i);
 			
 			/* 
 			 * If we used the piv-tool to generate a key,
@@ -1023,22 +1022,24 @@ sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE Adding pin %d label=%s",i, label);
 			 */
 	
 			
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,"DEE look for env %s", 
+			sc_log(card->ctx, "DEE look for env %s", 
 					pubkeys[i].getenvname?pubkeys[i].getenvname:"NULL");
 
 			if (pubkeys[i].getenvname == NULL) 
 				continue;
 
 			filename = getenv(pubkeys[i].getenvname); 
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,"DEE look for file %s", filename?filename:"NULL");
+			sc_log(card->ctx, "DEE look for file %s", filename?filename:"NULL");
 			if (filename == NULL)  
 				continue;
 			
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,"Adding pubkey from file %s",filename);
+			sc_log(card->ctx, "Adding pubkey from file %s",filename);
 
 			r = sc_pkcs15_pubkey_from_spki_file(card->ctx,  filename, &p15_key);
-			if (r < 0) 
+			if (r < 0) {
+				free(p15_key);
 				continue;
+			}
 
 			/* Lets also try another method. */
 			r = sc_pkcs15_encode_pubkey_as_spki(card->ctx, p15_key, &pubkey_info.direct.spki.value, &pubkey_info.direct.spki.len);
@@ -1069,7 +1070,7 @@ sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE Adding pin %d label=%s",i, label);
 					ckis[i].pubkey_from_file = 1;
 					break;
 				default:
-					sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,"Unsupported key_alg %d",p15_key->algorithm);
+					sc_log(card->ctx, "Unsupported key_alg %d",p15_key->algorithm);
 					continue;
 			}
 			pubkey_obj.emulated = p15_key;
@@ -1082,7 +1083,7 @@ sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE Adding pin %d label=%s",i, label);
 			pubkey_obj.emulated = ckis[i].pubkey_from_cert;
 		}
 
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,"adding pubkey for %d keyalg=%d",i, ckis[i].key_alg);
+		sc_log(card->ctx, "adding pubkey for %d keyalg=%d",i, ckis[i].key_alg);
 		switch (ckis[i].key_alg) {
 			case SC_ALGORITHM_RSA:
 				if (ckis[i].cert_keyUsage_present) {
@@ -1095,7 +1096,7 @@ sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE Adding pin %d label=%s",i, label);
 
 				r = sc_pkcs15emu_add_rsa_pubkey(p15card, &pubkey_obj, &pubkey_info);
 				if (r < 0)
-					SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r); /* should not fail */
+					LOG_FUNC_RETURN(card->ctx, r); /* should not fail */
 
 				ckis[i].pubkey_found = 1;
 				break;
@@ -1111,20 +1112,20 @@ sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE Adding pin %d label=%s",i, label);
 
 				r = sc_pkcs15emu_add_ec_pubkey(p15card, &pubkey_obj, &pubkey_info);
 				if (r < 0) 
-					SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r); /* should not fail */
+					LOG_FUNC_RETURN(card->ctx, r); /* should not fail */
 				ckis[i].pubkey_found = 1;
 				break;
 			default:
-				sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,"key_alg %d not supported", ckis[i].key_alg);
+				sc_log(card->ctx, "key_alg %d not supported", ckis[i].key_alg);
 				continue;
 		}
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,"USAGE: cert_keyUsage_present:%d usage:0x%8.8x",
+		sc_log(card->ctx, "USAGE: cert_keyUsage_present:%d usage:0x%8.8x",
 				ckis[i].cert_keyUsage_present ,pubkey_info.usage); 
 	}
 
 
 	/* set private keys */
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "PIV-II adding private keys...");
+	sc_log(card->ctx,  "PIV-II adding private keys...");
 	for (i = 0; i < PIV_NUM_CERTS_AND_KEYS; i++) {
 		struct sc_pkcs15_prkey_info prkey_info;
 		struct sc_pkcs15_object     prkey_obj;
@@ -1158,7 +1159,7 @@ sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE Adding pin %d label=%s",i, label);
 
 		if (ckis[i].pubkey_from_file == 1) {
 			prkey_info.usage = SC_PKCS15_PRKEY_USAGE_SIGN;
-			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Adding SC_PKCS15_PRKEY_USAGE_SIGN");
+			sc_log(card->ctx,  "Adding SC_PKCS15_PRKEY_USAGE_SIGN");
 		}
 
 		switch (ckis[i].key_alg) {
@@ -1186,38 +1187,33 @@ sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE Adding pin %d label=%s",i, label);
 					prkey_info.usage  |= prkeys[i].usage_ec;
 				}
 				prkey_info.field_length = ckis[i].pubkey_len;
-				sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "DEE added key_alg %2.2x prkey_obj.flags %8.8x",
+				sc_log(card->ctx,  "DEE added key_alg %2.2x prkey_obj.flags %8.8x",
 					 ckis[i].key_alg, prkey_obj.flags);
 				r = sc_pkcs15emu_add_ec_prkey(p15card, &prkey_obj, &prkey_info);
 				break;
 			default:
-				sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Unsupported key_alg %d", ckis[i].key_alg);
+				sc_log(card->ctx,  "Unsupported key_alg %d", ckis[i].key_alg);
 				r = 0; /* we just skip this one */
 		}
-		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,"USAGE: cert_keyUsage_present:%d usage:0x%8.8x", ckis[i].cert_keyUsage_present ,prkey_info.usage);
+		sc_log(card->ctx, "USAGE: cert_keyUsage_present:%d usage:0x%8.8x", ckis[i].cert_keyUsage_present ,prkey_info.usage);
 		if (r < 0)
-			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
+			LOG_FUNC_RETURN(card->ctx, r);
 	}
 
 	p15card->ops.get_guid = piv_get_guid;
 
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_SUCCESS);
+	LOG_FUNC_RETURN(card->ctx, SC_SUCCESS);
 }
 
 int sc_pkcs15emu_piv_init_ex(sc_pkcs15_card_t *p15card,
-		struct sc_aid *aid, sc_pkcs15emu_opt_t *opts)
+		struct sc_aid *aid)
 {
 	sc_card_t   *card = p15card->card;
 	sc_context_t    *ctx = card->ctx;
 
 	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_VERBOSE);
 
-	if (opts && opts->flags & SC_PKCS15EMU_FLAGS_NO_CHECK)
-		return sc_pkcs15emu_piv_init(p15card);
-	else {
-		int r = piv_detect_card(p15card);
-		if (r)
-			return SC_ERROR_WRONG_CARD;
-		return sc_pkcs15emu_piv_init(p15card);
-	}
+	if (piv_detect_card(p15card))
+		return SC_ERROR_WRONG_CARD;
+	return sc_pkcs15emu_piv_init(p15card);
 }
