@@ -37,12 +37,10 @@
 struct sc_pkcs15_emulator_handler builtin_emulators[] = {
 	{ "westcos",	sc_pkcs15emu_westcos_init_ex	},
 	{ "openpgp",	sc_pkcs15emu_openpgp_init_ex	},
-	{ "infocamere",	sc_pkcs15emu_infocamere_init_ex	},
 	{ "starcert",	sc_pkcs15emu_starcert_init_ex	},
 	{ "tcos",	sc_pkcs15emu_tcos_init_ex	},
 	{ "esteid",	sc_pkcs15emu_esteid_init_ex	},
 	{ "itacns",	sc_pkcs15emu_itacns_init_ex	},
-	{ "postecert",	sc_pkcs15emu_postecert_init_ex  },
 	{ "PIV-II",     sc_pkcs15emu_piv_init_ex	},
 	{ "cac",        sc_pkcs15emu_cac_init_ex	},
 	{ "gemsafeGPK",	sc_pkcs15emu_gemsafeGPK_init_ex	},
@@ -59,6 +57,9 @@ struct sc_pkcs15_emulator_handler builtin_emulators[] = {
 	{ "iasecc",	sc_pkcs15emu_iasecc_init_ex   },
 	{ "jpki",	sc_pkcs15emu_jpki_init_ex },
 	{ "coolkey",    sc_pkcs15emu_coolkey_init_ex	},
+	{ "din66291",    sc_pkcs15emu_din_66291_init_ex	},
+	{ "esteid2018",    sc_pkcs15emu_esteid2018_init_ex	},
+
 	{ NULL, NULL }
 };
 
@@ -70,17 +71,16 @@ static const char *builtin_name = "builtin";
 static const char *func_name    = "sc_pkcs15_init_func";
 static const char *exfunc_name  = "sc_pkcs15_init_func_ex";
 
-
+// FIXME: have a flag in card->flags to indicate the same
 int sc_pkcs15_is_emulation_only(sc_card_t *card)
 {
 	switch (card->type) {
-		case SC_CARD_TYPE_MCRD_ESTEID_V10:
-		case SC_CARD_TYPE_MCRD_ESTEID_V11:
 		case SC_CARD_TYPE_MCRD_ESTEID_V30:
 		case SC_CARD_TYPE_GEMSAFEV1_PTEID:
 		case SC_CARD_TYPE_OPENPGP_V1:
 		case SC_CARD_TYPE_OPENPGP_V2:
 		case SC_CARD_TYPE_OPENPGP_GNUK:
+		case SC_CARD_TYPE_OPENPGP_V3:
 		case SC_CARD_TYPE_SC_HSM:
 		case SC_CARD_TYPE_SC_HSM_SOC:
 		case SC_CARD_TYPE_DNIE_BASE:
@@ -89,6 +89,12 @@ int sc_pkcs15_is_emulation_only(sc_card_t *card)
 		case SC_CARD_TYPE_DNIE_USER:
 		case SC_CARD_TYPE_DNIE_TERMINATED:
 		case SC_CARD_TYPE_IASECC_GEMALTO:
+		case SC_CARD_TYPE_PIV_II_GENERIC:
+		case SC_CARD_TYPE_PIV_II_HIST:
+		case SC_CARD_TYPE_PIV_II_NEO:
+		case SC_CARD_TYPE_PIV_II_YUBIKEY4:
+		case SC_CARD_TYPE_ESTEID_2018:
+
 			return 1;
 		default:
 			return 0;
@@ -100,21 +106,19 @@ sc_pkcs15_bind_synthetic(sc_pkcs15_card_t *p15card, struct sc_aid *aid)
 {
 	sc_context_t		*ctx = p15card->card->ctx;
 	scconf_block		*conf_block, **blocks, *blk;
-	sc_pkcs15emu_opt_t	opts;
 	int			i, r = SC_ERROR_WRONG_CARD;
 
 	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_VERBOSE);
-	memset(&opts, 0, sizeof(opts));
 	conf_block = NULL;
 
 	conf_block = sc_get_conf_block(ctx, "framework", "pkcs15", 1);
 
 	if (!conf_block) {
-		/* no conf file found => try bultin drivers  */
+		/* no conf file found => try builtin drivers  */
 		sc_log(ctx, "no conf file (or section), trying all builtin emulators");
 		for (i = 0; builtin_emulators[i].name; i++) {
 			sc_log(ctx, "trying %s", builtin_emulators[i].name);
-			r = builtin_emulators[i].handler(p15card, aid, &opts);
+			r = builtin_emulators[i].handler(p15card, aid);
 			if (r == SC_SUCCESS)
 				/* we got a hit */
 				goto out;
@@ -136,7 +140,7 @@ sc_pkcs15_bind_synthetic(sc_pkcs15_card_t *p15card, struct sc_aid *aid)
 				sc_log(ctx, "trying %s", name);
 				for (i = 0; builtin_emulators[i].name; i++)
 					if (!strcmp(builtin_emulators[i].name, name)) {
-						r = builtin_emulators[i].handler(p15card, aid, &opts);
+						r = builtin_emulators[i].handler(p15card, aid);
 						if (r == SC_SUCCESS)
 							/* we got a hit */
 							goto out;
@@ -147,7 +151,7 @@ sc_pkcs15_bind_synthetic(sc_pkcs15_card_t *p15card, struct sc_aid *aid)
 			sc_log(ctx, "no emulator list in config file, trying all builtin emulators");
 			for (i = 0; builtin_emulators[i].name; i++) {
 				sc_log(ctx, "trying %s", builtin_emulators[i].name);
-				r = builtin_emulators[i].handler(p15card, aid, &opts);
+				r = builtin_emulators[i].handler(p15card, aid);
 				if (r == SC_SUCCESS)
 					/* we got a hit */
 					goto out;
@@ -188,10 +192,9 @@ static int parse_emu_block(sc_pkcs15_card_t *p15card, struct sc_aid *aid, scconf
 {
 	sc_card_t	*card = p15card->card;
 	sc_context_t	*ctx = card->ctx;
-	sc_pkcs15emu_opt_t opts;
 	void *handle = NULL;
 	int		(*init_func)(sc_pkcs15_card_t *);
-	int		(*init_func_ex)(sc_pkcs15_card_t *, struct sc_aid *, sc_pkcs15emu_opt_t *);
+	int		(*init_func_ex)(sc_pkcs15_card_t *, struct sc_aid *);
 	int		r;
 	const char	*driver, *module_name;
 
@@ -200,9 +203,6 @@ static int parse_emu_block(sc_pkcs15_card_t *p15card, struct sc_aid *aid, scconf
 	init_func    = NULL;
 	init_func_ex = NULL;
 
-	memset(&opts, 0, sizeof(opts));
-	opts.blk     = conf;
-
 	module_name = scconf_get_str(conf, "module", builtin_name);
 	if (!strcmp(module_name, "builtin")) {
 		int	i;
@@ -257,12 +257,12 @@ static int parse_emu_block(sc_pkcs15_card_t *p15card, struct sc_aid *aid, scconf
 
 			address = sc_dlsym(handle, name);
 			if (address)
-				init_func_ex = (int (*)(sc_pkcs15_card_t *, struct sc_aid *, sc_pkcs15emu_opt_t *)) address;
+				init_func_ex = (int (*)(sc_pkcs15_card_t *, struct sc_aid *)) address;
 		}
 	}
 	/* try to initialize the pkcs15 structures */
 	if (init_func_ex)
-		r = init_func_ex(p15card, aid, &opts);
+		r = init_func_ex(p15card, aid);
 	else if (init_func)
 		r = init_func(p15card);
 	else
