@@ -41,10 +41,6 @@
 #define TC_CARDOS_GLOBALPIN	0x3000
 #define TC_CARDOS_PIN_MASK	0x3000
 
-int sc_pkcs15emu_tccardos_init_ex(sc_pkcs15_card_t *p15card,
-				  struct sc_aid *,
-				  sc_pkcs15emu_opt_t *opts);
-
 static int read_file(struct sc_card *card, const char *file, u8 *buf,
 	size_t *len)
 {
@@ -219,7 +215,7 @@ static int parse_EF_CardInfo(sc_pkcs15_card_t *p15card)
 
 	/* read EF_CardInfo1 */
 	r = read_file(p15card->card, "3F001003b200", info1, &info1_len);
-	if (r != SC_SUCCESS)
+	if (r != SC_SUCCESS || info1_len < 4)
 		return SC_ERROR_WRONG_CARD;
 	/* read EF_CardInfo2 */
 	r = read_file(p15card->card, "3F001003b201", info2, &info2_len);
@@ -230,14 +226,19 @@ static int parse_EF_CardInfo(sc_pkcs15_card_t *p15card)
 		| (((unsigned int) info1[info1_len-2]) << 8)
 	   	| (((unsigned int) info1[info1_len-3]) << 16)
 	   	| (((unsigned int) info1[info1_len-4]) << 24);
-	sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+	sc_log(ctx, 
 		"found %d private keys\n", (int)key_num);
 	/* set p1 to the address of the first key descriptor */
 	offset = info1_len - 4 - key_num * 2;
-	if (offset >= sizeof info1)
+	if (offset >= info1_len)
 		return SC_ERROR_INVALID_DATA;
 	p1 = info1 + offset;
 	p2 = info2;
+
+	/* This is the minimum amount of data expected by the following code without
+	 * overunning the buffer without additional condition for cert_count == 4 */
+	if (info2_len < key_num * 14)
+		return SC_ERROR_INVALID_DATA;
 	for (i=0; i<key_num; i++) {
 		u8   pinId, keyId, cert_count;
 		int  ch_cert, ca_cert, r1_cert, r2_cert = 0;
@@ -304,10 +305,10 @@ static int sc_pkcs15_tccardos_init_func(sc_pkcs15_card_t *p15card)
 	int    r;
 	struct sc_path path;
 	struct sc_file *file = NULL;
-	u8     gdo[MAX_INFO1_SIZE];
 	char   hex_buf[256];
-	size_t gdo_len = MAX_INFO1_SIZE;
 	struct sc_card *card = p15card->card;
+	sc_serial_number_t iccsn;
+	iccsn.len = sizeof iccsn.value;
 
 	/* check if we have the correct card OS */
 	if (strcmp(card->name, "CardOS M4"))
@@ -329,10 +330,10 @@ static int sc_pkcs15_tccardos_init_func(sc_pkcs15_card_t *p15card)
 	if (p15card->tokeninfo->manufacturer_id == NULL)
 		return SC_ERROR_OUT_OF_MEMORY;
 	/* set the serial number */
-	r = read_file(p15card->card, "3F002F02", gdo, &gdo_len);
-	if (r != SC_SUCCESS)
+	r = sc_parse_ef_gdo(card, iccsn.value, &iccsn.len, NULL, 0);
+	if (r != SC_SUCCESS || iccsn.len < 5+8)
 		return SC_ERROR_INTERNAL;
-	sc_bin_to_hex(gdo + 7, 8, hex_buf, sizeof(hex_buf), 0);
+	sc_bin_to_hex(iccsn.value + 5, 8, hex_buf, sizeof(hex_buf), 0);
 	p15card->tokeninfo->serial_number = strdup(hex_buf);
 	if (p15card->tokeninfo->serial_number == NULL)
 		return SC_ERROR_OUT_OF_MEMORY;
@@ -350,8 +351,7 @@ static int sc_pkcs15_tccardos_init_func(sc_pkcs15_card_t *p15card)
 }
 
 int sc_pkcs15emu_tccardos_init_ex(sc_pkcs15_card_t *p15card,
-				  struct sc_aid *aid,
-				  sc_pkcs15emu_opt_t *opts)
+				  struct sc_aid *aid)
 {
 	return sc_pkcs15_tccardos_init_func(p15card);
 }
