@@ -31,8 +31,6 @@
 #include "cardctl.h"
 #include "common/compat_strlcpy.h"
 
-int sc_pkcs15emu_westcos_init_ex(sc_pkcs15_card_t *, struct sc_aid *, sc_pkcs15emu_opt_t *);
-
 static int sc_pkcs15emu_westcos_init(sc_pkcs15_card_t * p15card)
 {
 	int i, r;
@@ -54,6 +52,8 @@ static int sc_pkcs15emu_westcos_init(sc_pkcs15_card_t * p15card)
 
 	/* get serial number */
 	r = sc_card_ctl(card, SC_CARDCTL_GET_SERIALNR, &serial);
+	if (r)
+		goto out;
 	r = sc_bin_to_hex(serial.value, serial.len, buf, sizeof(buf), 0);
 	if (r)
 		goto out;
@@ -235,7 +235,7 @@ static int westcos_detect_card(sc_pkcs15_card_t * p15card)
 	sc_card_t *card = p15card->card;
 	sc_context_t *ctx = card->ctx;
 	const char *name = "WESTCOS";
-	sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+	sc_log(ctx, 
 		"westcos_detect_card (%s)", card->name);
 	if (strncmp(card->name, name, strlen(name)))
 		return SC_ERROR_WRONG_CARD;
@@ -243,16 +243,13 @@ static int westcos_detect_card(sc_pkcs15_card_t * p15card)
 }
 
 int sc_pkcs15emu_westcos_init_ex(sc_pkcs15_card_t * p15card,
-				 struct sc_aid *aid,
-				 sc_pkcs15emu_opt_t * opts)
+				 struct sc_aid *aid)
 {
 	int r;
 	sc_card_t *card = p15card->card;
 	sc_context_t *ctx = card->ctx;
-	sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+	sc_log(ctx, 
 		"sc_pkcs15_init_func_ex westcos\n");
-	if (opts && opts->flags & SC_PKCS15EMU_FLAGS_NO_CHECK)
-		return sc_pkcs15emu_westcos_init(p15card);
 	r = westcos_detect_card(p15card);
 	if (r)
 		return SC_ERROR_WRONG_CARD;
