@@ -42,40 +42,23 @@ extern "C" {
  */
 
 /*
- * EVP_CIPHER_CTX functions:
- * EVP_CIPHER_CTX_new	    not in 0.9.7
- * EVP_CIPHER_CTX_free	    not in 0.9.7
- * EVP_CIPHER_CTX_init	    in 0.9.7 to 1.0.2. defined in 1.1 as EVP_CIPHER_CTX_reset
- * EVP_CIPHER_CTX_cleanup   in 0.9.7 to 1.0.2, defined in 1.1 as EVP_CIPHER_CTX_reset
- * EVP_CIPHER_CTX_reset	    only in 1.1
- *
- * EVP_CIPHER_CTX_new	    does a EVP_CIPHER_CTX_init
- * EVP_CIPHER_CTX_free	    does a EVP_CIPHER_CTX_cleanup
- * EVP_CIPHER_CTX_cleanup   does equivalent of a EVP_CIPHER_CTX_init
- * Use EVP_CIPHER_CTX_new, EVP_CIPHER_CTX_free, and  EVP_CIPHER_CTX_cleanup between operations
+ * 1.1.0 depracated ERR_load_crypto_strings(), SSL_load_error_strings(), ERR_free_strings()
+ * and ENGINE_load_dynamic.EVP_CIPHER_CTX_cleanup and EVP_CIPHER_CTX_init are replaced
+ * by EVP_CIPHER_CTX_reset.
+ * But for compatability with LibreSSL and older OpenSSL. OpenSC uses the older functions
  */
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L  && !defined(LIBRESSL_VERSION_NUMBER)
+# if defined(OPENSSL_API_COMPAT) && OPENSSL_API_COMPAT >= 0x10100000L
+#define ERR_load_crypto_strings(x) while (0) continue
+#define SSL_load_error_strings(x)  while (0) continue
+#define ERR_free_strings(x)        while (0) continue
+#define ENGINE_load_dynamic(x)     while (0) continue
+#define EVP_CIPHER_CTX_cleanup(x) EVP_CIPHER_CTX_reset(x)
+#define EVP_CIPHER_CTX_init(x) EVP_CIPHER_CTX_reset(x)
+# endif
+#endif
 
-#if OPENSSL_VERSION_NUMBER  <= 0x009070dfL
-
-/* in 0.9.7  EVP_CIPHER_CTX was always allocated inline or in other structures */
-
-#define EVP_CIPHER_CTX_new() ({ \
-	EVP_CIPHER_CTX * tmp = NULL; \
-	tmp = OPENSSL_malloc(sizeof(struct evp_cipher_ctx_st)); \
-	if (tmp) { \
-	EVP_CIPHER_CTX_init(tmp); \
-	} \
-	tmp; \
-	})
-
-#define EVP_CIPHER_CTX_free(x) ({ \
-	if (x) { \
-		EVP_CIPHER_CTX_cleanup(x); \
-		OPENSSL_free(x); \
-	} \
-	})
-#endif /* OPENSSL_VERSION_NUMBER =< 0x00907000L */
-
+ 
 /*
  * 1.1 renames RSA_PKCS1_SSLeay to RSA_PKCS1_OpenSSL
  * use RSA_PKCS1_OpenSSL
@@ -90,17 +73,31 @@ extern "C" {
 
 #if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
 #define RSA_PKCS1_OpenSSL		RSA_PKCS1_SSLeay
-#define OPENSSL_malloc_init		CRYPTO_malloc_init
 
-#define EVP_PKEY_get0_RSA(x)		(x->pkey.rsa)
-#define EVP_PKEY_get0_DSA(x)		(x->pkey.dsa)
 #define X509_get_extension_flags(x)	(x->ex_flags)
 #define X509_get_key_usage(x)		(x->ex_kusage)
 #define X509_get_extended_key_usage(x)	(x->ex_xkusage)
-#define EVP_PKEY_up_ref(user_key)	CRYPTO_add(&user_key->references, 1, CRYPTO_LOCK_EVP_PKEY)
 #if !defined(LIBRESSL_VERSION_NUMBER) || LIBRESSL_VERSION_NUMBER < 0x2050300fL
 #define X509_up_ref(cert)		CRYPTO_add(&cert->references, 1, CRYPTO_LOCK_X509)
 #endif
+#if !defined(LIBRESSL_VERSION_NUMBER) || LIBRESSL_VERSION_NUMBER < 0x20700000L
+#define OPENSSL_malloc_init		CRYPTO_malloc_init
+#define EVP_PKEY_get0_RSA(x)		(x->pkey.rsa)
+#define EVP_PKEY_get0_EC_KEY(x)		(x->pkey.ec)
+#define EVP_PKEY_get0_DSA(x)		(x->pkey.dsa)
+#define EVP_PKEY_up_ref(user_key)	CRYPTO_add(&user_key->references, 1, CRYPTO_LOCK_EVP_PKEY)
+#define ASN1_STRING_get0_data(x)	ASN1_STRING_data(x)
+#endif
+#endif
+
+/* workaround unused value warning for a macro that does nothing */
+#if defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER >= 0x20700000L
+#define OPENSSL_malloc_init()
+#endif
+
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+#define EC_POINT_get_affine_coordinates_GFp     EC_POINT_get_affine_coordinates
+#define EC_POINT_set_affine_coordinates_GFp     EC_POINT_set_affine_coordinates
 #endif
 
 /*
@@ -110,7 +107,7 @@ extern "C" {
  * If that is not good enough, versions could be added to libopensc
  */
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || (defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER < 0x20700000L)
 /* based on OpenSSL-1.1.0 e_os2.h */
 /* sc_ossl_inline: portable inline definition usable in public headers */
 # if !defined(inline) && !defined(__cplusplus)
@@ -129,7 +126,7 @@ extern "C" {
 # endif
 #endif
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || (defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER < 0x2050300fL)
 
 #define RSA_bits(R) (BN_num_bits(R->n))
 
@@ -140,6 +137,9 @@ extern "C" {
 #ifndef OPENSSL_NO_DSA
 #include <openssl/dsa.h>
 #endif
+#ifndef OPENSSL_NO_EC
+#include <openssl/ecdsa.h>
+#endif
 
 #ifndef OPENSSL_NO_RSA
 static sc_ossl_inline int RSA_set0_key(RSA *r, BIGNUM *n, BIGNUM *e, BIGNUM *d)
@@ -239,6 +239,38 @@ static sc_ossl_inline void DSA_get0_key(const DSA *d, const BIGNUM **pub_key, co
 /* NOTE: DSA_set0_*  functions not defined because they are not currently used in OpenSC */
 #endif /* OPENSSL_NO_DSA */
 
+
+#ifndef OPENSSL_NO_EC
+static sc_ossl_inline int ECDSA_SIG_set0(ECDSA_SIG *sig, BIGNUM *r, BIGNUM *s)
+{
+    if (r == NULL || s == NULL)
+        return 0;
+    BN_clear_free(sig->r);
+    BN_clear_free(sig->s);
+    sig->r = r;
+    sig->s = s;
+    return 1;
+}
+#endif /* OPENSSL_NO_EC */
+
+static sc_ossl_inline int CRYPTO_secure_malloc_init(size_t size, int minsize)
+{
+    return 0;
+}
+
+static sc_ossl_inline int CRYPTO_secure_malloc_initialized()
+{
+    return 0;
+}
+
+static sc_ossl_inline void CRYPTO_secure_malloc_done()
+{
+}
+
+#else
+
+#include <openssl/crypto.h>
+
 #endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
 
 #ifdef __cplusplus
