@@ -40,7 +40,7 @@ struct signature_data {
 	struct hash_signature_info *info;
 	sc_pkcs11_operation_t *	md;
 	CK_BYTE			buffer[4096/8];
-	unsigned int		buffer_len;
+	unsigned int	buffer_len;
 };
 
 /*
@@ -95,7 +95,7 @@ sc_pkcs11_get_mechanism_list(struct sc_pkcs11_card *p11card,
 {
 	sc_pkcs11_mechanism_type_t *mt;
 	unsigned int n, count = 0;
-	int rv;
+	CK_RV rv;
 
 	if (!p11card)
 		return CKR_TOKEN_NOT_PRESENT;
@@ -166,7 +166,7 @@ sc_pkcs11_md_init(struct sc_pkcs11_session *session,
 	struct sc_pkcs11_card *p11card;
 	sc_pkcs11_operation_t *operation;
 	sc_pkcs11_mechanism_type_t *mt;
-	int rv;
+	CK_RV rv;
 
 	LOG_FUNC_CALLED(context);
 	if (!session || !session->slot || !(p11card = session->slot->p11card))
@@ -179,7 +179,7 @@ sc_pkcs11_md_init(struct sc_pkcs11_session *session,
 
 	rv = session_start_operation(session, SC_PKCS11_OPERATION_DIGEST, mt, &operation);
 	if (rv != CKR_OK)
-		LOG_FUNC_RETURN(context, rv);
+		LOG_FUNC_RETURN(context, (int) rv);
 
 	memcpy(&operation->mechanism, pMechanism, sizeof(CK_MECHANISM));
 
@@ -188,7 +188,7 @@ sc_pkcs11_md_init(struct sc_pkcs11_session *session,
 	if (rv != CKR_OK)
 		session_stop_operation(session, SC_PKCS11_OPERATION_DIGEST);
 
-	LOG_FUNC_RETURN(context, rv);
+	LOG_FUNC_RETURN(context, (int) rv);
 }
 
 CK_RV
@@ -196,7 +196,7 @@ sc_pkcs11_md_update(struct sc_pkcs11_session *session,
 			CK_BYTE_PTR pData, CK_ULONG ulDataLen)
 {
 	sc_pkcs11_operation_t *op;
-	int rv;
+	CK_RV rv;
 
 	rv = session_get_operation(session, SC_PKCS11_OPERATION_DIGEST, &op);
 	if (rv != CKR_OK)
@@ -208,7 +208,7 @@ done:
 	if (rv != CKR_OK)
 		session_stop_operation(session, SC_PKCS11_OPERATION_DIGEST);
 
-	LOG_FUNC_RETURN(context, rv);
+	LOG_FUNC_RETURN(context, (int) rv);
 }
 
 CK_RV
@@ -220,7 +220,7 @@ sc_pkcs11_md_final(struct sc_pkcs11_session *session,
 
 	rv = session_get_operation(session, SC_PKCS11_OPERATION_DIGEST, &op);
 	if (rv != CKR_OK)
-		LOG_FUNC_RETURN(context, rv);
+		LOG_FUNC_RETURN(context, (int) rv);
 
 	/* This is a request for the digest length */
 	if (pData == NULL)
@@ -231,7 +231,7 @@ sc_pkcs11_md_final(struct sc_pkcs11_session *session,
 		LOG_FUNC_RETURN(context,  pData == NULL ? CKR_OK : CKR_BUFFER_TOO_SMALL);
 
 	session_stop_operation(session, SC_PKCS11_OPERATION_DIGEST);
-	LOG_FUNC_RETURN(context, rv);
+	LOG_FUNC_RETURN(context, (int) rv);
 }
 
 /*
@@ -245,7 +245,7 @@ sc_pkcs11_sign_init(struct sc_pkcs11_session *session, CK_MECHANISM_PTR pMechani
 	struct sc_pkcs11_card *p11card;
 	sc_pkcs11_operation_t *operation;
 	sc_pkcs11_mechanism_type_t *mt;
-	int rv;
+	CK_RV rv;
 
 	LOG_FUNC_CALLED(context);
 	if (!session || !session->slot || !(p11card = session->slot->p11card))
@@ -262,16 +262,25 @@ sc_pkcs11_sign_init(struct sc_pkcs11_session *session, CK_MECHANISM_PTR pMechani
 	if (mt->key_type != key_type)
 		LOG_FUNC_RETURN(context, CKR_KEY_TYPE_INCONSISTENT);
 
+	if (pMechanism->pParameter &&
+	    pMechanism->ulParameterLen > sizeof(operation->mechanism_params))
+		LOG_FUNC_RETURN(context, CKR_ARGUMENTS_BAD);
+
 	rv = session_start_operation(session, SC_PKCS11_OPERATION_SIGN, mt, &operation);
 	if (rv != CKR_OK)
-		LOG_FUNC_RETURN(context, rv);
+		LOG_FUNC_RETURN(context, (int) rv);
 
 	memcpy(&operation->mechanism, pMechanism, sizeof(CK_MECHANISM));
+	if (pMechanism->pParameter) {
+		memcpy(&operation->mechanism_params, pMechanism->pParameter,
+		       pMechanism->ulParameterLen);
+		operation->mechanism.pParameter = &operation->mechanism_params;
+	}
 	rv = mt->sign_init(operation, key);
 	if (rv != CKR_OK)
 		session_stop_operation(session, SC_PKCS11_OPERATION_SIGN);
 
-	LOG_FUNC_RETURN(context, rv);
+	LOG_FUNC_RETURN(context, (int) rv);
 }
 
 CK_RV
@@ -279,12 +288,12 @@ sc_pkcs11_sign_update(struct sc_pkcs11_session *session,
 		      CK_BYTE_PTR pData, CK_ULONG ulDataLen)
 {
 	sc_pkcs11_operation_t *op;
-	int rv;
+	CK_RV rv;
 
 	LOG_FUNC_CALLED(context);
 	rv = session_get_operation(session, SC_PKCS11_OPERATION_SIGN, &op);
 	if (rv != CKR_OK)
-		LOG_FUNC_RETURN(context, rv);
+		LOG_FUNC_RETURN(context, (int) rv);
 
 	if (op->type->sign_update == NULL) {
 		rv = CKR_KEY_TYPE_INCONSISTENT;
@@ -297,7 +306,7 @@ done:
 	if (rv != CKR_OK)
 		session_stop_operation(session, SC_PKCS11_OPERATION_SIGN);
 
-	LOG_FUNC_RETURN(context, rv);
+	LOG_FUNC_RETURN(context, (int) rv);
 }
 
 CK_RV
@@ -305,12 +314,12 @@ sc_pkcs11_sign_final(struct sc_pkcs11_session *session,
 		     CK_BYTE_PTR pSignature, CK_ULONG_PTR pulSignatureLen)
 {
 	sc_pkcs11_operation_t *op;
-	int rv;
+	CK_RV rv;
 
 	LOG_FUNC_CALLED(context);
 	rv = session_get_operation(session, SC_PKCS11_OPERATION_SIGN, &op);
 	if (rv != CKR_OK)
-		LOG_FUNC_RETURN(context, rv);
+		LOG_FUNC_RETURN(context, (int) rv);
 
 	/* Bail out for signature mechanisms that don't do hashing */
 	if (op->type->sign_final == NULL) {
@@ -324,18 +333,18 @@ done:
 	if (rv != CKR_BUFFER_TOO_SMALL && pSignature != NULL)
 		session_stop_operation(session, SC_PKCS11_OPERATION_SIGN);
 
-	LOG_FUNC_RETURN(context, rv);
+	LOG_FUNC_RETURN(context, (int) rv);
 }
 
 CK_RV
 sc_pkcs11_sign_size(struct sc_pkcs11_session *session, CK_ULONG_PTR pLength)
 {
 	sc_pkcs11_operation_t *op;
-	int rv;
+	CK_RV rv;
 
 	rv = session_get_operation(session, SC_PKCS11_OPERATION_SIGN, &op);
 	if (rv != CKR_OK)
-		LOG_FUNC_RETURN(context, rv);
+		LOG_FUNC_RETURN(context, (int) rv);
 
 	/* Bail out for signature mechanisms that don't do hashing */
 	if (op->type->sign_size == NULL) {
@@ -349,7 +358,7 @@ done:
 	if (rv != CKR_OK)
 		session_stop_operation(session, SC_PKCS11_OPERATION_SIGN);
 
-	LOG_FUNC_RETURN(context, rv);
+	LOG_FUNC_RETURN(context, (int) rv);
 }
 
 /*
@@ -383,7 +392,17 @@ sc_pkcs11_signature_init(sc_pkcs11_operation_t *operation,
 		else  {
 			/* Mechanism recognised but cannot be performed by pkcs#15 card, or some general error. */
 			free(data);
-			LOG_FUNC_RETURN(context, rv);
+			LOG_FUNC_RETURN(context, (int) rv);
+		}
+	}
+
+	/* Validate the mechanism parameters */
+	if (key->ops->init_params) {
+		rv = key->ops->init_params(operation->session, &operation->mechanism);
+		if (rv != CKR_OK) {
+			/* Probably bad arguments */
+			free(data);
+			LOG_FUNC_RETURN(context, (int) rv);
 		}
 	}
 
@@ -402,7 +421,7 @@ sc_pkcs11_signature_init(sc_pkcs11_operation_t *operation,
 		if (rv != CKR_OK) {
 			sc_pkcs11_release_operation(&data->md);
 			free(data);
-			LOG_FUNC_RETURN(context, rv);
+			LOG_FUNC_RETURN(context, (int) rv);
 		}
 		data->info = info;
 	}
@@ -422,7 +441,7 @@ sc_pkcs11_signature_update(sc_pkcs11_operation_t *operation,
 	data = (struct signature_data *) operation->priv_data;
 	if (data->md) {
 		CK_RV rv = data->md->type->md_update(data->md, pPart, ulPartLen);
-		LOG_FUNC_RETURN(context, rv);
+		LOG_FUNC_RETURN(context, (int) rv);
 	}
 
 	/* This signature mechanism operates on the raw data */
@@ -430,7 +449,6 @@ sc_pkcs11_signature_update(sc_pkcs11_operation_t *operation,
 		LOG_FUNC_RETURN(context, CKR_DATA_LEN_RANGE);
 	memcpy(data->buffer + data->buffer_len, pPart, ulPartLen);
 	data->buffer_len += ulPartLen;
-	sc_log(context, "data length %u", data->buffer_len);
 	LOG_FUNC_RETURN(context, CKR_OK);
 }
 
@@ -443,7 +461,6 @@ sc_pkcs11_signature_final(sc_pkcs11_operation_t *operation,
 
 	LOG_FUNC_CALLED(context);
 	data = (struct signature_data *) operation->priv_data;
-	sc_log(context, "data length %u", data->buffer_len);
 	if (data->md) {
 		sc_pkcs11_operation_t	*md = data->md;
 		CK_ULONG len = sizeof(data->buffer);
@@ -452,14 +469,13 @@ sc_pkcs11_signature_final(sc_pkcs11_operation_t *operation,
 		if (rv == CKR_BUFFER_TOO_SMALL)
 			rv = CKR_FUNCTION_FAILED;
 		if (rv != CKR_OK)
-			LOG_FUNC_RETURN(context, rv);
-		data->buffer_len = len;
+			LOG_FUNC_RETURN(context, (int) rv);
+		data->buffer_len = (unsigned int) len;
 	}
 
-	sc_log(context, "%u bytes to sign", data->buffer_len);
 	rv = data->key->ops->sign(operation->session, data->key, &operation->mechanism,
 			data->buffer, data->buffer_len, pSignature, pulSignatureLen);
-	LOG_FUNC_RETURN(context, rv);
+	LOG_FUNC_RETURN(context, (int) rv);
 }
 
 static CK_RV
@@ -502,7 +518,7 @@ sc_pkcs11_signature_size(sc_pkcs11_operation_t *operation, CK_ULONG_PTR pLength)
 		}
 	}
 
-	LOG_FUNC_RETURN(context, rv);
+	LOG_FUNC_RETURN(context, (int) rv);
 }
 
 static void
@@ -530,7 +546,7 @@ sc_pkcs11_verif_init(struct sc_pkcs11_session *session, CK_MECHANISM_PTR pMechan
 	struct sc_pkcs11_card *p11card;
 	sc_pkcs11_operation_t *operation;
 	sc_pkcs11_mechanism_type_t *mt;
-	int rv;
+	CK_RV rv;
 
 	if (!session || !session->slot
 	 || !(p11card = session->slot->p11card))
@@ -550,6 +566,12 @@ sc_pkcs11_verif_init(struct sc_pkcs11_session *session, CK_MECHANISM_PTR pMechan
 		return rv;
 
 	memcpy(&operation->mechanism, pMechanism, sizeof(CK_MECHANISM));
+	if (pMechanism->pParameter) {
+		memcpy(&operation->mechanism_params, pMechanism->pParameter,
+			pMechanism->ulParameterLen);
+		operation->mechanism.pParameter = &operation->mechanism_params;
+	}
+
 	rv = mt->verif_init(operation, key);
 
 	if (rv != CKR_OK)
@@ -564,7 +586,7 @@ sc_pkcs11_verif_update(struct sc_pkcs11_session *session,
 		      CK_BYTE_PTR pData, CK_ULONG ulDataLen)
 {
 	sc_pkcs11_operation_t *op;
-	int rv;
+	CK_RV rv;
 
 	rv = session_get_operation(session, SC_PKCS11_OPERATION_VERIFY, &op);
 	if (rv != CKR_OK)
@@ -589,7 +611,7 @@ sc_pkcs11_verif_final(struct sc_pkcs11_session *session,
 		     CK_BYTE_PTR pSignature, CK_ULONG ulSignatureLen)
 {
 	sc_pkcs11_operation_t *op;
-	int rv;
+	CK_RV rv;
 
 	rv = session_get_operation(session, SC_PKCS11_OPERATION_VERIFY, &op);
 	if (rv != CKR_OK)
@@ -616,7 +638,7 @@ sc_pkcs11_verify_init(sc_pkcs11_operation_t *operation,
 {
 	struct hash_signature_info *info;
 	struct signature_data *data;
-	int rv;
+	CK_RV rv;
 
 	if (!(data = calloc(1, sizeof(*data))))
 		return CKR_HOST_MEMORY;
@@ -632,7 +654,17 @@ sc_pkcs11_verify_init(sc_pkcs11_operation_t *operation,
 		else {
 			/* Mechanism cannot be performed by pkcs#15 card, or some general error. */
 			free(data);
-			LOG_FUNC_RETURN(context, rv);
+			LOG_FUNC_RETURN(context, (int) rv);
+		}
+	}
+
+	/* Validate the mechanism parameters */
+	if (key->ops->init_params) {
+		rv = key->ops->init_params(operation->session, &operation->mechanism);
+		if (rv != CKR_OK) {
+			/* Probably bad arguments */
+			free(data);
+			LOG_FUNC_RETURN(context, (int) rv);
 		}
 	}
 
@@ -692,7 +724,7 @@ sc_pkcs11_verify_final(sc_pkcs11_operation_t *operation,
 	CK_ATTRIBUTE attr = {CKA_VALUE, NULL, 0};
 	CK_ATTRIBUTE attr_key_type = {CKA_KEY_TYPE, &key_type, sizeof(key_type)};
 	CK_ATTRIBUTE attr_key_params = {CKA_GOSTR3410_PARAMS, &params, sizeof(params)};
-	int rv;
+	CK_RV rv;
 
 	data = (struct signature_data *) operation->priv_data;
 
@@ -727,10 +759,10 @@ sc_pkcs11_verify_final(sc_pkcs11_operation_t *operation,
 			goto done;
 	}
 
-	rv = sc_pkcs11_verify_data(pubkey_value, attr.ulValueLen,
+	rv = sc_pkcs11_verify_data(pubkey_value, (unsigned int) attr.ulValueLen,
 		params, sizeof(params),
-		operation->mechanism.mechanism, data->md,
-		data->buffer, data->buffer_len, pSignature, ulSignatureLen);
+		&operation->mechanism, data->md,
+		data->buffer, data->buffer_len, pSignature, (unsigned int) ulSignatureLen);
 
 done:
 	free(pubkey_value);
@@ -772,6 +804,11 @@ sc_pkcs11_decr_init(struct sc_pkcs11_session *session,
 		return rv;
 
 	memcpy(&operation->mechanism, pMechanism, sizeof(CK_MECHANISM));
+	if (pMechanism->pParameter) {
+		memcpy(&operation->mechanism_params, pMechanism->pParameter,
+		       pMechanism->ulParameterLen);
+		operation->mechanism.pParameter = &operation->mechanism_params;
+	}
 	rv = mt->decrypt_init(operation, key);
 
 	if (rv != CKR_OK)
@@ -786,7 +823,7 @@ sc_pkcs11_decr(struct sc_pkcs11_session *session,
 		CK_BYTE_PTR pData, CK_ULONG_PTR pulDataLen)
 {
 	sc_pkcs11_operation_t *op;
-	int rv;
+	CK_RV rv;
 
 	rv = session_get_operation(session, SC_PKCS11_OPERATION_DECRYPT, &op);
 	if (rv != CKR_OK)
@@ -801,6 +838,104 @@ sc_pkcs11_decr(struct sc_pkcs11_session *session,
 	return rv;
 }
 
+CK_RV
+sc_pkcs11_wrap(struct sc_pkcs11_session *session,
+	CK_MECHANISM_PTR pMechanism,
+	struct sc_pkcs11_object *wrappingKey,	/* wrapping key */
+	CK_KEY_TYPE key_type,			/* type of the wrapping key */
+	struct sc_pkcs11_object *targetKey,	/* the key to be wrapped */
+	CK_BYTE_PTR wrappedData,
+	CK_ULONG_PTR wrappedDataLen)
+{
+	struct sc_pkcs11_card *p11card;
+	sc_pkcs11_operation_t *operation;
+	sc_pkcs11_mechanism_type_t *mt;
+	CK_RV rv;
+
+	if (!session || !session->slot || !(p11card = session->slot->p11card))
+		return CKR_ARGUMENTS_BAD;
+
+	/* See if we support this mechanism type */
+	mt = sc_pkcs11_find_mechanism(p11card, pMechanism->mechanism, CKF_UNWRAP);
+	if (mt == NULL)
+		return CKR_MECHANISM_INVALID;
+
+	/* See if compatible with key type */
+	/* TODO: what if there are several mechanisms with different key types? Should we loop through them? */
+	if (mt->key_type != key_type)
+		return CKR_KEY_TYPE_INCONSISTENT;
+
+	rv = session_start_operation(session, SC_PKCS11_OPERATION_WRAP, mt, &operation);
+	if (rv != CKR_OK)
+		return rv;
+
+	memcpy(&operation->mechanism, pMechanism, sizeof(CK_MECHANISM));
+
+	rv = operation->type->wrap(operation, wrappingKey,
+			targetKey, wrappedData,
+			wrappedDataLen);
+
+	session_stop_operation(session, SC_PKCS11_OPERATION_WRAP);
+
+	return rv;
+}
+
+/*
+ * Unwrap a wrapped key into card. A new key object is created on card.
+ */
+CK_RV
+sc_pkcs11_unwrap(struct sc_pkcs11_session *session,
+	CK_MECHANISM_PTR pMechanism,
+	struct sc_pkcs11_object *unwrappingKey,
+	CK_KEY_TYPE key_type, /* type of the unwrapping key */
+	CK_BYTE_PTR pWrappedKey,	/* the wrapped key */
+	CK_ULONG ulWrappedKeyLen,	/* bytes length of wrapped key */
+	struct sc_pkcs11_object *targetKey)
+{
+	struct sc_pkcs11_card *p11card;
+	sc_pkcs11_operation_t *operation;
+	sc_pkcs11_mechanism_type_t *mt;
+
+	CK_RV rv;
+
+	if (!session || !session->slot || !(p11card = session->slot->p11card))
+		return CKR_ARGUMENTS_BAD;
+
+	/* See if we support this mechanism type */
+	mt = sc_pkcs11_find_mechanism(p11card, pMechanism->mechanism, CKF_UNWRAP);
+	if (mt == NULL)
+		return CKR_MECHANISM_INVALID;
+
+	/* See if compatible with key type */
+	/* TODO: what if there are several mechanisms with different key types? Should we loop through them? */
+	if (mt->key_type != key_type)
+		return CKR_KEY_TYPE_INCONSISTENT;
+
+	rv = session_start_operation(session, SC_PKCS11_OPERATION_UNWRAP, mt, &operation);
+	if (rv != CKR_OK)
+		return rv;
+
+	memcpy(&operation->mechanism, pMechanism, sizeof(CK_MECHANISM));
+
+
+	/*
+	 *  TODO: does it make sense to support unwrapping to an in memory key object?
+	 *  This implementation assumes that the key should be unwrapped into a
+	 *  key object on card, regardless whether CKA_TOKEN = FALSE
+	 *  CKA_TOKEN = FALSE is considered an on card session object.
+	 */
+
+	rv = operation->type->unwrap(operation, unwrappingKey,
+			pWrappedKey, ulWrappedKeyLen,
+			targetKey);
+
+	session_stop_operation(session, SC_PKCS11_OPERATION_UNWRAP);
+
+	return rv;
+}
+
+
+
 /* Derive one key from another, and return results in created object */
 CK_RV
 sc_pkcs11_deri(struct sc_pkcs11_session *session,
@@ -847,7 +982,7 @@ sc_pkcs11_deri(struct sc_pkcs11_session *session,
 	/* Get the size of the data to be returned
 	 * If the card could derive a key an leave it on the card
 	 * then no data is returned.
-	 * If the card returns the data, we will store it in the sercet key CKA_VALUE
+	 * If the card returns the data, we will store it in the secret key CKA_VALUE
 	 */
 
 	ulDataLen = 0;
@@ -867,7 +1002,7 @@ sc_pkcs11_deri(struct sc_pkcs11_session *session,
 		goto out;
 	}
 
-	/* Now do the actuall derivation */
+	/* Now do the actual derivation */
 
 	rv = operation->type->derive(operation, basekey,
 	    pMechanism->pParameter, pMechanism->ulParameterLen,
@@ -922,7 +1057,7 @@ sc_pkcs11_decrypt_init(sc_pkcs11_operation_t *operation,
 		else {
 			/* Mechanism cannot be performed by pkcs#15 card, or some general error. */
 			free(data);
-			LOG_FUNC_RETURN(context, rv);
+			LOG_FUNC_RETURN(context, (int) rv);
 		}
 	}
 
@@ -961,6 +1096,39 @@ sc_pkcs11_derive(sc_pkcs11_operation_t *operation,
 		    pData, pulDataLen);
 }
 
+
+static CK_RV
+sc_pkcs11_wrap_operation(sc_pkcs11_operation_t *operation,
+	    struct sc_pkcs11_object *wrappingKey,
+	    struct sc_pkcs11_object *targetKey,
+	    CK_BYTE_PTR pWrappedData, CK_ULONG_PTR ulWrappedDataLen)
+{
+	if (!operation || !wrappingKey || !wrappingKey->ops || !wrappingKey->ops->wrap_key)
+		return CKR_ARGUMENTS_BAD;
+
+    return wrappingKey->ops->wrap_key(operation->session,
+		    wrappingKey,
+		    &operation->mechanism,
+		    targetKey, pWrappedData,
+		    ulWrappedDataLen);
+}
+
+static CK_RV
+sc_pkcs11_unwrap_operation(sc_pkcs11_operation_t *operation,
+	    struct sc_pkcs11_object *unwrappingKey,
+	    CK_BYTE_PTR pWrappedKey, CK_ULONG ulWrappedKeyLen,
+	    struct sc_pkcs11_object *targetKey)
+{
+	if (!operation || !unwrappingKey || !unwrappingKey->ops || !unwrappingKey->ops->unwrap_key)
+		return CKR_ARGUMENTS_BAD;
+
+    return unwrappingKey->ops->unwrap_key(operation->session,
+		    unwrappingKey,
+		    &operation->mechanism,
+		    pWrappedKey, ulWrappedKeyLen,
+		    targetKey);
+}
+
 /*
  * Create new mechanism type for a mechanism supported by
  * the card
@@ -997,8 +1165,11 @@ sc_pkcs11_new_fw_mechanism(CK_MECHANISM_TYPE mech,
 		mt->verif_final = sc_pkcs11_verify_final;
 #endif
 	}
+	if (pInfo->flags & CKF_WRAP) {
+		mt->wrap = sc_pkcs11_wrap_operation;
+	}
 	if (pInfo->flags & CKF_UNWRAP) {
-		/* TODO */
+		mt->unwrap = sc_pkcs11_unwrap_operation;
 	}
 	if (pInfo->flags & CKF_DERIVE) {
 		mt->derive = sc_pkcs11_derive;
@@ -1041,6 +1212,7 @@ sc_pkcs11_register_sign_and_hash_mechanism(struct sc_pkcs11_card *p11card,
 	sc_pkcs11_mechanism_type_t *hash_type, *new_type;
 	struct hash_signature_info *info;
 	CK_MECHANISM_INFO mech_info = sign_type->mech_info;
+	CK_RV rv;
 
 	if (!(hash_type = sc_pkcs11_find_mechanism(p11card, hash_mech, CKF_DIGEST)))
 		return CKR_MECHANISM_INVALID;
@@ -1050,7 +1222,7 @@ sc_pkcs11_register_sign_and_hash_mechanism(struct sc_pkcs11_card *p11card,
 
 	info = calloc(1, sizeof(*info));
 	if (!info)
-		LOG_FUNC_RETURN(p11card->card->ctx, SC_ERROR_OUT_OF_MEMORY);
+		return CKR_HOST_MEMORY;
 
 	info->mech = mech;
 	info->sign_type = sign_type;
@@ -1059,8 +1231,16 @@ sc_pkcs11_register_sign_and_hash_mechanism(struct sc_pkcs11_card *p11card,
 	info->hash_mech = hash_mech;
 
 	new_type = sc_pkcs11_new_fw_mechanism(mech, &mech_info, sign_type->key_type, info, free_info);
-
-	if (!new_type)
+	if (!new_type) {
+		free_info(info);
 		return CKR_HOST_MEMORY;
-	return sc_pkcs11_register_mechanism(p11card, new_type);
+	}
+
+	rv = sc_pkcs11_register_mechanism(p11card, new_type);
+	if (CKR_OK != rv) {
+		new_type->free_mech_data(new_type->mech_data);
+		free(new_type);
+	}
+
+	return rv;
 }
