@@ -43,16 +43,26 @@ static struct sc_to_cryptoki_error_conversion sc_to_cryptoki_error_map[]  = {
 
 void strcpy_bp(u8 * dst, const char *src, size_t dstsize)
 {
-	size_t c;
-
-	if (!dst || !src || !dstsize)
+	if (!dst || !dstsize)
 		return;
 
 	memset((char *)dst, ' ', dstsize);
 
-	c = strlen(src) > dstsize ? dstsize : strlen(src);
+	if (src) {
+		size_t src_len = strlen(src);
 
-	memcpy((char *)dst, src, c);
+		if (src_len > dstsize) {
+			/* string will be truncated */
+			memcpy((char *)dst, src, dstsize);
+			if (dstsize > 3) {
+				/* show truncation with '...' */
+				/* FIXME avoid breaking an UTF-8 character on multiple bytes */
+				memset((char *)dst + dstsize - 3, '.', 3);
+			}
+		} else {
+			memcpy((char *)dst, src, src_len);
+		}
+	}
 }
 
 
@@ -140,7 +150,7 @@ CK_RV restore_login_state(struct sc_pkcs11_slot *slot)
 	if (sc_pkcs11_conf.atomic && slot) {
 		if (list_iterator_start(&slot->logins)) {
 			struct sc_pkcs11_login *login = list_iterator_next(&slot->logins);
-			while (login) {
+			while (login && slot->p11card && slot->p11card->framework) {
 				r = slot->p11card->framework->login(slot, login->userType,
 						login->pPin, login->ulPinLen);
 				if (r != CKR_OK)
@@ -187,7 +197,7 @@ CK_RV push_login_state(struct sc_pkcs11_slot *slot,
 	}
 
 	if (pPin && ulPinLen) {
-		login->pPin = sc_mem_alloc_secure(context, (sizeof *pPin)*ulPinLen);
+		login->pPin = sc_mem_secure_alloc((sizeof *pPin)*ulPinLen);
 		if (login->pPin == NULL) {
 			goto err;
 		}
@@ -207,7 +217,7 @@ err:
 	if (login) {
 		if (login->pPin) {
 			sc_mem_clear(login->pPin, login->ulPinLen);
-			free(login->pPin);
+			sc_mem_secure_free(login->pPin, login->ulPinLen);
 		}
 		free(login);
 	}
@@ -223,7 +233,7 @@ void pop_login_state(struct sc_pkcs11_slot *slot)
 			struct sc_pkcs11_login *login = list_get_at(&slot->logins, size-1);
 			if (login) {
 				sc_mem_clear(login->pPin, login->ulPinLen);
-				free(login->pPin);
+				sc_mem_secure_free(login->pPin, login->ulPinLen);
 				free(login);
 			}
 			if (0 > list_delete_at(&slot->logins, size-1))
@@ -238,7 +248,7 @@ void pop_all_login_states(struct sc_pkcs11_slot *slot)
 		struct sc_pkcs11_login *login = list_fetch(&slot->logins);
 		while (login) {
 			sc_mem_clear(login->pPin, login->ulPinLen);
-			free(login->pPin);
+			sc_mem_secure_free(login->pPin, login->ulPinLen);
 			free(login);
 			login = list_fetch(&slot->logins);
 		}
@@ -305,7 +315,7 @@ CK_RV session_stop_operation(struct sc_pkcs11_session * session, int type)
 
 CK_RV attr_extract(CK_ATTRIBUTE_PTR pAttr, void *ptr, size_t * sizep)
 {
-	unsigned int size;
+	size_t size;
 
 	if (sizep) {
 		size = *sizep;
@@ -452,13 +462,11 @@ void load_pkcs11_parameters(struct sc_pkcs11_config *conf, sc_context_t * ctx)
 	} else {
 		conf->slots_per_card = 4;
 	}
-	conf->hide_empty_tokens = 1;
 	conf->atomic = 0;
 	conf->lock_login = 0;
 	conf->init_sloppy = 1;
 	conf->pin_unblock_style = SC_PKCS11_PIN_UNBLOCK_NOT_ALLOWED;
 	conf->create_puk_slot = 0;
-	conf->zero_ckaid_for_ca_certs = 0;
 	conf->create_slots_flags = SC_PKCS11_SLOT_CREATE_ALL;
 
 	conf_block = sc_get_conf_block(ctx, "pkcs11", NULL, 1);
@@ -468,7 +476,6 @@ void load_pkcs11_parameters(struct sc_pkcs11_config *conf, sc_context_t * ctx)
 	/* contains the defaults, if there is a "pkcs11" config block */
 	conf->max_virtual_slots = scconf_get_int(conf_block, "max_virtual_slots", conf->max_virtual_slots);
 	conf->slots_per_card = scconf_get_int(conf_block, "slots_per_card", conf->slots_per_card);
-	conf->hide_empty_tokens = scconf_get_bool(conf_block, "hide_empty_tokens", conf->hide_empty_tokens);
 	conf->atomic = scconf_get_bool(conf_block, "atomic", conf->atomic);
 	if (conf->atomic)
 		conf->lock_login = 1;
@@ -484,7 +491,6 @@ void load_pkcs11_parameters(struct sc_pkcs11_config *conf, sc_context_t * ctx)
 		conf->pin_unblock_style = SC_PKCS11_PIN_UNBLOCK_SO_LOGGED_INITPIN;
 
 	conf->create_puk_slot = scconf_get_bool(conf_block, "create_puk_slot", conf->create_puk_slot);
-	conf->zero_ckaid_for_ca_certs = scconf_get_bool(conf_block, "zero_ckaid_for_ca_certs", conf->zero_ckaid_for_ca_certs);
 
 	create_slots_for_pins = (char *)scconf_get_str(conf_block, "create_slots_for_pins", "all");
 	conf->create_slots_flags = 0;
@@ -499,12 +505,12 @@ void load_pkcs11_parameters(struct sc_pkcs11_config *conf, sc_context_t * ctx)
 			conf->create_slots_flags |= SC_PKCS11_SLOT_CREATE_ALL;
 		op = strtok(NULL, " ,");
 	}
-        free(tmp);
+	free(tmp);
 
 	sc_log(ctx, "PKCS#11 options: max_virtual_slots=%d slots_per_card=%d "
-		 "hide_empty_tokens=%d lock_login=%d atomic=%d pin_unblock_style=%d "
-		 "zero_ckaid_for_ca_certs=%d create_slots_flags=0x%X",
+		 "lock_login=%d atomic=%d pin_unblock_style=%d "
+		 "create_slots_flags=0x%X",
 		 conf->max_virtual_slots, conf->slots_per_card,
-		 conf->hide_empty_tokens, conf->lock_login, conf->atomic, conf->pin_unblock_style,
-		 conf->zero_ckaid_for_ca_certs, conf->create_slots_flags);
+		 conf->lock_login, conf->atomic, conf->pin_unblock_style,
+		 conf->create_slots_flags);
 }
