@@ -35,6 +35,12 @@
 #endif /* PKCS11_THREAD_LOCKING */
 
 #include "sc-pkcs11.h"
+#include "ui/notify.h"
+
+#include "libopensc/sc-ossl-compat.h"
+#ifdef ENABLE_OPENPACE
+#include <eac/eac.h>
+#endif
 
 #ifndef MODULE_APP_NAME
 #define MODULE_APP_NAME "opensc-pkcs11"
@@ -209,7 +215,52 @@ static int slot_list_seeker(const void *el, const void *key) {
 	return 0;
 }
 
+#ifndef _WIN32
+__attribute__((constructor))
+#endif
+int module_init()
+{
+	sc_notify_init();
+	return 1;
+}
 
+#ifndef _WIN32
+__attribute__((destructor))
+#endif
+int module_close()
+{
+	sc_notify_close();
+#if defined(ENABLE_OPENSSL) && defined(OPENSSL_SECURE_MALLOC_SIZE)
+	CRYPTO_secure_malloc_done();
+#endif
+#ifdef ENABLE_OPENPACE
+	EAC_cleanup();
+#endif
+	return 1;
+}
+
+#ifdef _WIN32
+BOOL APIENTRY DllMain( HINSTANCE hinstDLL,
+	DWORD  ul_reason_for_call,
+	LPVOID lpReserved
+)
+{
+	switch (ul_reason_for_call)
+	{
+	case DLL_PROCESS_ATTACH:
+		if (!module_init())
+			return FALSE;
+		break;
+	case DLL_PROCESS_DETACH:
+		if (lpReserved == NULL) {
+			if (!module_close())
+				return FALSE;
+		}
+		break;
+	}
+	return TRUE;
+}
+#endif
 
 CK_RV C_Initialize(CK_VOID_PTR pInitArgs)
 {
@@ -257,11 +308,17 @@ CK_RV C_Initialize(CK_VOID_PTR pInitArgs)
 	load_pkcs11_parameters(&sc_pkcs11_conf, context);
 
 	/* List of sessions */
-	list_init(&sessions);
+	if (0 != list_init(&sessions)) {
+		rv = CKR_HOST_MEMORY;
+		goto out;
+	}
 	list_attributes_seeker(&sessions, session_list_seeker);
 
 	/* List of slots */
-	list_init(&virtual_slots);
+	if (0 != list_init(&virtual_slots)) {
+		rv = CKR_HOST_MEMORY;
+		goto out;
+	}
 	list_attributes_seeker(&virtual_slots, slot_list_seeker);
 
 	/* Create slots for readers found on initialization, only if in 2.11 mode */
@@ -294,6 +351,10 @@ CK_RV C_Finalize(CK_VOID_PTR pReserved)
 	if (pReserved != NULL_PTR)
 		return CKR_ARGUMENTS_BAD;
 
+#if !defined(_WIN32)
+	sc_notify_close();
+#endif
+
 	if (context == NULL)
 		return CKR_CRYPTOKI_NOT_INITIALIZED;
 
@@ -316,7 +377,6 @@ CK_RV C_Finalize(CK_VOID_PTR pReserved)
 
 	while ((slot = list_fetch(&virtual_slots))) {
 		list_destroy(&slot->objects);
-		pop_all_login_states(slot);
 		list_destroy(&slot->logins);
 		free(slot);
 	}
@@ -391,8 +451,13 @@ CK_RV C_GetSlotList(CK_BBOOL       tokenPresent,  /* only slots with token prese
 			pSlotList==NULL_PTR? "plug-n-play":"refresh");
 
 	/* Slot list can only change in v2.20 */
-	if (pSlotList == NULL_PTR)
+	if (pSlotList == NULL_PTR) {
 		sc_ctx_detect_readers(context);
+		for (i=0; i<list_size(&virtual_slots); i++) {
+			slot = (sc_pkcs11_slot_t *) list_get_at(&virtual_slots, i);
+			slot->flags &= ~SC_PKCS11_SLOT_FLAG_SEEN;
+		}
+	}
 
 	card_detect_all();
 
@@ -406,28 +471,47 @@ CK_RV C_GetSlotList(CK_BBOOL       tokenPresent,  /* only slots with token prese
 	prev_reader = NULL;
 	numMatches = 0;
 	for (i=0; i<list_size(&virtual_slots); i++) {
-	        slot = (sc_pkcs11_slot_t *) list_get_at(&virtual_slots, i);
+		slot = (sc_pkcs11_slot_t *) list_get_at(&virtual_slots, i);
 		/* the list of available slots contains:
 		 * - if present, virtual hotplug slot;
 		 * - any slot with token;
 		 * - without token(s), one empty slot per reader;
+		 * - any slot that has already been seen;
 		 */
-	        if ((!tokenPresent && !slot->reader)
+		if ((!tokenPresent && !slot->reader)
 				|| (!tokenPresent && slot->reader != prev_reader)
-				|| (slot->slot_info.flags & CKF_TOKEN_PRESENT))
+				|| (slot->slot_info.flags & CKF_TOKEN_PRESENT)
+				|| (slot->flags & SC_PKCS11_SLOT_FLAG_SEEN)) {
 			found[numMatches++] = slot->id;
+			slot->flags |= SC_PKCS11_SLOT_FLAG_SEEN;
+		}
 		prev_reader = slot->reader;
 	}
 
+	/* Slot list can only change in v2.20 */
 	if (pSlotList == NULL_PTR) {
-		sc_log(context, "was only a size inquiry (%d)\n", numMatches);
+		/* slot->id is derived from its location in the list virtual_slots.
+		 * When the slot list changes, so does slot->id, so we reindex the
+		 * slots here the same way it is done in `create_slot()`
+		 *
+		 * TODO use a persistent CK_SLOT_ID, e.g. by using something like
+		 * `slot->id = sc_crc32(slot, sizeof *slot);` (this example, however,
+		 * is currently not thread safe).  */
+		for (i=0; i<list_size(&virtual_slots); i++) {
+			slot = (sc_pkcs11_slot_t *) list_get_at(&virtual_slots, i);
+			slot->id = (CK_SLOT_ID) list_locate(&virtual_slots, slot);
+		}
+	}
+
+	if (pSlotList == NULL_PTR) {
+		sc_log(context, "was only a size inquiry (%lu)\n", numMatches);
 		*pulCount = numMatches;
 		rv = CKR_OK;
 		goto out;
 	}
 
 	if (*pulCount < numMatches) {
-		sc_log(context, "buffer was too small (needed %d)\n", numMatches);
+		sc_log(context, "buffer was too small (needed %lu)\n", numMatches);
 		*pulCount = numMatches;
 		rv = CKR_BUFFER_TOO_SMALL;
 		goto out;
@@ -437,7 +521,7 @@ CK_RV C_GetSlotList(CK_BBOOL       tokenPresent,  /* only slots with token prese
 	*pulCount = numMatches;
 	rv = CKR_OK;
 
-	sc_log(context, "returned %d slots\n", numMatches);
+	sc_log(context, "returned %lu slots\n", numMatches);
 
 out:
 	if (found != NULL) {
@@ -500,17 +584,16 @@ CK_RV C_GetSlotInfo(CK_SLOT_ID slotID, CK_SLOT_INFO_PTR pInfo)
 	}
 
 	rv = slot_get_slot(slotID, &slot);
-	sc_log(context, "C_GetSlotInfo() get slot rv %i", rv);
-	if (rv == CKR_OK)   {
-		if (slot->reader == NULL)   {
+	sc_log(context, "C_GetSlotInfo() get slot rv %s", lookup_enum( RV_T, rv));
+	if (rv == CKR_OK) {
+		if (slot->reader == NULL) {
 			rv = CKR_TOKEN_NOT_PRESENT;
-		}
-		else {
+		} else {
 			now = get_current_time();
 			if (now >= slot->slot_state_expires || now == 0) {
 				/* Update slot status */
 				rv = card_detect(slot->reader);
-				sc_log(context, "C_GetSlotInfo() card detect rv 0x%X", rv);
+				sc_log(context, "C_GetSlotInfo() card detect rv 0x%lX", rv);
 
 				if (rv == CKR_TOKEN_NOT_RECOGNIZED || rv == CKR_OK)
 					slot->slot_info.flags |= CKF_TOKEN_PRESENT;
@@ -527,7 +610,7 @@ CK_RV C_GetSlotInfo(CK_SLOT_ID slotID, CK_SLOT_INFO_PTR pInfo)
 	if (rv == CKR_OK)
 		memcpy(pInfo, &slot->slot_info, sizeof(CK_SLOT_INFO));
 
-	sc_log(context, "C_GetSlotInfo() flags 0x%X", pInfo->flags);
+	sc_log(context, "C_GetSlotInfo() flags 0x%lX", pInfo->flags);
 	sc_log(context, "C_GetSlotInfo(0x%lx) = %s", slotID, lookup_enum( RV_T, rv));
 	sc_pkcs11_unlock();
 	return rv;
@@ -641,10 +724,11 @@ CK_RV C_WaitForSlotEvent(CK_FLAGS flags,   /* blocking/nonblocking flag */
 		return  CKR_ARGUMENTS_BAD;
 
 	sc_log(context, "C_WaitForSlotEvent(block=%d)", !(flags & CKF_DONT_BLOCK));
+#ifndef PCSCLITE_GOOD
 	/* Not all pcsc-lite versions implement consistently used functions as they are */
-	/* FIXME: add proper checking into build to check correct pcsc-lite version for SCardStatusChange/SCardCancel */
 	if (!(flags & CKF_DONT_BLOCK))
 		return CKR_FUNCTION_NOT_SUPPORTED;
+#endif /* PCSCLITE_GOOD */
 	rv = sc_pkcs11_lock();
 	if (rv != CKR_OK)
 		return rv;
@@ -742,7 +826,7 @@ sc_pkcs11_init_lock(CK_C_INITIALIZE_ARGS_PTR args)
 		/* Shall be used in threaded environment, must use operating system locking */
 		global_locking = default_mutex_funcs;
 	} else if (applock && !oslock) {
-		/* Shall be used in threaded envirnoment, must use app provided locking */
+		/* Shall be used in threaded environment, must use app provided locking */
 		global_locking = args;
 	} else if (!applock && !oslock) {
 		/* Shall not be used in threaded environment, use operating system locking */
