@@ -47,6 +47,8 @@ static sc_pkcs11_mechanism_type_t find_mechanism = {
 	NULL,		/* decrypt_init */
 	NULL,		/* decrypt */
 	NULL,		/* derive */
+	NULL,		/* wrap */
+	NULL,		/* unwrap */
 	NULL,		/* mech_data */
 	NULL,		/* free_mech_data */
 };
@@ -56,7 +58,6 @@ sc_find_release(sc_pkcs11_operation_t *operation)
 {
 	struct sc_pkcs11_find_operation *fop = (struct sc_pkcs11_find_operation *)operation;
 
-	sc_log(context,"freeing %d handles used %d  at %p", fop->allocated_handles, fop->num_handles, fop->handles);
 	if (fop->handles) {
 		free(fop->handles);
 		fop->handles = NULL;
@@ -95,6 +96,7 @@ CK_RV sc_create_object_int(CK_SESSION_HANDLE hSession,	/* the session's handle *
 	CK_RV rv = CKR_OK;
 	struct sc_pkcs11_session *session;
 	struct sc_pkcs11_card *card;
+	CK_BBOOL is_token = FALSE;
 
 	LOG_FUNC_CALLED(context);
 	if (pTemplate == NULL_PTR || ulCount == 0)
@@ -114,6 +116,22 @@ CK_RV sc_create_object_int(CK_SESSION_HANDLE hSession,	/* the session's handle *
 		goto out;
 	}
 
+	rv = attr_find(pTemplate, ulCount, CKA_TOKEN, &is_token, NULL);
+	if (rv != CKR_TEMPLATE_INCOMPLETE && rv != CKR_OK) {
+		goto out;
+	}
+
+	if (is_token == TRUE) {
+		if (session->slot->token_info.flags & CKF_WRITE_PROTECTED) {
+			rv = CKR_TOKEN_WRITE_PROTECTED;
+			goto out;
+		}
+		if (!(session->flags & CKF_RW_SESSION)) {
+			rv = CKR_SESSION_READ_ONLY;
+			goto out;
+		}
+	}
+
 	card = session->slot->p11card;
 	if (card->framework->create_object == NULL)
 		rv = CKR_FUNCTION_NOT_SUPPORTED;
@@ -123,7 +141,8 @@ CK_RV sc_create_object_int(CK_SESSION_HANDLE hSession,	/* the session's handle *
 out:
 	if (use_lock)
 		sc_pkcs11_unlock();
-	LOG_FUNC_RETURN(context, rv);
+
+	return rv;
 }
 
 
@@ -156,7 +175,7 @@ C_DestroyObject(CK_SESSION_HANDLE hSession,	/* the session's handle */
 	struct sc_pkcs11_session *session;
 	struct sc_pkcs11_object *object;
 	CK_BBOOL is_token = FALSE;
-	CK_ATTRIBUTE token_attribure = {CKA_TOKEN, &is_token, sizeof(is_token)};
+	CK_ATTRIBUTE token_attribute = {CKA_TOKEN, &is_token, sizeof(is_token)};
 
 	rv = sc_pkcs11_lock();
 	if (rv != CKR_OK)
@@ -167,10 +186,16 @@ C_DestroyObject(CK_SESSION_HANDLE hSession,	/* the session's handle */
 	if (rv != CKR_OK)
 		goto out;
 
-	object->ops->get_attribute(session, object, &token_attribure);
-	if (is_token == TRUE && !(session->flags & CKF_RW_SESSION)) {
-		rv = CKR_SESSION_READ_ONLY;
-		goto out;
+	object->ops->get_attribute(session, object, &token_attribute);
+	if (is_token == TRUE) {
+		if (session->slot->token_info.flags & CKF_WRITE_PROTECTED) {
+			rv = CKR_TOKEN_WRITE_PROTECTED;
+			goto out;
+		}
+		if (!(session->flags & CKF_RW_SESSION)) {
+			rv = CKR_SESSION_READ_ONLY;
+			goto out;
+		}
 	}
 
 	if (object->ops->destroy_object == NULL)
@@ -199,7 +224,7 @@ C_GetAttributeValue(CK_SESSION_HANDLE hSession,	/* the session's handle */
 		CK_ATTRIBUTE_PTR pTemplate,	/* specifies attributes, gets values */
 		CK_ULONG ulCount)		/* attributes in template */
 {
-	static int precedence[] = {
+	static CK_RV precedence[] = {
 		CKR_OK,
 		CKR_BUFFER_TOO_SMALL,
 		CKR_ATTRIBUTE_TYPE_INVALID,
@@ -207,11 +232,12 @@ C_GetAttributeValue(CK_SESSION_HANDLE hSession,	/* the session's handle */
 		-1
 	};
 	char object_name[64];
-	int j;
+	CK_RV j;
 	CK_RV rv;
 	struct sc_pkcs11_session *session;
 	struct sc_pkcs11_object *object;
-	int res, res_type;
+	CK_RV res;
+	CK_RV res_type;
 	unsigned int i;
 
 	if (pTemplate == NULL_PTR || ulCount == 0)
@@ -245,7 +271,7 @@ C_GetAttributeValue(CK_SESSION_HANDLE hSession,	/* the session's handle */
 		 * should be handled - we give them highest
 		 * precedence
 		 */
-		for (j = 0; precedence[j] != -1; j++) {
+		for (j = 0; precedence[j] != (CK_RV) -1; j++) {
 			if (precedence[j] == res)
 				break;
 		}
@@ -1001,7 +1027,8 @@ CK_RV C_GenerateKeyPair(CK_SESSION_HANDLE hSession,	/* the session's handle */
 	}
 
 	slot = session->slot;
-	if (slot->p11card->framework->gen_keypair == NULL)
+	if (slot == NULL || slot->p11card == NULL || slot->p11card->framework == NULL
+			|| slot->p11card->framework->gen_keypair == NULL)
 		rv = CKR_FUNCTION_NOT_SUPPORTED;
 	else {
 		rv = restore_login_state(slot);
@@ -1026,7 +1053,71 @@ CK_RV C_WrapKey(CK_SESSION_HANDLE hSession,	/* the session's handle */
 		CK_BYTE_PTR pWrappedKey,	/* receives the wrapped key */
 		CK_ULONG_PTR pulWrappedKeyLen)
 {				/* receives byte size of wrapped key */
-	return CKR_FUNCTION_NOT_SUPPORTED;
+	CK_RV rv;
+	CK_BBOOL can_wrap,
+			 can_be_wrapped;
+	CK_KEY_TYPE key_type;
+	CK_ATTRIBUTE wrap_attribute = { CKA_WRAP, &can_wrap, sizeof(can_wrap) };
+	CK_ATTRIBUTE extractable_attribute = { CKA_EXTRACTABLE, &can_be_wrapped, sizeof(can_be_wrapped) };
+	CK_ATTRIBUTE key_type_attr = { CKA_KEY_TYPE, &key_type, sizeof(key_type) };
+	struct sc_pkcs11_session *session;
+	struct sc_pkcs11_object *wrapping_object;
+	struct sc_pkcs11_object *key_object;
+
+	if (pMechanism == NULL_PTR)
+		return CKR_ARGUMENTS_BAD;
+
+	rv = sc_pkcs11_lock();
+	if (rv != CKR_OK)
+		return rv;
+
+	/* Check if the wrapping key is OK to do wrapping */
+	rv = get_object_from_session(hSession, hWrappingKey, &session, &wrapping_object);
+	if (rv != CKR_OK) {
+		if (rv == CKR_OBJECT_HANDLE_INVALID)
+			rv = CKR_KEY_HANDLE_INVALID;
+		goto out;
+	}
+	if (wrapping_object->ops->wrap_key == NULL_PTR) {
+		rv = CKR_KEY_TYPE_INCONSISTENT;
+		goto out;
+	}
+
+	rv = wrapping_object->ops->get_attribute(session, wrapping_object, &wrap_attribute);
+	if (rv != CKR_OK || !can_wrap) {
+		rv = CKR_KEY_TYPE_INCONSISTENT;
+		goto out;
+	}
+	rv = wrapping_object->ops->get_attribute(session, wrapping_object, &key_type_attr);
+	if (rv != CKR_OK) {
+		rv = CKR_KEY_TYPE_INCONSISTENT;
+		goto out;
+	}
+
+	/* Check if the key to be wrapped exists and is extractable*/
+	rv = get_object_from_session(hSession, hKey, &session, &key_object);
+	if (rv != CKR_OK) {
+		if (rv == CKR_OBJECT_HANDLE_INVALID)
+			rv = CKR_KEY_HANDLE_INVALID;
+		goto out;
+	}
+
+	rv = key_object->ops->get_attribute(session, key_object, &extractable_attribute);
+	if (rv != CKR_OK || !can_be_wrapped) {
+		rv = CKR_KEY_TYPE_INCONSISTENT;
+		goto out;
+	}
+
+	rv = restore_login_state(session->slot);
+	if (rv == CKR_OK)
+		rv = sc_pkcs11_wrap(session, pMechanism, wrapping_object, key_type,
+				key_object, pWrappedKey, pulWrappedKeyLen);
+
+	rv = reset_login_state(session->slot, rv);
+
+out:
+	sc_pkcs11_unlock();
+	return rv;
 }
 
 CK_RV C_UnwrapKey(CK_SESSION_HANDLE hSession,	/* the session's handle */
@@ -1038,7 +1129,67 @@ CK_RV C_UnwrapKey(CK_SESSION_HANDLE hSession,	/* the session's handle */
 		  CK_ULONG ulAttributeCount,	/* # of attributes in template */
 		  CK_OBJECT_HANDLE_PTR phKey)
 {				/* gets handle of recovered key */
-	return CKR_FUNCTION_NOT_SUPPORTED;
+	CK_RV rv;
+	CK_BBOOL can_unwrap;
+	CK_KEY_TYPE key_type;
+	CK_ATTRIBUTE unwrap_attribute = { CKA_UNWRAP, &can_unwrap, sizeof(can_unwrap) };
+	CK_ATTRIBUTE key_type_attr = { CKA_KEY_TYPE, &key_type, sizeof(key_type) };
+	struct sc_pkcs11_session *session;
+	struct sc_pkcs11_object *object;
+	struct sc_pkcs11_object *key_object;
+
+	if (pMechanism == NULL_PTR)
+		return CKR_ARGUMENTS_BAD;
+
+	rv = sc_pkcs11_lock();
+	if (rv != CKR_OK)
+		return rv;
+
+	rv = get_object_from_session(hSession, hUnwrappingKey, &session, &object);
+	if (rv != CKR_OK) {
+		if (rv == CKR_OBJECT_HANDLE_INVALID)
+			rv = CKR_KEY_HANDLE_INVALID;
+		goto out;
+	}
+	if (object->ops->unwrap_key == NULL_PTR) {
+		rv = CKR_KEY_TYPE_INCONSISTENT;
+		goto out;
+	}
+
+	rv = object->ops->get_attribute(session, object, &unwrap_attribute);
+	if (rv != CKR_OK || !can_unwrap) {
+		rv = CKR_KEY_TYPE_INCONSISTENT;
+		goto out;
+	}
+	rv = object->ops->get_attribute(session, object, &key_type_attr);
+	if (rv != CKR_OK) {
+		rv = CKR_KEY_TYPE_INCONSISTENT;
+		goto out;
+	}
+
+	/* Create the target object in memory */
+	rv = sc_create_object_int(hSession, pTemplate, ulAttributeCount, phKey, 0);
+
+	if (rv != CKR_OK)
+	    goto out;
+
+	rv = get_object_from_session(hSession, *phKey, &session, &key_object);
+	if (rv != CKR_OK) {
+		if (rv == CKR_OBJECT_HANDLE_INVALID)
+			rv = CKR_KEY_HANDLE_INVALID;
+		goto out;
+	}
+
+	rv = restore_login_state(session->slot);
+	if (rv == CKR_OK)
+		rv = sc_pkcs11_unwrap(session, pMechanism, object, key_type,
+				pWrappedKey, ulWrappedKeyLen, key_object);
+	/* TODO if (rv != CK_OK) need to destroy the object */
+	rv = reset_login_state(session->slot, rv);
+
+out:
+	sc_pkcs11_unlock();
+	return rv;
 }
 
 CK_RV C_DeriveKey(CK_SESSION_HANDLE hSession,	/* the session's handle */
@@ -1145,7 +1296,8 @@ CK_RV C_GenerateRandom(CK_SESSION_HANDLE hSession,	/* the session's handle */
 	rv = get_session(hSession, &session);
 	if (rv == CKR_OK) {
 		slot = session->slot;
-		if (slot->p11card->framework->get_random == NULL)
+		if (slot == NULL || slot->p11card == NULL || slot->p11card->framework == NULL
+				|| slot->p11card->framework->get_random == NULL)
 			rv = CKR_RANDOM_NO_RNG;
 		else
 			rv = slot->p11card->framework->get_random(slot, RandomData, ulRandomLen);
@@ -1314,14 +1466,13 @@ CK_RV C_VerifyRecover(CK_SESSION_HANDLE hSession,	/* the session's handle */
 /*
  * Helper function to compare attributes on any sort of object
  */
-int sc_pkcs11_any_cmp_attribute(struct sc_pkcs11_session *session, void *ptr, CK_ATTRIBUTE_PTR attr)
+CK_RV sc_pkcs11_any_cmp_attribute(struct sc_pkcs11_session *session, void *ptr, CK_ATTRIBUTE_PTR attr)
 {
-	int rv;
+	CK_RV rv;
 	struct sc_pkcs11_object *object;
 	u8 temp1[1024];
 	u8 *temp2 = NULL;	/* dynamic allocation for large attributes */
 	CK_ATTRIBUTE temp_attr;
-	int res;
 
 	object = (struct sc_pkcs11_object *)ptr;
 	temp_attr.type = attr->type;
@@ -1345,7 +1496,7 @@ int sc_pkcs11_any_cmp_attribute(struct sc_pkcs11_session *session, void *ptr, CK
 	/* Get the attribute */
 	rv = object->ops->get_attribute(session, object, &temp_attr);
 	if (rv != CKR_OK) {
-		res = 0;
+		rv = 0;
 		goto done;
 	}
 #ifdef DEBUG
@@ -1356,12 +1507,12 @@ int sc_pkcs11_any_cmp_attribute(struct sc_pkcs11_session *session, void *ptr, CK
 		dump_template(SC_LOG_DEBUG_NORMAL, foo, &temp_attr, 1);
 	}
 #endif
-	res = temp_attr.ulValueLen == attr->ulValueLen
+	rv = temp_attr.ulValueLen == attr->ulValueLen
 	    && !memcmp(temp_attr.pValue, attr->pValue, attr->ulValueLen);
 
       done:
 	if (temp2 != NULL)
 		free(temp2);
 
-	return res;
+	return rv;
 }
