@@ -115,7 +115,6 @@ CK_RV create_slot(sc_reader_t *reader)
 	}
 
 	slot->login_user = -1;
-	slot->id = (CK_SLOT_ID) list_locate(&virtual_slots, slot);
 	init_slot_info(&slot->slot_info, reader);
 	sc_log(context, "Initializing slot with id 0x%lx", slot->id);
 
@@ -126,6 +125,7 @@ CK_RV create_slot(sc_reader_t *reader)
 		slot->slot_info.hardwareVersion.major = reader->version_major;
 		slot->slot_info.hardwareVersion.minor = reader->version_minor;
 	}
+	slot->id = (CK_SLOT_ID) list_locate(&virtual_slots, slot);
 
 	return CKR_OK;
 }
@@ -374,10 +374,10 @@ again:
 
 fail:
 	if (free_p11card) {
-		if (p11card->card != NULL)
-			sc_disconnect_card(p11card->card);
 		if (p11card->framework)
 			p11card->framework->unbind(p11card);
+		if (p11card->card != NULL)
+			sc_disconnect_card(p11card->card);
 		free(p11card);
 	}
 
@@ -447,7 +447,7 @@ CK_RV slot_get_slot(CK_SLOT_ID id, struct sc_pkcs11_slot ** slot)
 
 CK_RV slot_get_token(CK_SLOT_ID id, struct sc_pkcs11_slot ** slot)
 {
-	int rv;
+	CK_RV rv;
 
 	sc_log(context, "Slot(id=0x%lX): get token", id);
 	rv = slot_get_slot(id, slot);
@@ -473,7 +473,8 @@ CK_RV slot_get_token(CK_SLOT_ID id, struct sc_pkcs11_slot ** slot)
 
 CK_RV slot_token_removed(CK_SLOT_ID id)
 {
-	int rv, token_was_present;
+	CK_RV rv;
+	int token_was_present;
 	struct sc_pkcs11_slot *slot;
 	struct sc_pkcs11_object *object;
 
@@ -494,8 +495,8 @@ CK_RV slot_token_removed(CK_SLOT_ID id)
 
 	/* Release framework stuff */
 	if (slot->p11card != NULL) {
-		if (slot->fw_data != NULL &&
-				slot->p11card->framework != NULL && slot->p11card->framework->release_token != NULL) {
+		if (slot->fw_data != NULL && slot->p11card->framework != NULL
+				&& slot->p11card->framework->release_token != NULL) {
 			slot->p11card->framework->release_token(slot->p11card, slot->fw_data);
 			slot->fw_data = NULL;
 		}
