@@ -92,12 +92,15 @@ add_padding(const struct iso_sm_ctx *ctx, const u8 *data, size_t datalen,
 	switch (ctx->padding_indicator) {
 		case SM_NO_PADDING:
 			if (*padded != data) {
-				p = realloc(*padded, datalen);
-				if (!p)
-					return SC_ERROR_OUT_OF_MEMORY;
-				*padded = p;
-				/* Flawfinder: ignore */
-				memcpy(*padded, data, datalen);
+				if (datalen != 0) {
+					p = realloc(*padded, datalen);
+					if (!p)
+						return SC_ERROR_OUT_OF_MEMORY;
+					*padded = p;
+					memcpy(*padded, data, datalen);
+				} else {
+					*padded = NULL;
+				}
 			}
 			return datalen;
 		case SM_ISO_PADDING:
@@ -146,7 +149,7 @@ static int format_le(size_t le, struct sc_asn1_entry *le_entry,
 {
 	u8 *p;
 
-	if (!lebuf || !le_len)
+	if (!lebuf || !le_len || !*le_len)
 		return SC_ERROR_INVALID_ARGUMENTS;
 
 	p = realloc(*lebuf, *le_len);
@@ -156,15 +159,15 @@ static int format_le(size_t le, struct sc_asn1_entry *le_entry,
 
 	switch (*le_len) {
 		case 1:
-			p[0] = le;
+			p[0] = le & 0xff;
 			break;
 		case 2:
-			p[0] = le >> 8;
+			p[0] = (le >> 8) & 0xff;
 			p[1] = le & 0xff;
 			break;
 		case 3:
 			p[0] = 0x00;
-			p[1] = le >> 8;
+			p[1] = (le >> 8) & 0xff;
 			p[2] = le & 0xff;
 			break;
 		default:
@@ -219,13 +222,13 @@ static int format_data(sc_card_t *card, const struct iso_sm_ctx *ctx,
 	}
 	pad_data_len = r;
 
-	sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Data to encrypt", pad_data, pad_data_len);
+	sc_log_hex(card->ctx, "Data to encrypt", pad_data, pad_data_len);
 	r = ctx->encrypt(card, ctx, pad_data, pad_data_len, formatted_data);
 	if (r < 0) {
 		sc_debug(card->ctx, SC_LOG_DEBUG_VERBOSE, "Could not encrypt the data");
 		goto err;
 	}
-	sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Cryptogram", *formatted_data, r);
+	sc_log_hex(card->ctx, "Cryptogram", *formatted_data, r);
 
 	if (prepend_padding_indicator) {
 		r = prefix_buf(ctx->padding_indicator, *formatted_data, r, formatted_data);
@@ -279,7 +282,7 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 	u8 *p, *le = NULL, *sm_data = NULL, *fdata = NULL, *mac_data = NULL,
 	   *asn1 = NULL, *mac = NULL, *resp_data = NULL;
 	size_t sm_data_len, fdata_len, mac_data_len, asn1_len, mac_len, le_len;
-	int r, cse;
+	int r;
 	sc_apdu_t *sm_apdu = NULL;
 
 	if (!apdu || !ctx || !card || !card->reader || !psm_apdu) {
@@ -313,17 +316,7 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 	}
 	mac_data_len = r;
 
-	/* get le and data depending on the case of the insecure command */
-	cse = apdu->cse;
-	if ((apdu->le/ctx->block_length + 1)*ctx->block_length + 18 > 0xff+1)
-		/* for encrypted APDUs we usually get authenticated status bytes (4B),
-		 * a MAC (11B) and a cryptogram with padding indicator (3B without
-		 * data).  The cryptogram is always padded to the block size. */
-		/*cse |= SC_APDU_EXT;*/
-		sc_debug(card->ctx, SC_LOG_DEBUG_VERBOSE,
-				"Response data may be truncated, because it doesn't fit into a short length APDU.");
-
-	switch (cse) {
+	switch (apdu->cse) {
 		case SC_APDU_CASE_1:
 			break;
 	case SC_APDU_CASE_2_SHORT:
@@ -333,7 +326,7 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 				sc_debug(card->ctx, SC_LOG_DEBUG_VERBOSE, "Could not format Le of SM apdu");
 				goto err;
 			}
-			sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Protected Le (plain)", le, le_len);
+			sc_log_hex(card->ctx, "Protected Le (plain)", le, le_len);
 			break;
 	case SC_APDU_CASE_2_EXT:
 			if (card->reader->active_protocol == SC_PROTO_T0) {
@@ -353,7 +346,7 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 					goto err;
 				}
 			}
-			sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Protected Le (plain)", le, le_len);
+			sc_log_hex(card->ctx, "Protected Le (plain)", le, le_len);
 			break;
 		case SC_APDU_CASE_3_SHORT:
 		case SC_APDU_CASE_3_EXT:
@@ -368,7 +361,7 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 				sc_debug(card->ctx, SC_LOG_DEBUG_VERBOSE, "Could not format data of SM apdu");
 				goto err;
 			}
-			sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Padding-content indicator followed by cryptogram (plain)",
+			sc_log_hex(card->ctx, "Padding-content indicator followed by cryptogram (plain)",
 					fdata, fdata_len);
 			break;
 		case SC_APDU_CASE_4_SHORT:
@@ -380,7 +373,7 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 					sc_debug(card->ctx, SC_LOG_DEBUG_VERBOSE, "Could not format Le of SM apdu");
 					goto err;
 				}
-				sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Protected Le (plain)", le, le_len);
+				sc_log_hex(card->ctx, "Protected Le (plain)", le, le_len);
 			}
 
 			if (apdu->ins & 1) {
@@ -394,7 +387,7 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 				sc_debug(card->ctx, SC_LOG_DEBUG_VERBOSE, "Could not format data of SM apdu");
 				goto err;
 			}
-			sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Padding-content indicator followed by cryptogram (plain)",
+			sc_log_hex(card->ctx, "Padding-content indicator followed by cryptogram (plain)",
 					fdata, fdata_len);
 			break;
 		case SC_APDU_CASE_4_EXT:
@@ -411,7 +404,7 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 					sc_debug(card->ctx, SC_LOG_DEBUG_VERBOSE, "Could not format Le of SM apdu");
 					goto err;
 				}
-				sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Protected Le (plain)", le, le_len);
+				sc_log_hex(card->ctx, "Protected Le (plain)", le, le_len);
 			}
 
 			if (apdu->ins & 1) {
@@ -425,7 +418,7 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 				sc_debug(card->ctx, SC_LOG_DEBUG_VERBOSE, "Could not format data of SM apdu");
 				goto err;
 			}
-			sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Padding-content indicator followed by cryptogram (plain)",
+			sc_log_hex(card->ctx, "Padding-content indicator followed by cryptogram (plain)",
 					fdata, fdata_len);
 			break;
 		default:
@@ -455,7 +448,7 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 		}
 		mac_data_len = r;
 	}
-	sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Data to authenticate", mac_data, mac_data_len);
+	sc_log_hex(card->ctx, "Data to authenticate", mac_data, mac_data_len);
 
 	r = ctx->authenticate(card, ctx, mac_data, mac_data_len,
 			&mac);
@@ -464,7 +457,7 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 		goto err;
 	}
 	mac_len = r;
-	sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Cryptographic Checksum (plain)", mac, mac_len);
+	sc_log_hex(card->ctx, "Cryptographic Checksum (plain)", mac, mac_len);
 
 
 	/* format SM apdu */
@@ -476,28 +469,23 @@ static int sm_encrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 	sm_apdu->datalen = sm_data_len;
 	sm_apdu->lc = sm_data_len;
 	sm_apdu->le = 0;
-	if (cse & SC_APDU_EXT) {
+	if (apdu->cse & SC_APDU_EXT) {
 		sm_apdu->cse = SC_APDU_CASE_4_EXT;
-#if OPENSC_NOT_BOGUS_ANYMORE
-		sm_apdu->resplen = 0xffff+1;
-#else
-		sm_apdu->resplen = SC_MAX_EXT_APDU_BUFFER_SIZE;
-#endif
 	} else {
 		sm_apdu->cse = SC_APDU_CASE_4_SHORT;
-#if OPENSC_NOT_BOGUS_ANYMORE
-		sm_apdu->resplen = 0xff+1;
-#else
-		sm_apdu->resplen = SC_MAX_APDU_BUFFER_SIZE;
-#endif
 	}
-	resp_data = malloc(sm_apdu->resplen);
+	/* for encrypted APDUs we usually get authenticated status bytes
+	 * (4B), a MAC (2B without data) and a cryptogram with padding
+	 * indicator (3B without data). The cryptogram is always padded to
+	 * the block size. */
+	sm_apdu->resplen = 4 + 2 + mac_len + 3 + ((apdu->resplen+1)/ctx->block_length+1)*ctx->block_length;
+	resp_data = calloc(sm_apdu->resplen, 1);
 	if (!resp_data) {
 		r = SC_ERROR_OUT_OF_MEMORY;
 		goto err;
 	}
 	sm_apdu->resp = resp_data;
-	sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "ASN.1 encoded encrypted APDU data", sm_apdu->data, sm_apdu->datalen);
+	sc_log_hex(card->ctx, "ASN.1 encoded encrypted APDU data", sm_apdu->data, sm_apdu->datalen);
 
 	*psm_apdu = sm_apdu;
 
@@ -618,9 +606,9 @@ static int sm_decrypt(const struct iso_sm_ctx *ctx, sc_card_t *card,
 		goto err;
 	}
 
-	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Decrypted APDU sw1=%02x sw2=%02x",
+	sc_log(card->ctx,  "Decrypted APDU sw1=%02x sw2=%02x",
 			apdu->sw1, apdu->sw2);
-	sc_debug_hex(card->ctx, SC_LOG_DEBUG_NORMAL, "Decrypted APDU response data",
+	sc_log_hex(card->ctx, "Decrypted APDU response data",
 			apdu->resp, apdu->resplen);
 
 	r = SC_SUCCESS;
@@ -644,15 +632,15 @@ static int iso_add_sm(struct iso_sm_ctx *sctx, sc_card_t *card,
 
 	if ((apdu->cla & 0x0C) == 0x0C) {
 		sc_debug(card->ctx, SC_LOG_DEBUG_VERBOSE, "Given APDU is already protected with some secure messaging. Closing own SM context.");
-		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, sc_sm_stop(card),
+		LOG_TEST_RET(card->ctx, sc_sm_stop(card),
 				"Could not close ISO SM session");
 		return SC_ERROR_SM_NOT_APPLIED;
 	}
 
 	if (sctx->pre_transmit)
-		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, sctx->pre_transmit(card, sctx, apdu),
+		LOG_TEST_RET(card->ctx, sctx->pre_transmit(card, sctx, apdu),
 				"Could not complete SM specific pre transmit routine");
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, sm_encrypt(sctx, card, apdu, sm_apdu),
+	LOG_TEST_RET(card->ctx, sm_encrypt(sctx, card, apdu, sm_apdu),
 			"Could not encrypt APDU");
 
 	return SC_SUCCESS;
@@ -661,13 +649,17 @@ static int iso_add_sm(struct iso_sm_ctx *sctx, sc_card_t *card,
 static int iso_rm_sm(struct iso_sm_ctx *sctx, sc_card_t *card,
 		sc_apdu_t *sm_apdu, sc_apdu_t *apdu)
 {
+	if (!sctx)
+		LOG_TEST_RET(card->ctx, SC_ERROR_INVALID_ARGUMENTS,
+			"Invalid SM context. No SM processing performed.");
+
 	if (sctx->post_transmit)
-		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, sctx->post_transmit(card, sctx, sm_apdu),
+		LOG_TEST_RET(card->ctx, sctx->post_transmit(card, sctx, sm_apdu),
 				"Could not complete SM specific post transmit routine");
-	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, sm_decrypt(sctx, card, sm_apdu, apdu),
+	LOG_TEST_RET(card->ctx, sm_decrypt(sctx, card, sm_apdu, apdu),
 			"Could not decrypt APDU");
 	if (sctx->finish)
-		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, sctx->finish(card, sctx, apdu),
+		LOG_TEST_RET(card->ctx, sctx->finish(card, sctx, apdu),
 				"Could not complete SM specific post transmit routine");
 
 	return SC_SUCCESS;
