@@ -10,7 +10,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "common/compat_getopt.h"
+#include <getopt.h>
 #include "libopensc/opensc.h"
 #include "sc-test.h"
 
