@@ -188,6 +188,9 @@ static int cardos_info(void)
 		printf(" (that's CardOS M4.4)\n");
 	} else if (apdu.resp[0] == 0xc9 && apdu.resp[1] == 0x01) {
 		printf(" (that's CardOS V5.0)\n");
+	} else if (apdu.resp[0] == 0xc9 &&
+			(apdu.resp[1] == 0x02 || apdu.resp[1] == 0x03)) {
+		printf(" (that's CardOS V5.3)\n");
 	} else {
 		printf(" (unknown Version)\n");
 	}
@@ -462,7 +465,7 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 		memcpy(&enc_input[0],&in[5],plain_lc);
 	for (i=0; i < 8; i++) enc_input[i+plain_lc] = des_out[i];
 	enc_input[plain_lc+8] = 0x80; /* iso padding */
-	/* calloc already cleard the remaining bytes to 00 */
+	/* calloc already cleared the remaining bytes to 00 */
 
 	if (outlen < 5 + enc_input_len) {
 		free(mac_input);
@@ -482,18 +485,18 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 	/* xor data and IV (8 bytes 00) to get input data */
 	for (i=0; i < 8; i++) des_in[i] = enc_input[i] ^ 00;
 
-	/* encrypt with des2 (tripple des, but using keys A-B-A) */
+	/* encrypt with des2 (triple des, but using keys A-B-A) */
 	DES_ecb2_encrypt(&des_in, &des_out, &ks_a, &ks_b, 1);
 
 	/* copy encrypted bytes into output */
 	for (i=0; i < 8; i++) out[5+i] = des_out[i];
 
-	/* encrypt other blocks (usualy one) */
+	/* encrypt other blocks (usually one) */
 	for (j=1; j < (enc_input_len / 8); j++) {
 		/* xor data and prev. result to get input data */
 		for (i=0; i < 8; i++) des_in[i] = enc_input[i+j*8] ^ des_out[i];
 
-		/* encrypt with des2 (tripple des, but using keys A-B-A) */
+		/* encrypt with des2 (triple des, but using keys A-B-A) */
 		DES_ecb2_encrypt(&des_in, &des_out, &ks_a, &ks_b, 1);
 
 		/* copy encrypted bytes into output */
@@ -850,6 +853,11 @@ static int cardos_change_startkey(const char *change_startkey_apdu)
 	u8 rbuf[256];
 	int r;
 
+	if (!change_startkey_apdu) {
+		printf("Missing change StartKey, aborting\n");
+		return 1;
+	}
+
 	if (verbose)	{
 		printf ("Change StartKey APDU:\n");
 		util_hex_dump_asc(stdout, (unsigned char *)change_startkey_apdu,
@@ -1024,7 +1032,7 @@ static int cardos_change_startkey(const char *change_startkey_apdu)   {
 }
 #endif
 
-int main(int argc, char *const argv[])
+int main(int argc, char *argv[])
 {
 	int err = 0, r, c, long_optind = 0;
 	int do_info = 0;
@@ -1087,11 +1095,6 @@ int main(int argc, char *const argv[])
 		return 1;
 	}
 
-	if (verbose > 1) {
-		ctx->debug = verbose;
-		sc_ctx_log_to_file(ctx, "stderr");
-	}
-
 	if (opt_driver != NULL) {
 		err = sc_set_card_driver(ctx, opt_driver);
 		if (err) {
@@ -1124,12 +1127,11 @@ int main(int argc, char *const argv[])
 		}
 		action_count--;
 	}
-      end:
+end:
 	if (card) {
 		sc_unlock(card);
 		sc_disconnect_card(card);
 	}
-	if (ctx)
-		sc_release_context(ctx);
+	sc_release_context(ctx);
 	return err;
 }
