@@ -2,7 +2,7 @@
  * piv-tool.c: Tool for accessing smart cards with libopensc
  *
  * Copyright (C) 2001  Juha Yrjölä <juha.yrjola@iki.fi>
- * Copyright (C) 2005,2010 Douglas E. Engert <deengert@anl.gov>
+ * Copyright (C) 2005,2010 Douglas E. Engert <deengert@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,17 +33,14 @@
 
 /* Module only built if OPENSSL is enabled */
 #include <openssl/opensslv.h>
-#include "libopensc/sc-ossl-compat.h"
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
 #include <openssl/opensslconf.h>
 #include <openssl/crypto.h>
 #endif
-#if OPENSSL_VERSION_NUMBER >= 0x00907000L
 #include <openssl/conf.h>
-#endif
 
 #include <openssl/rsa.h>
-#if OPENSSL_VERSION_NUMBER >= 0x00908000L && !defined(OPENSSL_NO_EC) && !defined(OPENSSL_NO_ECDSA)
+#if !defined(OPENSSL_NO_EC) && !defined(OPENSSL_NO_ECDSA)
 #include <openssl/ec.h>
 #include <openssl/ecdsa.h>
 #endif
@@ -59,6 +56,7 @@
 #include "libopensc/cardctl.h"
 #include "libopensc/asn1.h"
 #include "util.h"
+#include "libopensc/sc-ossl-compat.h"
 
 static const char *app_name = "piv-tool";
 
@@ -194,7 +192,7 @@ static int load_cert(const char * cert_id, const char * cert_file,
 				cert_file, strerror(errno));
         goto err;
     }
-	if (compress) { /* file is gziped already */
+	if (compress) { /* file is gzipped already */
 		struct stat stat_buf;
 
 		if (0 != stat(cert_file, &stat_buf)) {
@@ -264,7 +262,7 @@ static int admin_mode(const char* admin_info)
 	size_t buflen = 2;
 
 
-	if (strlen(admin_info) == 7 &&
+	if (admin_info && strlen(admin_info) == 7 &&
 			(admin_info[0] == 'A' || admin_info[0] == 'M') &&
 			admin_info[1] == ':' &&
 			(sc_hex_to_bin(admin_info+2, opts+1, &buflen) == 0) &&
@@ -292,7 +290,7 @@ static int gen_key(const char * key_info)
 		keydata = {0, 0, 0, 0, NULL, 0, NULL, 0, NULL, 0};
 	unsigned long expl;
 	u8 expc[4];
-#if OPENSSL_VERSION_NUMBER >= 0x00908000L && !defined(OPENSSL_NO_EC)
+#if !defined(OPENSSL_NO_EC)
 	int nid = -1;
 #endif
 	sc_hex_to_bin(key_info, buf, &buflen);
@@ -316,7 +314,7 @@ static int gen_key(const char * key_info)
 		case 0x05: keydata.key_bits = 3072; break;
 		case 0x06: keydata.key_bits = 1024; break;
 		case 0x07: keydata.key_bits = 2048; break;
-#if OPENSSL_VERSION_NUMBER >= 0x00908000L && !defined(OPENSSL_NO_EC)
+#if !defined(OPENSSL_NO_EC)
 		case 0x11: keydata.key_bits = 0;
 			nid = NID_X9_62_prime256v1; /* We only support one curve per algid */
 			break;
@@ -338,7 +336,7 @@ static int gen_key(const char * key_info)
 		return r;
 	}
 
-		evpkey = EVP_PKEY_new();
+	evpkey = EVP_PKEY_new();
 
 	if (keydata.key_bits > 0) { /* RSA key */
 		RSA * newkey = NULL;
@@ -368,7 +366,7 @@ static int gen_key(const char * key_info)
 		EVP_PKEY_assign_RSA(evpkey, newkey);
 
 	} else { /* EC key */
-#if OPENSSL_VERSION_NUMBER >= 0x00908000L && !defined(OPENSSL_NO_EC)
+#if !defined(OPENSSL_NO_EC)
 		int i;
 		BIGNUM *x;
 		BIGNUM *y;
@@ -385,9 +383,21 @@ static int gen_key(const char * key_info)
 		x = BN_bin2bn(keydata.ecpoint + 1, i, NULL);
 		y = BN_bin2bn(keydata.ecpoint + 1 + i, i, NULL) ;
 		r = EC_POINT_set_affine_coordinates_GFp(ecgroup, ecpoint, x, y, NULL);
+		if (r == 0) {
+			fprintf(stderr, "EC_POINT_set_affine_coordinates_GFp failed\n");
+			return -1;
+		}
 		eckey = EC_KEY_new();
 		r = EC_KEY_set_group(eckey, ecgroup);
+		if (r == 0) {
+			fprintf(stderr, "EC_KEY_set_group failed\n");
+			return -1;
+		}
 		r = EC_KEY_set_public_key(eckey, ecpoint);
+		if (r == 0) {
+			fprintf(stderr, "EC_KEY_set_public_key failed\n");
+			return -1;
+		}
 
 		if (verbose)
 			EC_KEY_print_fp(stdout, eckey, 0);
@@ -459,7 +469,7 @@ static void print_serial(sc_card_t *in_card)
 		util_hex_dump_asc(stdout, serial.value, serial.len, -1);
 }
 
-int main(int argc, char * const argv[])
+int main(int argc, char *argv[])
 {
 	int err = 0, r, c, long_optind = 0;
 	int do_send_apdu = 0;
@@ -479,9 +489,7 @@ int main(int argc, char * const argv[])
 	const char *key_info = NULL;
 	const char *admin_info = NULL;
 	sc_context_param_t ctx_param;
-
-	setbuf(stderr, NULL);
-	setbuf(stdout, NULL);
+	char **old_apdus = NULL;
 
 	while (1) {
 		c = getopt_long(argc, argv, "nA:G:O:Z:C:i:o:fvs:c:w", options, &long_optind);
@@ -495,9 +503,11 @@ int main(int argc, char * const argv[])
 			action_count++;
 			break;
 		case 's':
+			old_apdus = opt_apdus;
 			opt_apdus = (char **) realloc(opt_apdus,
 					(opt_apdu_count + 1) * sizeof(char *));
 			if (!opt_apdus) {
+				free(old_apdus);
 				err = 1;
 				goto end;
 			}
@@ -558,7 +568,7 @@ int main(int argc, char * const argv[])
 		util_print_usage_and_die(app_name, options, option_help, NULL);
 
 
-//#if (OPENSSL_VERSION_NUMBER >= 0x00907000L && OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(LIBRESSL_VERSION_NUMBER)
+//#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
 //	OPENSSL_config(NULL);
 //#endif
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
@@ -592,12 +602,6 @@ int main(int argc, char * const argv[])
 		return 1;
 	}
 
-	/* Only change if not in opensc.conf */
-	if (verbose > 1 && ctx->debug == 0) {
-		ctx->debug = verbose;
-		sc_ctx_log_to_file(ctx, "stderr");
-	}
-
 	if (action_count <= 0)
 		goto end;
 
@@ -658,8 +662,7 @@ end:
 		sc_unlock(card);
 		sc_disconnect_card(card);
 	}
-	if (ctx)
-		sc_release_context(ctx);
+	sc_release_context(ctx);
 
 	ERR_print_errors_fp(stderr);
 	return err;
