@@ -292,7 +292,7 @@ static int decipher(struct sc_pkcs15_object *obj)
 
 static int get_key(unsigned int usage, sc_pkcs15_object_t **result)
 {
-	sc_pkcs15_object_t *key, *pin;
+	sc_pkcs15_object_t *key, *pin = NULL;
 	const char	*usage_name;
 	sc_pkcs15_id_t	id;
 	int		r;
@@ -330,7 +330,7 @@ static int get_key(unsigned int usage, sc_pkcs15_object_t **result)
 		}
 
 		/* Pin already verified previously */
-		if (pin == prev_pin)
+		if (pin == prev_pin && key->user_consent == 0)
 			return 0;
 
 		pincode = get_pin(pin);
@@ -339,8 +339,25 @@ static int get_key(unsigned int usage, sc_pkcs15_object_t **result)
 			free(pincode);
 			return 5;
 		}
+		
+		/*
+		 * Do what PKCS#11 would do for keys requiring CKA_ALWAYS_AUTHENTICATE
+		 * and CKU_CONTEXT_SPECIFIC login to let driver know this verify will be followed by 
+		 * a crypto operation.  Card drivers can test for SC_AC_CONTEXT_SPECIFIC
+		 * to do any special handling. 
+		 */
+		if (key->user_consent && pin) {
+			int auth_meth_saved;
+			struct sc_pkcs15_auth_info *pinfo = (struct sc_pkcs15_auth_info *) pin->data;
+
+			auth_meth_saved = pinfo->auth_method;
+
+			pinfo->auth_method = SC_AC_CONTEXT_SPECIFIC;
+			r = sc_pkcs15_verify_pin(p15card, pin, (const u8 *)pincode, pincode ? strlen(pincode) : 0);
+			pinfo->auth_method = auth_meth_saved;
+		} else
+			r = sc_pkcs15_verify_pin(p15card, pin, (const u8 *)pincode, pincode ? strlen(pincode) : 0);
 
-		r = sc_pkcs15_verify_pin(p15card, pin, (const u8 *)pincode, pincode ? strlen(pincode) : 0);
 		free(pincode);
 		if (r) {
 			fprintf(stderr, "PIN code verification failed: %s\n", sc_strerror(r));
@@ -458,11 +475,6 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-	if (verbose > 1) {
-		ctx->debug = verbose;
-		sc_ctx_log_to_file(ctx, "stderr");
-	}
-
 	err = util_connect_card_ex(ctx, &card, opt_reader, opt_wait, 0, verbose);
 	if (err)
 		goto end;
