@@ -63,7 +63,7 @@ static int	verbose = 0;
 // Some reasonable maximums
 #define MAX_CERT		4096
 #define MAX_PRKD		256
-#define MAX_KEY			1024
+#define MAX_KEY			1500
 #define MAX_WRAPPED_KEY	(MAX_CERT + MAX_PRKD + MAX_KEY)
 
 #define SEED_LENGTH 16
@@ -72,6 +72,8 @@ enum {
 	OPT_SO_PIN = 0x100,
 	OPT_PIN,
 	OPT_RETRY,
+	OPT_BIO1,
+	OPT_BIO2,
 	OPT_PASSWORD,
 	OPT_PASSWORD_SHARES_THRESHOLD,
 	OPT_PASSWORD_SHARES_TOTAL
@@ -90,6 +92,8 @@ static const struct option options[] = {
 	{ "so-pin",					1, NULL,		OPT_SO_PIN },
 	{ "pin",					1, NULL,		OPT_PIN },
 	{ "pin-retry",				1, NULL,		OPT_RETRY },
+	{ "bio-server1",			1, NULL,		OPT_BIO1 },
+	{ "bio-server2",			1, NULL,		OPT_BIO2 },
 	{ "password",				1, NULL,		OPT_PASSWORD },
 	{ "pwd-shares-threshold",	1, NULL,		OPT_PASSWORD_SHARES_THRESHOLD },
 	{ "pwd-shares-total",		1, NULL,		OPT_PASSWORD_SHARES_TOTAL },
@@ -115,6 +119,8 @@ static const char *option_help[] = {
 	"Define security officer PIN (SO-PIN)",
 	"Define user PIN",
 	"Define user PIN retry counter",
+	"AID of biometric server for template 1 (hex)",
+	"AID of biometric server for template 2 (hex)",
 	"Define password for DKEK share",
 	"Define threshold for number of password shares required for reconstruction",
 	"Define number of password shares",
@@ -131,9 +137,6 @@ typedef struct {
 	BIGNUM * y;
 } secret_share_t;
 
-static sc_context_t *ctx = NULL;
-static sc_card_t *card = NULL;
-
 
 
 
@@ -161,12 +164,7 @@ static int generatePrime(BIGNUM *prime, const BIGNUM *s, const int bits, unsigne
 
 	do {
 		// Generate random prime
-#if OPENSSL_VERSION_NUMBER  >= 0x00908000L /* last parm is BN_GENCB which is null in our case */
 		BN_generate_prime_ex(prime, bits, 1, NULL, NULL, NULL);
-#else
-		BN_generate_prime(prime, bits, 1, NULL, NULL, NULL, NULL );
-#endif
-
 	} while ((BN_ucmp(prime, s) == -1) && (max_rounds-- > 0));	// If prime < s or not reached 1000 tries
 
 	if (max_rounds > 0)
@@ -194,10 +192,10 @@ static void calculatePolynomialValue(const BIGNUM *x, BIGNUM **polynomial, const
 	BIGNUM *exponent;
 
 	unsigned long exp;
-	BN_CTX *ctx;
+	BN_CTX *bn_ctx;
 
 	// Create context for temporary variables of OpenSSL engine
-	ctx = BN_CTX_new();
+	bn_ctx = BN_CTX_new();
 
 	temp = BN_new();
 	exponent = BN_new();
@@ -217,19 +215,19 @@ static void calculatePolynomialValue(const BIGNUM *x, BIGNUM **polynomial, const
 
 		BN_set_word(exponent, exp);
 		// temp = x^exponent mod prime
-		BN_mod_exp(temp, x, exponent, prime, ctx);
+		BN_mod_exp(temp, x, exponent, prime, bn_ctx);
 		// exponent = temp * a = a * x^exponent mod prime
-		BN_mod_mul(exponent, temp, *pp, prime, ctx);
+		BN_mod_mul(exponent, temp, *pp, prime, bn_ctx);
 		// add the temp value from exponent to y
 		BN_copy(temp, y);
-		BN_mod_add(y, temp, exponent, prime, ctx);
+		BN_mod_add(y, temp, exponent, prime, bn_ctx);
 		pp++;
 	}
 
 	BN_clear_free(temp);
 	BN_clear_free(exponent);
 
-	BN_CTX_free(ctx);
+	BN_CTX_free(bn_ctx);
 }
 
 
@@ -563,7 +561,7 @@ static void print_info(sc_card_t *card, sc_file_t *file)
 
 
 
-static int initialize(sc_card_t *card, const char *so_pin, const char *user_pin, int retry_counter, int dkek_shares, const char *label)
+static int initialize(sc_card_t *card, const char *so_pin, const char *user_pin, int retry_counter, const char *bio1, const char *bio2, int dkek_shares, const char *label)
 {
 	sc_cardctl_sc_hsm_init_param_t param;
 	size_t len;
@@ -629,8 +627,32 @@ static int initialize(sc_card_t *card, const char *so_pin, const char *user_pin,
 
 	param.user_pin_retry_counter = (u8)retry_counter;
 
+	if (bio1) {
+		param.bio1.len = sizeof(param.bio1.value);
+		r = sc_hex_to_bin(bio1, param.bio1.value, &param.bio1.len);
+		if (r < 0) {
+			fprintf(stderr, "Error decoding AID of biometric server for template 1 (%s)\n", sc_strerror(r));
+			return -1;
+		}
+	} else {
+		param.bio1.len = 0;
+	}
+	if (bio2) {
+		param.bio2.len = sizeof(param.bio2.value);
+		r = sc_hex_to_bin(bio2, param.bio2.value, &param.bio2.len);
+		if (r < 0) {
+			fprintf(stderr, "Error decoding AID of biometric server for template 2 (%s)\n", sc_strerror(r));
+			return -1;
+		}
+	} else {
+		param.bio2.len = 0;
+	}
+
 	param.options[0] = 0x00;
-	param.options[1] = 0x01;
+	param.options[1] = 0x01; /* RESET RETRY COUNTER enabled */
+	if (param.bio1.len || param.bio2.len) {
+		param.options[1] |= 0x04; /* Session-PIN enabled with clear on reset */
+	}
 
 	param.dkek_shares = (char)dkek_shares;
 	param.label = (char *)label;
@@ -759,7 +781,7 @@ static int recreate_password_from_shares(char **pwd, int *pwdlen, int num_of_pas
 static int import_dkek_share(sc_card_t *card, const char *inf, int iter, const char *password, int num_of_password_shares)
 {
 	sc_cardctl_sc_hsm_dkek_t dkekinfo;
-	EVP_CIPHER_CTX *ctx = NULL;
+	EVP_CIPHER_CTX *bn_ctx = NULL;
 	FILE *in = NULL;
 	u8 filebuff[64],key[EVP_MAX_KEY_LENGTH], iv[EVP_MAX_IV_LENGTH],outbuff[64];
 	char *pwd = NULL;
@@ -817,14 +839,14 @@ static int import_dkek_share(sc_card_t *card, const char *inf, int iter, const c
 		free(pwd);
 	}
 
-	ctx = EVP_CIPHER_CTX_new();
-	EVP_DecryptInit_ex(ctx, EVP_aes_256_cbc(), NULL, key, iv);
-	if (!EVP_DecryptUpdate(ctx, outbuff, &outlen, filebuff + 16, sizeof(filebuff) - 16)) {
+	bn_ctx = EVP_CIPHER_CTX_new();
+	EVP_DecryptInit_ex(bn_ctx, EVP_aes_256_cbc(), NULL, key, iv);
+	if (!EVP_DecryptUpdate(bn_ctx, outbuff, &outlen, filebuff + 16, sizeof(filebuff) - 16)) {
 		fprintf(stderr, "Error decrypting DKEK share. Password correct ?\n");
 		return -1;
 	}
 
-	if (!EVP_DecryptFinal_ex(ctx, outbuff + outlen, &r)) {
+	if (!EVP_DecryptFinal_ex(bn_ctx, outbuff + outlen, &r)) {
 		fprintf(stderr, "Error decrypting DKEK share. Password correct ?\n");
 		return -1;
 	}
@@ -838,7 +860,7 @@ static int import_dkek_share(sc_card_t *card, const char *inf, int iter, const c
 	r = sc_card_ctl(card, SC_CARDCTL_SC_HSM_IMPORT_DKEK_SHARE, (void *)&dkekinfo);
 
 	OPENSSL_cleanse(&dkekinfo.dkek_share, sizeof(dkekinfo.dkek_share));
-	EVP_CIPHER_CTX_free(ctx);
+	EVP_CIPHER_CTX_free(bn_ctx);
 
 	if (r == SC_ERROR_INS_NOT_SUPPORTED) {			// Not supported or not initialized for key shares
 		fprintf(stderr, "Not supported by card or card not initialized for key share usage\n");
@@ -859,7 +881,7 @@ static int print_dkek_share(sc_card_t *card, const char *inf, int iter, const ch
 	// hex output can be used in the SCSH shell with the 
 	// decrypt_keyblob.js file
 	sc_cardctl_sc_hsm_dkek_t dkekinfo;
-	EVP_CIPHER_CTX *ctx = NULL;
+	EVP_CIPHER_CTX *bn_ctx = NULL;
 	FILE *in = NULL;
 	u8 filebuff[64],key[EVP_MAX_KEY_LENGTH], iv[EVP_MAX_IV_LENGTH],outbuff[64];
 	char *pwd = NULL;
@@ -918,14 +940,14 @@ static int print_dkek_share(sc_card_t *card, const char *inf, int iter, const ch
 		free(pwd);
 	}
 
-	ctx = EVP_CIPHER_CTX_new();
-	EVP_DecryptInit_ex(ctx, EVP_aes_256_cbc(), NULL, key, iv);
-	if (!EVP_DecryptUpdate(ctx, outbuff, &outlen, filebuff + 16, sizeof(filebuff) - 16)) {
+	bn_ctx = EVP_CIPHER_CTX_new();
+	EVP_DecryptInit_ex(bn_ctx, EVP_aes_256_cbc(), NULL, key, iv);
+	if (!EVP_DecryptUpdate(bn_ctx, outbuff, &outlen, filebuff + 16, sizeof(filebuff) - 16)) {
 		fprintf(stderr, "Error decrypting DKEK share. Password correct ?\n");
 		return -1;
 	}
 
-	if (!EVP_DecryptFinal_ex(ctx, outbuff + outlen, &r)) {
+	if (!EVP_DecryptFinal_ex(bn_ctx, outbuff + outlen, &r)) {
 		fprintf(stderr, "Error decrypting DKEK share. Password correct ?\n");
 		return -1;
 	}
@@ -945,7 +967,7 @@ static int print_dkek_share(sc_card_t *card, const char *inf, int iter, const ch
 	printf("\n\n");
 
 	OPENSSL_cleanse(&dkekinfo.dkek_share, sizeof(dkekinfo.dkek_share));
-	EVP_CIPHER_CTX_free(ctx);
+	EVP_CIPHER_CTX_free(bn_ctx);
 
 	if (r == SC_ERROR_INS_NOT_SUPPORTED) {			// Not supported or not initialized for key shares
 		fprintf(stderr, "Not supported by card or card not initialized for key share usage\n");
@@ -1088,7 +1110,13 @@ static int generate_pwd_shares(sc_card_t *card, char **pwd, int *pwdlen, int pas
 	// Allocate data buffer for the generated shares
 	shares = malloc(password_shares_total * sizeof(secret_share_t));
 
-	createShares(secret, password_shares_threshold, password_shares_total, prime, shares);
+	if (!shares || 0 > createShares(secret, password_shares_threshold, password_shares_total, prime, shares)) {
+		printf("Error generating Shares. Please try again.");
+		OPENSSL_cleanse(*pwd, *pwdlen);
+		free(*pwd);
+		free(shares);
+		return -1;
+	}
 
 	sp = shares;
 	for (i = 0; i < password_shares_total; i++) {
@@ -1130,7 +1158,7 @@ static int generate_pwd_shares(sc_card_t *card, char **pwd, int *pwdlen, int pas
 
 static int create_dkek_share(sc_card_t *card, const char *outf, int iter, const char *password, int password_shares_threshold, int password_shares_total)
 {
-	EVP_CIPHER_CTX *ctx = NULL;
+	EVP_CIPHER_CTX *c_ctx = NULL;
 	FILE *out = NULL;
 	u8 filebuff[64], key[EVP_MAX_KEY_LENGTH], iv[EVP_MAX_IV_LENGTH];
 	u8 dkek_share[32];
@@ -1181,14 +1209,14 @@ static int create_dkek_share(sc_card_t *card, const char *outf, int iter, const
 		return -1;
 	}
 
-	ctx = EVP_CIPHER_CTX_new();
-	EVP_EncryptInit_ex(ctx, EVP_aes_256_cbc(), NULL, key, iv);
-	if (!EVP_EncryptUpdate(ctx, filebuff + 16, &outlen, dkek_share, sizeof(dkek_share))) {
+	c_ctx = EVP_CIPHER_CTX_new();
+	EVP_EncryptInit_ex(c_ctx, EVP_aes_256_cbc(), NULL, key, iv);
+	if (!EVP_EncryptUpdate(c_ctx, filebuff + 16, &outlen, dkek_share, sizeof(dkek_share))) {
 		fprintf(stderr, "Error encrypting DKEK share\n");
 		return -1;
 	}
 
-	if (!EVP_EncryptFinal_ex(ctx, filebuff + 16 + outlen, &r)) {
+	if (!EVP_EncryptFinal_ex(c_ctx, filebuff + 16 + outlen, &r)) {
 		fprintf(stderr, "Error encrypting DKEK share\n");
 		return -1;
 	}
@@ -1209,7 +1237,7 @@ static int create_dkek_share(sc_card_t *card, const char *outf, int iter, const
 	fclose(out);
 
 	OPENSSL_cleanse(filebuff, sizeof(filebuff));
-	EVP_CIPHER_CTX_free(ctx);
+	EVP_CIPHER_CTX_free(c_ctx);
 
 	printf("DKEK share created and saved to %s\n", outf);
 	return 0;
@@ -1244,40 +1272,25 @@ static size_t determineLength(const u8 *tlv, size_t buflen)
  */
 static int wrap_with_tag(u8 tag, u8 *indata, size_t inlen, u8 **outdata, size_t *outlen)
 {
-	int nlc = 0;
-	u8 *ptr;
+	int r = sc_asn1_put_tag(tag, indata, inlen, NULL, 0, NULL);
+	if (r < 0)
+		return r;
+	if (r == 0)
+		return SC_ERROR_INVALID_ASN1_OBJECT;
 
-	if (inlen > 127) {
-		do	{
-			nlc++;
-		} while (inlen >= (unsigned)(1 << (nlc << 3)));
-	}
-
-	*outlen = 2 + nlc + inlen;
-	ptr = malloc(*outlen);
+	u8 *ptr = calloc(r, sizeof *ptr);
 	if (ptr == NULL) {
 		return SC_ERROR_OUT_OF_MEMORY;
 	}
-
 	*outdata = ptr;
-	*ptr++ = tag;
+	*outlen = r;
 
-	if (nlc) {
-		*ptr++ = 0x80 | nlc;
-		while (nlc--) {
-			*ptr++ = (inlen >> (nlc << 3)) & 0xFF;
-		}
-	} else {
-		*ptr++ = inlen & 0x7F;
-	}
-
-	memcpy(ptr, indata, inlen);
-	return SC_SUCCESS;
+	return sc_asn1_put_tag(tag, indata, inlen, *outdata, *outlen, NULL);
 }
 
 
 
-static int wrap_key(sc_card_t *card, int keyid, const char *outf, const char *pin)
+static int wrap_key(sc_context_t *ctx, sc_card_t *card, int keyid, const char *outf, const char *pin)
 {
 	sc_cardctl_sc_hsm_wrapped_key_t wrapped_key;
 	struct sc_pin_cmd_data data;
@@ -1508,13 +1521,13 @@ static int unwrap_key(sc_card_t *card, int keyid, const char *inf, const char *p
 		return -1;
 	}
 
-	if ((keybloblen = fread(keyblob, 1, sizeof(keyblob), in)) < 0) {
+	keybloblen = fread(keyblob, 1, sizeof(keyblob), in);
+	fclose(in);
+	if (keybloblen < 0) {
 		perror(inf);
 		return -1;
 	}
 
-	fclose(in);
-
 	ptr = keyblob;
 	if ((sc_asn1_read_tag(&ptr, keybloblen, &cla, &tag, &len) != SC_SUCCESS)
 		   	|| ((cla & SC_ASN1_TAG_CONSTRUCTED) != SC_ASN1_TAG_CONSTRUCTED)
@@ -1524,7 +1537,7 @@ static int unwrap_key(sc_card_t *card, int keyid, const char *inf, const char *p
 	}
 
 	if ((sc_asn1_read_tag(&ptr, len, &cla, &tag, &olen) != SC_SUCCESS)
-		   	|| ((cla & SC_ASN1_TAG_CONSTRUCTED) != SC_ASN1_TAG_CONSTRUCTED)
+		   	|| ((cla & SC_ASN1_TAG_CONSTRUCTED) == SC_ASN1_TAG_CONSTRUCTED)
 		   	|| (tag != SC_ASN1_TAG_OCTET_STRING) ){
 		fprintf(stderr, "Invalid wrapped key format (Key binary).\n");
 		return -1;
@@ -1671,6 +1684,8 @@ int main(int argc, char *argv[])
 	const char *opt_pin = NULL;
 	const char *opt_filename = NULL;
 	const char *opt_password = NULL;
+	const char *opt_bio1 = NULL;
+	const char *opt_bio2 = NULL;
 	int opt_retry_counter = 3;
 	int opt_dkek_shares = -1;
 	int opt_key_reference = -1;
@@ -1679,9 +1694,8 @@ int main(int argc, char *argv[])
 	int opt_force = 0;
 	int opt_iter = 10000000;
 	sc_context_param_t ctx_param;
-
-	setbuf(stderr, NULL);
-	setbuf(stdout, NULL);
+	sc_context_t *ctx = NULL;
+	sc_card_t *card = NULL;
 
 	while (1) {
 		c = getopt_long(argc, argv, "XC:I:P:W:U:s:i:fr:wv", options, &long_optind);
@@ -1731,6 +1745,12 @@ int main(int argc, char *argv[])
 		case OPT_RETRY:
 			opt_retry_counter = atol(optarg);
 			break;
+		case OPT_BIO1:
+			opt_bio1 = optarg;
+			break;
+		case OPT_BIO2:
+			opt_bio2 = optarg;
+			break;
 		case OPT_PASSWORD_SHARES_THRESHOLD:
 			opt_password_shares_threshold = atol(optarg);
 			break;
@@ -1761,7 +1781,7 @@ int main(int argc, char *argv[])
 		}
 	}
 
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L && !(defined LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L || (defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER >= 0x20700000L)
 	OPENSSL_init_crypto(OPENSSL_INIT_LOAD_CRYPTO_STRINGS
 		| OPENSSL_INIT_ADD_ALL_CIPHERS
 		| OPENSSL_INIT_ADD_ALL_DIGESTS,
@@ -1781,12 +1801,6 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	/* Only change if not in opensc.conf */
-	if (verbose > 1 && ctx->debug == 0) {
-		ctx->debug = verbose;
-		sc_ctx_log_to_file(ctx, "stderr");
-	}
-
 	r = util_connect_card_ex(ctx, &card, opt_reader, opt_wait, 0, verbose);
 	if (r != SC_SUCCESS) {
 		if (r < 0) {
@@ -1803,7 +1817,7 @@ int main(int argc, char *argv[])
 		goto fail;
 	}
 
-	if (do_initialize && initialize(card, opt_so_pin, opt_pin, opt_retry_counter, opt_dkek_shares, opt_label))
+	if (do_initialize && initialize(card, opt_so_pin, opt_pin, opt_retry_counter, opt_bio1, opt_bio2, opt_dkek_shares, opt_label))
 		goto fail;
 
 	if (do_create_dkek_share && create_dkek_share(card, opt_filename, opt_iter, opt_password, opt_password_shares_threshold, opt_password_shares_total))
@@ -1815,7 +1829,7 @@ int main(int argc, char *argv[])
 	if (do_print_dkek_share && print_dkek_share(card, opt_filename, opt_iter, opt_password, opt_password_shares_total))
 		goto fail;
 
-	if (do_wrap_key && wrap_key(card, opt_key_reference, opt_filename, opt_pin))
+	if (do_wrap_key && wrap_key(ctx, card, opt_key_reference, opt_filename, opt_pin))
 		goto fail;
 
 	if (do_unwrap_key && unwrap_key(card, opt_key_reference, opt_filename, opt_pin, opt_force))
@@ -1830,11 +1844,8 @@ int main(int argc, char *argv[])
 fail:
 	err = 1;
 end:
-	if (card) {
-		sc_disconnect_card(card);
-	}
-	if (ctx)
-		sc_release_context(ctx);
+	sc_disconnect_card(card);
+	sc_release_context(ctx);
 
 	ERR_print_errors_fp(stderr);
 	return err;
