@@ -292,13 +292,13 @@ static int unlock_pin(sc_card_t *card,
 	return 0;
 }
 
-static int cert2der(X509 *cert, u8 **value)
+static int cert2der(X509 *in_cert, u8 **value)
 {
 	int len;
 	u8 *p;
-	len = i2d_X509(cert, NULL);
+	len = i2d_X509(in_cert, NULL);
 	p = *value = malloc(len);
-	i2d_X509(cert, &p);
+	i2d_X509(in_cert, &p);
 	return len;
 }
 
@@ -443,11 +443,6 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-	if (verbose > 1) {
-		ctx->debug = verbose;
-		sc_ctx_log_to_file(ctx, "stderr");
-	}
-
 	if (opt_driver != NULL)
 	{
 		r = sc_set_card_driver(ctx, opt_driver);
@@ -589,7 +584,6 @@ int main(int argc, char *argv[])
 
 		printf("Generate key of length %d.\n", keylen);
 
-#if OPENSSL_VERSION_NUMBER>=0x00908000L
 		rsa = RSA_new();
 		bn = BN_new();
 		mem = BIO_new(BIO_s_mem());
@@ -602,18 +596,6 @@ int main(int argc, char *argv[])
 
 		if(!BN_set_word(bn, RSA_F4) ||
 			!RSA_generate_key_ex(rsa, keylen, bn, NULL))
-#else
-		rsa = RSA_generate_key(keylen, RSA_F4, NULL, NULL);
-		mem = BIO_new(BIO_s_mem());
-
-		if(mem == NULL)
-		{
-			printf("Not enough memory.\n");
-			goto out;
-		}
-
-		if (!rsa)
-#endif
 		{
 			printf("RSA_generate_key_ex return %ld\n", ERR_get_error());
 			goto out;
